@@ -13,18 +13,22 @@
 // limitations under the License.
 
 #![deny(clippy::all)]
+use std::cell::RefCell;
+use std::cell::RefMut;
 use std::collections::BTreeMap;
 use std::collections::BTreeSet;
 use std::ffi::OsStr;
+use std::fs::File;
 use std::io::BufRead;
 use std::io::BufReader;
-use std::io::ErrorKind;
+use std::os::fd::AsRawFd;
+use std::os::fd::BorrowedFd;
 use std::path::Path;
 use std::path::PathBuf;
 use std::str::FromStr;
 
-use nix::sys::statfs::fstatfs;
 use nix::sys::statfs::CGROUP2_SUPER_MAGIC;
+use nix::sys::statfs::fstatfs;
 use openat::AsPath;
 use openat::Dir;
 use openat::SimpleType;
@@ -36,6 +40,8 @@ pub use types::*;
 #[cfg(test)]
 mod test;
 
+use common::util;
+
 pub const DEFAULT_CG_ROOT: &str = "/sys/fs/cgroup";
 
 #[derive(Error, Debug)]
@@ -57,6 +63,7 @@ pub type Result<T> = std::result::Result<T, Error>;
 pub struct CgroupReader {
     relative_path: PathBuf,
     dir: Dir,
+    buffer: RefCell<Vec<u8>>,
 }
 
 fn parse_integer_or_max(s: &str) -> std::result::Result<i64, String> {
@@ -64,21 +71,21 @@ fn parse_integer_or_max(s: &str) -> std::result::Result<i64, String> {
         return Ok(-1);
     }
     s.parse::<u64>()
-        .map_err(|e| format!("Invalid integer or max value {}", e))
+        .map_err(|e| format!("Invalid integer or max value {e}"))
         .map(|v| v as i64)
 }
 
 fn parse_node_range(s: &str) -> std::result::Result<BTreeSet<u32>, String> {
     fn parse_node(s: &str) -> std::result::Result<u32, String> {
         s.parse()
-            .map_err(|_| format!("id must be non-negative int: {}", s))
+            .map_err(|_| format!("id must be non-negative int: {s}"))
     }
     match s.split_once('-') {
         Some((first, last)) => {
             let first = parse_node(first)?;
             let last = parse_node(last)?;
             if first > last {
-                return Err(format!("Invalid range: {}", s));
+                return Err(format!("Invalid range: {s}"));
             }
             Ok((first..(last + 1)).collect())
         }
@@ -105,9 +112,9 @@ fn fmt_nodes(f: &mut std::fmt::Formatter<'_>, nodes: &BTreeSet<u32>) -> std::fmt
         range_end: u32,
     ) -> std::fmt::Result {
         if range_start == range_end {
-            write!(f, "{}", range_start)
+            write!(f, "{range_start}")
         } else {
-            write!(f, "{}-{}", range_start, range_end)
+            write!(f, "{range_start}-{range_end}")
         }
     }
 
@@ -134,13 +141,13 @@ impl FromStr for CpuMax {
     fn from_str(s: &str) -> std::result::Result<Self, String> {
         let nums: Vec<&str> = s.split(' ').collect();
         if nums.len() != 2 {
-            return Err(format!("Invalid cpu.max {}", s));
+            return Err(format!("Invalid cpu.max {s}"));
         }
         Ok(CpuMax {
             max_usec: parse_integer_or_max(nums[0])?,
             period_usec: nums[1]
                 .parse()
-                .map_err(|e| format!("Invalid non-negative integer {}", e))?,
+                .map_err(|e| format!("Invalid non-negative integer {e}"))?,
         })
     }
 }
@@ -244,12 +251,15 @@ impl CgroupReader {
 
         // Check that it's a cgroup2 fs
         if validate {
-            let statfs = match fstatfs(&dir) {
+            // SAFETY: Fix when https://github.com/nix-rust/nix/issues/2546 is
+            let dir = unsafe { BorrowedFd::borrow_raw(dir.as_raw_fd()) };
+
+            let statfs = match fstatfs(dir) {
                 Ok(s) => s,
                 Err(e) => {
                     return Err(Error::IoError(
                         path,
-                        std::io::Error::new(ErrorKind::Other, format!("Failed to fstatfs: {}", e)),
+                        std::io::Error::other(format!("Failed to fstatfs: {e}")),
                     ));
                 }
             };
@@ -259,7 +269,11 @@ impl CgroupReader {
             }
         }
 
-        Ok(CgroupReader { relative_path, dir })
+        Ok(CgroupReader {
+            relative_path,
+            dir,
+            buffer: RefCell::new(Vec::new()),
+        })
     }
 
     pub fn root() -> Result<CgroupReader> {
@@ -272,6 +286,15 @@ impl CgroupReader {
         &self.relative_path
     }
 
+    fn read_file_to_str(
+        &self,
+        file_name: impl AsRef<Path>,
+        file: &File,
+    ) -> Result<RefMut<'_, str>> {
+        util::read_kern_file_to_internal_buffer(&self.buffer, file)
+            .map_err(|e| self.io_error(file_name.as_ref(), e))
+    }
+
     pub fn read_inode_number(&self) -> Result<u64> {
         let meta = self
             .dir
@@ -287,14 +310,10 @@ impl CgroupReader {
             .dir
             .open_file(file_name)
             .map_err(|e| self.io_error(file_name, e))?;
-        let buf_reader = BufReader::new(file);
-        let line = buf_reader
-            .lines()
-            .next()
-            .unwrap_or_else(|| Ok("".to_owned()));
-        let line = line.map_err(|e| self.io_error(file_name, e))?;
+        let content = self.read_file_to_str(file_name, &file)?;
+        let line = content.lines().next().unwrap_or("");
         line.parse::<T>()
-            .map_err(move |_| self.unexpected_line(file_name, line))
+            .map_err(move |_| self.unexpected_line(file_name, line.to_string()))
     }
 
     /// Read a value from a file that has a single line. If the file is empty,
@@ -304,12 +323,11 @@ impl CgroupReader {
             .dir
             .open_file(file_name)
             .map_err(|e| self.io_error(file_name, e))?;
-        let buf_reader = BufReader::new(file);
-        if let Some(line) = buf_reader.lines().next() {
-            let line = line.map_err(|e| self.io_error(file_name, e))?;
+        let content = self.read_file_to_str(file_name, &file)?;
+        if let Some(line) = content.lines().next() {
             return line
                 .parse::<T>()
-                .map_err(move |_| self.unexpected_line(file_name, line));
+                .map_err(move |_| self.unexpected_line(file_name, line.to_string()));
         }
         Err(self.invalid_file_format(file_name))
     }
@@ -390,6 +408,16 @@ impl CgroupReader {
         self.read_singleline_integer_or_max_stat_file("memory.zswap.max")
     }
 
+    /// Read memory.zswap.writeback - returning 0/1 if zswap writeback is disabled/enabled
+    pub fn read_memory_zswap_writeback(&self) -> Result<u32> {
+        self.read_singleline_file("memory.zswap.writeback")
+    }
+
+    /// Read memory.oom.group - returning 0/1 if oom group is disabled/enabled
+    pub fn read_memory_oom_group(&self) -> Result<u32> {
+        self.read_singleline_file("memory.oom.group")
+    }
+
     /// Read memory.current - returning current cgroup memory
     /// consumption in bytes
     pub fn read_memory_current(&self) -> Result<u64> {
@@ -428,6 +456,10 @@ impl CgroupReader {
         MemoryEvents::read(self)
     }
 
+    pub fn read_memory_events_local(&self) -> Result<MemoryEventsLocal> {
+        MemoryEventsLocal::read(self)
+    }
+
     pub fn read_cgroup_stat(&self) -> Result<CgroupStat> {
         CgroupStat::read(self)
     }
@@ -501,30 +533,29 @@ impl CgroupReader {
             .dir
             .open_file(file_name)
             .map_err(|e| self.io_error(file_name, e))?;
-        let buf_reader = BufReader::new(file);
-        for line in buf_reader.lines() {
-            let line = line.map_err(|e| self.io_error(file_name, e))?;
-            let items = line.split_whitespace().collect::<Vec<_>>();
+        let content = self.read_file_to_str(file_name, &file)?;
+        for line in content.lines() {
+            let items = line.split_ascii_whitespace().collect::<Vec<_>>();
             // Need to have at least the field name + at least one N0=val item
             if items.len() < 2 {
-                return Err(self.unexpected_line(file_name, line));
+                return Err(self.unexpected_line(file_name, line.to_string()));
             }
             let field = items[0];
             for item in items.iter().skip(1) {
                 let kv = item.split('=').collect::<Vec<_>>();
                 if kv.len() != 2 || kv[0].len() < 2 || !kv[0].starts_with('N') {
-                    return Err(self.unexpected_line(file_name, line));
+                    return Err(self.unexpected_line(file_name, line.to_string()));
                 }
                 let mut kchars = kv[0].chars();
                 kchars.next();
                 let numa_node = kchars
                     .as_str()
                     .parse::<u32>()
-                    .map_err(|_| self.unexpected_line(file_name, line.clone()))?;
+                    .map_err(|_| self.unexpected_line(file_name, line.to_string()))?;
                 parse_and_set_fields!(
                     s.entry(numa_node).or_default();
                     field;
-                    kv[1].parse().map_err(|_| self.unexpected_line(file_name, line.clone()))?;
+                    kv[1].parse().map_err(|_| self.unexpected_line(file_name, line.to_string()))?;
                     [
                         anon,
                         file,
@@ -584,7 +615,11 @@ impl CgroupReader {
                     };
                     let mut relative_path = self.relative_path.clone();
                     relative_path.push(entry.file_name());
-                    Some(CgroupReader { relative_path, dir })
+                    Some(CgroupReader {
+                        relative_path,
+                        dir,
+                        buffer: RefCell::new(Vec::new()),
+                    })
                 }
                 _ => None,
             }))
@@ -634,13 +669,14 @@ macro_rules! key_values_format {
                 let buf_reader = BufReader::new(file);
                 for line in buf_reader.lines() {
                     let line = line.map_err(|e| r.io_error(file_name, e))?;
-                    let items = line.split_whitespace().collect::<Vec<_>>();
-                    if items.len() != 2 {
+                    let mut items = line.split_ascii_whitespace();
+                    let key = items.next().ok_or_else(|| r.unexpected_line(file_name, line.clone()))?;
+                    let val_str = items.next().ok_or_else(|| r.unexpected_line(file_name, line.clone()))?;
+                    if items.next().is_some() {
                         return Err(r.unexpected_line(file_name, line));
                     }
-                    let key = items[0];
-                    let val = items[1].parse::<_>().map_err(|_| r.unexpected_line(file_name, line.clone()))?;
-                    match key.as_ref() {
+                    let val = val_str.parse::<_>().map_err(|_| r.unexpected_line(file_name, line.clone()))?;
+                    match key {
                         $(stringify!($field) => s.$field = Some(val),)*
                         _ => (),
                     };
@@ -677,6 +713,7 @@ key_values_format!(MemoryStat; memory.stat; [
     file_mapped,
     file_dirty,
     file_writeback,
+    file_thp,
     anon_thp,
     inactive_anon,
     active_anon,
@@ -713,6 +750,14 @@ key_values_format!(MemoryEvents; memory.events; [
     oom_kill
 ]);
 
+key_values_format!(MemoryEventsLocal; memory.events.local; [
+    low,
+    high,
+    max,
+    oom,
+    oom_kill
+]);
+
 key_values_format!(CgroupStat; cgroup.stat; [nr_descendants, nr_dying_descendants]);
 
 // Trait to add a read() method for `<string> key=value` formatted files
@@ -746,31 +791,36 @@ macro_rules! name_key_equal_value_format {
                         }
                         r.io_error(file_name.clone(), e)
                     })?;
-                    let items = line.split_whitespace().collect::<Vec<_>>();
                     // as an example, io.stat looks like:
+                    // 7:0
                     // 253:0 rbytes=531745786880 wbytes=1623798909952 ...
+                    let mut items = line.split_terminator(' ').peekable();
+                    let Some(first_item) = items.next() else {
+                      continue
+                    };
+                    // Skip lines with no kv pairs
+                    if items.peek().is_none() {
+                      continue
+                    }
                     let mut s = $struct::default();
-                    for item in items.iter().skip(1) {
-                        let kv = item.split("=").collect::<Vec<_>>();
-                        if kv.len() != 2 {
-                            return Err(r.invalid_file_format(file_name));
-                        }
-                        // Certain keys such as cost.usage can not be struct fields so must use cost_usage
-                        let key = kv[0].replace(".", "_");
+                    for item in items {
+                        let (key, value) = item.split_once('=').ok_or_else(|| r.invalid_file_format(file_name.clone()))?;
+                        // Certain keys such as cost.usage cannot be struct fields so must use cost_usage
+                        let key = key.replace(".", "_");
                         parse_and_set_fields!(
                             s;
                             key.as_ref();
-                            kv[1].parse().map_err(|_| r.unexpected_line(file_name.clone(), line.clone()))?;
+                            value.parse().map_err(|_| r.unexpected_line(file_name.clone(), line.clone()))?;
                             [ $($field,)* ]
                         )
-                    };
-                    if s == $struct::default() {
-                        return Err(r.invalid_file_format(file_name))
                     }
-                    map.insert(items[0].to_string(), s);
+                    if s == $struct::default() {
+                        return Err(r.invalid_file_format(file_name));
+                    }
+                    map.insert(first_item.to_string(), s);
                 }
                 if !$allows_empty.0 && map.is_empty() {
-                     Err(r.invalid_file_format(file_name))
+                    Err(r.invalid_file_format(file_name))
                 } else {
                     Ok(map)
                 }
