@@ -146,6 +146,13 @@ singleline_integer_or_max_test!(read_memory_high, "memory.high");
 singleline_integer_or_max_test!(read_memory_max, "memory.max");
 singleline_integer_or_max_test!(read_memory_swap_max, "memory.swap.max");
 singleline_integer_or_max_test!(read_memory_zswap_max, "memory.zswap.max");
+test_success!(
+    read_memory_zswap_writeback,
+    "memory.zswap.writeback",
+    b"1\n",
+    1
+);
+test_success!(read_memory_oom_group, "memory.oom.group", b"1\n", 1);
 
 test_success!(read_pids_current, "pids.current", b"10000\n", 10000);
 test_success!(read_cpu_weight, "cpu.weight", b"10000\n", 10000);
@@ -287,7 +294,7 @@ fn test_memory_current_parse_failure() {
         .expect_err("Did not fail to read memory.current");
     match err {
         Error::UnexpectedLine(_, _) => {}
-        _ => panic!("Got unexpected error type {}", err),
+        _ => panic!("Got unexpected error type {err}"),
     }
 }
 
@@ -303,7 +310,7 @@ fn test_memory_current_invalid_format() {
         .expect_err("Did not fail to read memory.current");
     match err {
         Error::InvalidFileFormat(_) => {}
-        _ => panic!("Got unexpected error type {}", err),
+        _ => panic!("Got unexpected error type {err}"),
     }
 }
 
@@ -369,7 +376,7 @@ fn test_memory_stat_parse_failure() {
         .expect_err("Did not fail to read memory.stat");
     match err {
         Error::UnexpectedLine(_, _) => {}
-        _ => panic!("Got unexpected error type {}", err),
+        _ => panic!("Got unexpected error type {err}"),
     }
 }
 
@@ -384,7 +391,7 @@ fn test_memory_stat_invalid_format() {
         .expect_err("Did not fail to read memory.stat");
     match err {
         Error::InvalidFileFormat(_) => {}
-        _ => panic!("Got unexpected error type: {}", err),
+        _ => panic!("Got unexpected error type: {err}"),
     }
 }
 
@@ -414,7 +421,7 @@ fn test_cpu_stat_parse_failure() {
         .expect_err("Did not fail to read cpu.stat");
     match err {
         Error::UnexpectedLine(_, _) => {}
-        _ => panic!("Got unexpected error type {}", err),
+        _ => panic!("Got unexpected error type {err}"),
     }
 }
 
@@ -429,14 +436,14 @@ fn test_cpu_stat_invalid_format() {
         .expect_err("Did not fail to read cpu.stat");
     match err {
         Error::InvalidFileFormat(_) => {}
-        _ => panic!("Got unexpected error type: {}", err),
+        _ => panic!("Got unexpected error type: {err}"),
     }
 }
 
 #[test]
 fn test_io_stat_success() {
     let cgroup = TestCgroup::new();
-    cgroup.create_file_with_content("io.stat", b"253:0 rbytes=531 wbytes=162379 rios=61 wios=81 dbytes=0 dios=0\n13:0 rbytes=135 wbytes=162379 rios=61 wios=81 dbytes=0 dios=0 cost.usage=25 cost.wait=38 cost.indebt=64 cost.indelay=0\n");
+    cgroup.create_file_with_content("io.stat", b"7:0 \n253:0 rbytes=531 wbytes=162379 rios=61 wios=81 dbytes=0 dios=0\n13:0 rbytes=135 wbytes=162379 rios=61 wios=81 dbytes=0 dios=0 cost.usage=25 cost.wait=38 cost.indebt=64 cost.indelay=0\n");
 
     let cgroup_reader = cgroup.get_reader();
     let val = cgroup_reader
@@ -472,7 +479,7 @@ fn test_io_stat_parse_failure() {
         .expect_err("Did not fail to read io.stat");
     match err {
         Error::InvalidFileFormat(_) => {}
-        _ => panic!("Got unexpected error type {}", err),
+        _ => panic!("Got unexpected error type {err}"),
     }
 }
 
@@ -544,7 +551,7 @@ fn test_cpu_pressure_empty_file() {
         .expect_err("Did not fail to read cpu.pressure");
     match err {
         Error::InvalidFileFormat(_) => {}
-        _ => panic!("Got unexpected error type: {}", err),
+        _ => panic!("Got unexpected error type: {err}"),
     }
 }
 
@@ -578,7 +585,7 @@ fn test_io_pressure_empty_file() {
         .expect_err("Did not fail to read io.pressure");
     match err {
         Error::InvalidFileFormat(_) => {}
-        _ => panic!("Got unexpected error type: {}", err),
+        _ => panic!("Got unexpected error type: {err}"),
     }
 }
 
@@ -612,7 +619,7 @@ fn test_memory_pressure_empty_file() {
         .expect_err("Did not fail to read memory.pressure");
     match err {
         Error::InvalidFileFormat(_) => {}
-        _ => panic!("Got unexpected error type: {}", err),
+        _ => panic!("Got unexpected error type: {err}"),
     }
 }
 
@@ -746,7 +753,7 @@ fn test_cgroup_stat_parse_failure() {
         .expect_err("Failed to read cgroup.stat");
     match err {
         Error::UnexpectedLine(_, _) => {}
-        _ => panic!("Got unexpected error type {}", err),
+        _ => panic!("Got unexpected error type {err}"),
     }
 }
 
@@ -761,7 +768,7 @@ fn test_cgroup_stat_invalid_format() {
         .expect_err("Did not fail to read cgroup.stat");
     match err {
         Error::InvalidFileFormat(_) => {}
-        _ => panic!("Got unexpected error type: {}", err),
+        _ => panic!("Got unexpected error type: {err}"),
     }
 }
 
@@ -933,7 +940,7 @@ fn test_memory_numa_stat_parse_failure() {
         .expect_err("Did not fail to read memory.numa.stat");
     match err {
         Error::UnexpectedLine(_, _) => {}
-        _ => panic!("Got unexpected error type {}", err),
+        _ => panic!("Got unexpected error type {err}"),
     }
 }
 
@@ -948,6 +955,6 @@ fn test_memory_numa_stat_invalid_format() {
         .expect_err("Did not fail to read memory.numa_stat");
     match err {
         Error::InvalidFileFormat(_) => {}
-        _ => panic!("Got unexpected error type: {}", err),
+        _ => panic!("Got unexpected error type: {err}"),
     }
 }
