@@ -12,11 +12,12 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use std::fmt::Display;
 use std::str::FromStr;
 
-use anyhow::bail;
 use anyhow::Error;
 use anyhow::Result;
+use anyhow::bail;
 use clap::Parser;
 use model::BtrfsModelFieldId;
 use model::FieldId;
@@ -26,6 +27,7 @@ use model::SingleDiskModelFieldId;
 use model::SingleNetModelFieldId;
 use model::SingleProcessModelFieldId;
 use model::SingleQueueModelFieldId;
+use model::SingleTcModelFieldId;
 use model::SystemModelFieldId;
 use once_cell::sync::Lazy;
 use regex::Regex;
@@ -85,12 +87,12 @@ impl<F: FieldId + FromStr, A: AggField<F> + FromStr> FromStr for DumpOptionField
 }
 
 /// Used for generating help string that lists all supported fields.
-impl<F: FieldId + ToString, A: AggField<F> + ToString> ToString for DumpOptionField<F, A> {
-    fn to_string(&self) -> String {
+impl<F: FieldId + Display, A: AggField<F> + Display> Display for DumpOptionField<F, A> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            Self::Unit(DumpField::Common(common)) => common.to_string(),
-            Self::Unit(DumpField::FieldId(field_id)) => field_id.to_string(),
-            Self::Agg(agg) => agg.to_string(),
+            Self::Unit(DumpField::Common(common)) => write!(f, "{}", common),
+            Self::Unit(DumpField::FieldId(field_id)) => write!(f, "{}", field_id),
+            Self::Agg(agg) => write!(f, "{}", agg),
         }
     }
 }
@@ -1036,6 +1038,81 @@ $ below dump ethtool-queue -b "08:30:00" -e "08:30:30" -O json
     )
 });
 
+/// Represents the fields of the tc model.
+#[derive(
+    Clone,
+    Debug,
+    PartialEq,
+    below_derive::EnumFromStr,
+    below_derive::EnumToString
+)]
+pub enum TcAggField {
+    Stats,
+    XStats,
+    QDisc,
+}
+
+impl AggField<SingleTcModelFieldId> for TcAggField {
+    fn expand(&self, _detail: bool) -> Vec<SingleTcModelFieldId> {
+        use model::SingleTcModelFieldId as FieldId;
+
+        match self {
+            Self::Stats => vec![
+                FieldId::Interface,
+                FieldId::Kind,
+                FieldId::Qlen,
+                FieldId::Bps,
+                FieldId::Pps,
+                FieldId::BytesPerSec,
+                FieldId::PacketsPerSec,
+                FieldId::BacklogPerSec,
+                FieldId::DropsPerSec,
+                FieldId::RequeuesPerSec,
+                FieldId::OverlimitsPerSec,
+            ],
+            Self::XStats => enum_iterator::all::<model::XStatsModelFieldId>()
+                .map(FieldId::Xstats)
+                .collect::<Vec<_>>(),
+            Self::QDisc => enum_iterator::all::<model::QDiscModelFieldId>()
+                .map(FieldId::Qdisc)
+                .collect::<Vec<_>>(),
+        }
+    }
+}
+
+pub type TcOptionField = DumpOptionField<SingleTcModelFieldId, TcAggField>;
+
+pub static DEFAULT_TC_FIELDS: &[TcOptionField] = &[
+    DumpOptionField::Unit(DumpField::Common(CommonField::Datetime)),
+    DumpOptionField::Agg(TcAggField::Stats),
+    DumpOptionField::Agg(TcAggField::XStats),
+    DumpOptionField::Agg(TcAggField::QDisc),
+    DumpOptionField::Unit(DumpField::Common(CommonField::Timestamp)),
+];
+
+const TC_ABOUT: &str = "Dump the tc related stats with qdiscs";
+
+/// Generated about message for tc (traffic control) dump so supported fields are up-to-date.
+static TC_LONG_ABOUT: Lazy<String> = Lazy::new(|| {
+    format!(
+        r#"{about}
+********************** Available fields **********************
+{common_fields}, {tc_fields}.
+********************** Aggregated fields **********************
+* --detail: no effect.
+* --default: includes [{default_fields}].
+* --everything: includes everything (equivalent to --default --detail).
+********************** Example Commands **********************
+Example:
+$ below dump tc -b "08:30:00" -e "08:30:30" -O json
+"#,
+        about = TC_ABOUT,
+        common_fields = join(enum_iterator::all::<CommonField>()),
+        tc_fields = join(enum_iterator::all::<SingleTcModelFieldId>()),
+        default_fields = join(DEFAULT_TC_FIELDS.to_owned()),
+    )
+});
+
 make_option! (OutputFormat {
     "raw": Raw,
     "csv": Csv,
@@ -1219,4 +1296,15 @@ pub enum DumpCommand {
         #[clap(long, short, conflicts_with("fields"))]
         pattern: Option<String>,
     },
+    #[clap(about = TC_ABOUT, long_about = TC_LONG_ABOUT.as_str())]
+    Tc {
+        /// Select which fields to display and in what order.
+        #[clap(short, long)]
+        fields: Option<Vec<TcOptionField>>,
+        #[clap(flatten)]
+        opts: GeneralOpt,
+        /// Saved pattern in the dumprc file under [tc] section.
+        #[clap(long, short, conflicts_with("fields"))]
+        pattern: Option<String>,
+    },
 }
