@@ -12,6 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#![deny(clippy::all)]
+#![allow(clippy::too_many_arguments)]
+
 use std::fs;
 use std::fs::File;
 use std::io;
@@ -21,10 +24,11 @@ use std::str::FromStr;
 use std::sync::mpsc::Receiver;
 use std::time::SystemTime;
 
-use anyhow::bail;
 use anyhow::Context;
 use anyhow::Error;
 use anyhow::Result;
+use anyhow::anyhow;
+use anyhow::bail;
 use common::cliutil;
 use common::util::get_belowrc_dump_section_key;
 use common::util::get_belowrc_filename;
@@ -32,12 +36,12 @@ use common::util::timestamp_to_datetime;
 use model::Field;
 use model::FieldId;
 use model::Queriable;
-use serde_json::json;
 use serde_json::Value;
-use store::advance::new_advance_local;
-use store::advance::new_advance_remote;
+use serde_json::json;
 use store::Advance;
 use store::Direction;
+use store::advance::new_advance_local;
+use store::advance::new_advance_remote;
 use tar::Archive;
 use tempfile::TempDir;
 use toml::value::Value as TValue;
@@ -52,20 +56,21 @@ pub mod network;
 pub mod print;
 pub mod process;
 pub mod system;
+pub mod tc;
 pub mod tmain;
 pub mod transport;
 
 #[cfg(test)]
 mod test;
 
-use command::expand_fields;
 pub use command::DumpCommand;
 use command::GeneralOpt;
 use command::OutputFormat;
+use command::expand_fields;
 use render::HasRenderConfigForDump;
-use tmain::dump_timeseries;
 use tmain::Dumper;
 use tmain::IterExecResult;
+use tmain::dump_timeseries;
 
 /// Fields available to all commands. Each enum represents some semantics and
 /// knows how to extract relevant data from a CommonFieldContext.
@@ -117,6 +122,7 @@ pub type IfaceField = DumpField<model::SingleNetModelFieldId>;
 // Essentially the same as NetworkField
 pub type TransportField = DumpField<model::NetworkModelFieldId>;
 pub type EthtoolQueueField = DumpField<model::SingleQueueModelFieldId>;
+pub type TcField = DumpField<model::SingleTcModelFieldId>;
 
 fn get_advance(
     logger: slog::Logger,
@@ -138,9 +144,11 @@ fn get_advance(
         (Some(host), None) => new_advance_remote(logger.clone(), host, port, time_begin)?,
         (None, Some(snapshot)) => {
             let mut tarball =
-                Archive::new(fs::File::open(&snapshot).context("Failed to open snapshot file")?);
+                Archive::new(fs::File::open(snapshot).context("Failed to open snapshot file")?);
             let mut snapshot_dir = TempDir::with_prefix("snapshot_replay.")?.into_path();
-            tarball.unpack(&snapshot_dir)?;
+            tarball
+                .unpack(&snapshot_dir)
+                .map_err(|_| anyhow!("Failed to unpack snapshot file"))?;
             // Find and append the name of the original snapshot directory
             for path in fs::read_dir(&snapshot_dir)? {
                 snapshot_dir.push(path.unwrap().file_name());
@@ -165,7 +173,7 @@ pub fn parse_pattern<T: FromStr>(
     section_key: &str,
 ) -> Option<Vec<T>> {
     let dump_map = match std::fs::read_to_string(filename) {
-        Ok(belowrc_str) => match belowrc_str.parse::<TValue>() {
+        Ok(belowrc_str) => match toml::from_str::<TValue>(&belowrc_str) {
             Ok(belowrc_val) => belowrc_val
                 .get(get_belowrc_dump_section_key())
                 .unwrap_or_else(|| {
@@ -202,7 +210,7 @@ pub fn parse_pattern<T: FromStr>(
                         panic!("Failed to parse field key {} into string", field)
                     }),
                 )
-                .or_else(|_| Err(format!("Failed to parse field key: {}", field)))
+                .map_err(|_| format!("Failed to parse field key: {}", field))
                 .unwrap()
             })
             .collect(),
@@ -559,5 +567,41 @@ pub fn run(
                 errs,
             )
         }
+        DumpCommand::Tc {
+            fields,
+            opts,
+            pattern,
+        } => {
+            let (time_begin, time_end, advance) =
+                get_advance(logger, dir, host, port, snapshot, &opts)?;
+            let detail = opts.everything || opts.detail;
+            let fields = if let Some(pattern_key) = pattern {
+                parse_pattern(filename, pattern_key, "tc")
+            } else {
+                fields
+            };
+            let fields = expand_fields(
+                match fields.as_ref() {
+                    Some(fields) => fields,
+                    _ => command::DEFAULT_TC_FIELDS,
+                },
+                detail,
+            );
+            let tc = tc::Tc::new(&opts, fields);
+            let mut output: Box<dyn Write> = match opts.output.as_ref() {
+                Some(file_path) => Box::new(File::create(file_path)?),
+                None => Box::new(io::stdout()),
+            };
+            dump_timeseries(
+                advance,
+                time_begin,
+                time_end,
+                &tc,
+                output.as_mut(),
+                opts.output_format,
+                opts.br,
+                errs,
+            )
+        }
     }
 }
