@@ -15,9 +15,9 @@
 use std::collections::BTreeMap;
 use std::time::Duration;
 
-use command::expand_fields;
 use command::GeneralOpt;
 use command::OutputFormat;
+use command::expand_fields;
 use common::logutil::get_logger;
 use model::Collector;
 use model::Queriable;
@@ -41,10 +41,7 @@ fn test_dump_sys_content() {
     let mut fields = command::expand_fields(command::DEFAULT_SYSTEM_FIELDS, true);
     for subquery_id in enum_iterator::all::<model::SingleCpuModelFieldId>() {
         fields.push(DumpField::FieldId(model::SystemModelFieldId::Cpus(
-            model::BTreeMapFieldId {
-                key: Some(31),
-                subquery_id,
-            },
+            model::BTreeMapFieldId::new(Some(31), subquery_id),
         )));
     }
     opts.output_format = Some(OutputFormat::Json);
@@ -83,8 +80,7 @@ fn test_dump_sys_content() {
                             rc.render_title(false)
                         ))
                         .to_owned(),
-                    "Model value and json value do not match for field: {}",
-                    field_id.to_string(),
+                    "Model value and json value do not match for field: {field_id}",
                 );
             }
         }
@@ -97,10 +93,9 @@ fn test_dump_sys_titles() {
         .into_iter()
         .chain(
             enum_iterator::all::<model::SingleCpuModelFieldId>().map(|subquery_id| {
-                DumpField::FieldId(model::SystemModelFieldId::Cpus(model::BTreeMapFieldId {
-                    key: Some(31),
-                    subquery_id,
-                }))
+                DumpField::FieldId(model::SystemModelFieldId::Cpus(
+                    model::BTreeMapFieldId::new(Some(31), subquery_id),
+                ))
             }),
         )
         .filter_map(|dump_field| match dump_field {
@@ -252,8 +247,7 @@ fn test_dump_process_content() {
                                 rc.render_title(false)
                             ))
                             .to_owned(),
-                        "Model value and json value do not match for field: {}",
-                        field_id.to_string(),
+                        "Model value and json value do not match for field: {field_id}",
                     );
                 }
             }
@@ -286,6 +280,7 @@ fn test_dump_proc_titles() {
         "User CPU",
         "Sys CPU",
         "Threads",
+        "Processor",
         "Minflt",
         "Majflt",
         "RSS",
@@ -377,7 +372,7 @@ fn test_dump_proc_select() {
         let pid = item["Pid"].as_str().unwrap();
         let cur_id = pid.parse::<i32>().unwrap();
         if prev_id > 0 {
-            assert!(prev_id < cur_id, "prev_id: {}, cur_id: {}", prev_id, cur_id);
+            assert!(prev_id < cur_id, "prev_id: {prev_id}, cur_id: {cur_id}");
         }
         prev_id = cur_id;
     }
@@ -403,7 +398,7 @@ fn test_dump_proc_select() {
         let pid = item["Pid"].as_str().unwrap();
         let cur_id = pid.parse::<i32>().unwrap();
         if prev_id > 0 {
-            assert!(prev_id > cur_id, "prev_id: {}, cur_id: {}", prev_id, cur_id);
+            assert!(prev_id > cur_id, "prev_id: {prev_id}, cur_id: {cur_id}");
         }
         prev_id = cur_id;
     }
@@ -424,8 +419,7 @@ fn traverse_cgroup_tree(model: &model::CgroupModel, jval: &Value) {
                             rc.render_title(false)
                         ))
                         .to_owned(),
-                    "Model value and json value do not match for field: {}",
-                    field_id.to_string(),
+                    "Model value and json value do not match for field: {field_id}",
                 );
             }
         }
@@ -507,6 +501,7 @@ fn test_dump_cgroup_titles() {
         "File Mapped",
         "File Dirty",
         "File WB",
+        "File THP",
         "Anon THP",
         "Inactive Anon",
         "Active Anon",
@@ -533,11 +528,16 @@ fn test_dump_cgroup_titles() {
         "Pglazyfreed",
         "THP Fault Alloc",
         "THP Collapse Alloc",
-        "Events Low",
-        "Events High",
-        "Events Max",
-        "Events OOM",
-        "Events Kill",
+        "Events Low/s",
+        "Events High/s",
+        "Events Max/s",
+        "Events OOM/s",
+        "Events Kill/s",
+        "Events Local Low/s",
+        "Events Local High/s",
+        "Events Local Max/s",
+        "Events Local OOM/s",
+        "Events Local Kill/s",
         "RBytes",
         "WBytes",
         "R I/O",
@@ -617,8 +617,7 @@ fn test_dump_iface_content() {
                                 rc.render_title(false)
                             ))
                             .to_owned(),
-                        "Model value and json value do not match for field: {}",
-                        field_id.to_string(),
+                        "Model value and json value do not match for field: {field_id}",
                     );
                 }
             }
@@ -727,8 +726,7 @@ fn test_dump_network_content() {
                             rc.render_title(false),
                         ))
                         .to_owned(),
-                    "Model value and json value do not match for field: {}",
-                    field_id.to_string(),
+                    "Model value and json value do not match for field: {field_id}",
                 );
             }
         }
@@ -848,8 +846,7 @@ fn test_dump_transport_content() {
                             rc.render_title(false),
                         ))
                         .to_owned(),
-                    "Model value and json value do not match for field: {}",
-                    field_id.to_string(),
+                    "Model value and json value do not match for field: {field_id}",
                 );
             }
         }
@@ -987,6 +984,7 @@ fn test_dump_queue_content() {
         network,
         gpu: None,
         resctrl: None,
+        tc: None,
     };
 
     let mut opts: GeneralOpt = Default::default();
@@ -1002,6 +1000,11 @@ fn test_dump_queue_content() {
         hostname: "h".to_string(),
     };
 
+    // we are dumping timestamps assuming they are local time
+    // so the timezone needs to be set to the expected TZ
+    // TODO: Audit that the environment access only happens in single-threaded code.
+    unsafe { std::env::set_var("TZ", "US/Pacific") };
+
     let result = queue_dumper
         .dump_model(&ctx, &model, &mut queue_content, &mut round, false)
         .expect("Failed to dump queue model");
@@ -1113,8 +1116,7 @@ fn test_dump_disk_content() {
                                 rc.render_title(false),
                             ))
                             .to_owned(),
-                        "Model value and json value do not match for field: {}",
-                        field_id.to_string(),
+                        "Model value and json value do not match for field: {field_id}",
                     );
                 }
             }
@@ -1224,3 +1226,212 @@ proc = ["datetime", "mem.anon"]
         ))
     );
 }
+
+#[test]
+fn test_tc_titles() {
+    let titles = expand_fields(command::DEFAULT_TC_FIELDS, true)
+        .iter()
+        .filter_map(|dump_field| match dump_field {
+            DumpField::Common(_) => None,
+            DumpField::FieldId(field_id) => Some(field_id.to_string()),
+        })
+        .collect::<Vec<_>>();
+
+    let expected_titles = vec![
+        "interface",
+        "kind",
+        "qlen",
+        "bps",
+        "pps",
+        "bytes_per_sec",
+        "packets_per_sec",
+        "backlog_per_sec",
+        "drops_per_sec",
+        "requeues_per_sec",
+        "overlimits_per_sec",
+        "xstats.fq_codel.maxpacket",
+        "xstats.fq_codel.ecn_mark",
+        "xstats.fq_codel.new_flows_len",
+        "xstats.fq_codel.old_flows_len",
+        "xstats.fq_codel.ce_mark",
+        "xstats.fq_codel.drop_overlimit_per_sec",
+        "xstats.fq_codel.new_flow_count_per_sec",
+        "xstats.fq_codel.memory_usage_per_sec",
+        "xstats.fq_codel.drop_overmemory_per_sec",
+        "qdisc.fq_codel.target",
+        "qdisc.fq_codel.limit",
+        "qdisc.fq_codel.interval",
+        "qdisc.fq_codel.ecn",
+        "qdisc.fq_codel.quantum",
+        "qdisc.fq_codel.ce_threshold",
+        "qdisc.fq_codel.drop_batch_size",
+        "qdisc.fq_codel.memory_limit",
+        "qdisc.fq_codel.flows",
+    ];
+    assert_eq!(titles, expected_titles);
+}
+
+#[test]
+fn test_dump_tc_content() {
+    let tc_models = vec![
+        model::SingleTcModel {
+            interface: "eth0".to_string(),
+            kind: "mq".to_string(),
+            qlen: Some(42),
+            bps: Some(420),
+            pps: Some(1337),
+            bytes_per_sec: Some(299792458),
+            packets_per_sec: Some(314),
+            backlog_per_sec: Some(271828182),
+            drops_per_sec: Some(8675309),
+            requeues_per_sec: Some(12345),
+            overlimits_per_sec: Some(314159),
+            qdisc: None,
+            xstats: None,
+        },
+        model::SingleTcModel {
+            interface: "eth0".to_string(),
+            kind: "fq_codel".to_string(),
+            qlen: Some(42),
+            bps: Some(420),
+            pps: Some(1337),
+            bytes_per_sec: Some(299792458),
+            packets_per_sec: Some(314),
+            backlog_per_sec: Some(271828182),
+            drops_per_sec: Some(8675309),
+            requeues_per_sec: Some(12345),
+            overlimits_per_sec: Some(314159),
+            qdisc: Some(model::QDiscModel {
+                fq_codel: Some(model::FqCodelQDiscModel {
+                    target: 2701,
+                    limit: 7,
+                    interval: 3,
+                    ecn: 6,
+                    quantum: 42,
+                    ce_threshold: 101,
+                    drop_batch_size: 9000,
+                    memory_limit: 123456,
+                    flows: 1024,
+                }),
+            }),
+            xstats: Some(model::XStatsModel {
+                fq_codel: Some(model::FqCodelXStatsModel {
+                    maxpacket: 8675309,
+                    ecn_mark: 299792458,
+                    new_flows_len: 314,
+                    old_flows_len: 1729,
+                    ce_mark: 42,
+                    drop_overlimit_per_sec: Some(420),
+                    new_flow_count_per_sec: Some(1337),
+                    memory_usage_per_sec: Some(271828182),
+                    drop_overmemory_per_sec: Some(27182),
+                }),
+            }),
+        },
+    ];
+
+    let model = model::Model {
+        time_elapsed: Duration::from_secs(60 * 10),
+        timestamp: SystemTime::now(),
+        system: model::SystemModel::default(),
+        cgroup: model::CgroupModel::default(),
+        process: model::ProcessModel::default(),
+        network: model::NetworkModel::default(),
+        gpu: None,
+        resctrl: None,
+        tc: Some(model::TcModel { tc: tc_models }),
+    };
+
+    let mut opts: GeneralOpt = Default::default();
+    let fields = command::expand_fields(command::DEFAULT_TC_FIELDS, true);
+
+    opts.output_format = Some(OutputFormat::Json);
+    let queue_dumper = tc::Tc::new(&opts, fields.clone());
+
+    let mut queue_content: Vec<u8> = Vec::new();
+    let mut round = 0;
+    let ctx = CommonFieldContext {
+        timestamp: 0,
+        hostname: "h".to_string(),
+    };
+
+    let result = queue_dumper
+        .dump_model(&ctx, &model, &mut queue_content, &mut round, false)
+        .expect("Failed to dump queue model");
+    assert!(result == tmain::IterExecResult::Success);
+
+    // verify json correctness
+    assert!(!queue_content.is_empty());
+    let jval: Value =
+        serde_json::from_slice(&queue_content).expect("Fail parse json of queue dump");
+
+    let expected_json = json!([
+        {
+            "Datetime": "1969-12-31 16:00:00",
+            "Interface": "eth0",
+            "Kind": "mq",
+            "Queue Length": "42",
+            "Bps": "420 B/s",
+            "Pps": "1337/s",
+            "Bytes": "285.9 MB/s",
+            "Packets": "314/s",
+            "Backlog": "271828182/s",
+            "Drops": "8675309/s",
+            "Requeues": "12345/s",
+            "Overlimits": "314159/s",
+            "Target": "?",
+            "Limit": "?",
+            "Interval": "?",
+            "Ecn": "?",
+            "Quantum": "?",
+            "CeThreshold": "?",
+            "DropBatchSize": "?",
+            "MemoryLimit": "?",
+            "Flows": "?",
+            "MaxPacket": "?",
+            "EcnMark": "?",
+            "NewFlowsLen": "?",
+            "OldFlowsLen": "?",
+            "CeMark": "?",
+            "DropOverlimit": "?",
+            "NewFlowCount": "?",
+            "MemoryUsage": "?",
+            "DropOvermemory": "?",
+            "Timestamp": "0"
+        },
+        {
+            "Datetime": "1969-12-31 16:00:00",
+            "Interface": "eth0",
+            "Kind": "fq_codel",
+            "Queue Length": "42",
+            "Bps": "420 B/s",
+            "Pps": "1337/s",
+            "Bytes": "285.9 MB/s",
+            "Packets": "314/s",
+            "Backlog": "271828182/s",
+            "Drops": "8675309/s",
+            "Requeues": "12345/s",
+            "Overlimits": "314159/s",
+            "Target": "2701",
+            "Limit": "7",
+            "Interval": "3",
+            "Ecn": "6",
+            "Quantum": "42",
+            "CeThreshold": "101",
+            "DropBatchSize": "9000",
+            "MemoryLimit": "123456",
+            "Flows": "1024",
+            "MaxPacket": "8675309",
+            "EcnMark": "299792458",
+            "NewFlowsLen": "314",
+            "OldFlowsLen": "1729",
+            "CeMark": "42",
+            "DropOverlimit": "420/s",
+            "NewFlowCount": "1337/s",
+            "MemoryUsage": "271828182/s",
+            "DropOvermemory": "27182/s",
+            "Timestamp": "0"
+        }
+    ]);
+    assert_eq!(jval, expected_json);
+}
