@@ -16,7 +16,7 @@ use super::*;
 
 /// Collection of all data local to the cgroup, e.g. its memory/io/cpu/pids usage.
 /// Nothing about child cgroups or siblings, and therefore "Single" in its name.
-#[derive(Clone, Debug, Default, Serialize, Deserialize, below_derive::Queriable)]
+#[::below_derive::queriable_derives]
 pub struct SingleCgroupModel {
     pub name: String,
     pub full_path: String,
@@ -70,75 +70,46 @@ pub struct CgroupModel {
 /// The path is used to drill into the Cgroup Model tree. If Vec empty, the
 /// current CgroupModel is selected and queried with the subquery_id.
 /// The path is optional in parsing and converting to String.
+pub type CgroupModelFieldId = QueriableContainerFieldId<CgroupModel>;
+
 #[derive(Clone, Debug, PartialEq)]
-pub struct CgroupModelFieldId {
-    /// To drill into children recursively. If Vec empty, queries self.
-    /// None is only for listing variants and otherwise invalid.
-    pub path: Option<Vec<String>>,
-    pub subquery_id: SingleCgroupModelFieldId,
+pub struct CgroupPath {
+    pub path: Vec<String>,
 }
 
-impl FieldId for CgroupModelFieldId {
-    type Queriable = CgroupModel;
-}
-
-impl DelegatedSequence for CgroupModelFieldId {
-    type Delegate = SingleCgroupModelFieldId;
-    fn get_delegate(&self) -> &Self::Delegate {
-        &self.subquery_id
-    }
-    fn from_delegate(delegate: Self::Delegate) -> Self {
-        Self {
-            path: None,
-            subquery_id: delegate,
-        }
+impl std::fmt::Display for CgroupPath {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "path:/{}/", self.path.join("/"))
     }
 }
 
-impl Sequence for CgroupModelFieldId {
-    impl_sequence_for_delegated_sequence!();
-}
-
-impl std::string::ToString for CgroupModelFieldId {
-    fn to_string(&self) -> String {
-        match &self.path {
-            Some(path) if path.is_empty() => self.subquery_id.to_string(),
-            Some(path) => format!("path:/{}/.{}", path.join("/"), self.subquery_id.to_string()),
-            None => format!("[path:/<cgroup_path>/.]{}", self.subquery_id.to_string()),
-        }
-    }
-}
-
-impl std::str::FromStr for CgroupModelFieldId {
+impl FromStr for CgroupPath {
     type Err = anyhow::Error;
     fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
-        let (path_str, subquery_id_str) = if s.starts_with("path:") {
-            s["path:".len()..]
-                .rsplit_once("/.")
-                .ok_or_else(|| anyhow!("Path is not terminated by `/.`: {}", s))?
-        } else {
-            ("", s)
-        };
-        let path = Some(
-            path_str
+        if !s.starts_with("path:/") {
+            return Err(anyhow!("Path is not prefixed with `path:/`: {}", s));
+        }
+        Ok(Self {
+            path: s["path:/".len()..]
                 .split('/')
                 .filter(|part| !part.is_empty())
                 .map(|part| part.to_owned())
                 .collect(),
-        );
-        let subquery_id = SingleCgroupModelFieldId::from_str(subquery_id_str)?;
-        Ok(Self { path, subquery_id })
+        })
     }
 }
 
-impl Queriable for CgroupModel {
-    type FieldId = CgroupModelFieldId;
-    fn query(&self, field_id: &Self::FieldId) -> Option<Field> {
-        let mut model = self;
-        for part in field_id.path.as_ref()?.iter() {
-            model = model.children.get(part.as_str())?;
-        }
-        model.data.query(&field_id.subquery_id)
+impl QueriableContainer for CgroupModel {
+    type Idx = CgroupPath;
+    type SubqueryId = SingleCgroupModelFieldId;
+    const IDX_PLACEHOLDER: &'static str = "[path:/<cgroup_path>/.]";
+    fn split(s: &str) -> Option<(&str, &str)> {
+        let idx_end = s.rfind("/.")?;
+        Some((&s[..idx_end + 1], &s[idx_end + 2..]))
+    }
+    fn get_item(&self, idx: &Self::Idx) -> Option<&SingleCgroupModel> {
+        self.get_by_path_iter(idx.path.iter())
+            .map(|model| &model.data)
     }
 }
 
@@ -201,7 +172,7 @@ impl CgroupModel {
                             begin.get(device_name).map(|begin_io_stat| {
                                 (
                                     device_name.clone(),
-                                    CgroupIoModel::new(&begin_io_stat, &end_io_stat, delta),
+                                    CgroupIoModel::new(begin_io_stat, end_io_stat, delta),
                                 )
                             })
                         })
@@ -225,10 +196,7 @@ impl CgroupModel {
 
         let pids = Some(CgroupPidsModel::new(sample));
 
-        let pressure = sample
-            .pressure
-            .as_ref()
-            .map(|p| CgroupPressureModel::new(p));
+        let pressure = sample.pressure.as_ref().map(CgroupPressureModel::new);
 
         let cgroup_stat = sample.cgroup_stat.as_ref().map(CgroupStatModel::new);
 
@@ -261,9 +229,9 @@ impl CgroupModel {
             .map(|(child_name, child_sample)| {
                 CgroupModel::new(
                     child_name.clone(),
-                    format!("{}/{}", full_path, child_name),
+                    format!("{full_path}/{child_name}"),
                     depth + 1,
-                    &child_sample,
+                    child_sample,
                     last.and_then(|(last, delta)| {
                         last.children
                             .as_ref()
@@ -298,11 +266,55 @@ impl CgroupModel {
     }
 
     pub fn aggr_top_level_val(mut self) -> Self {
-        self.data.memory = self.children.iter().fold(Default::default(), |acc, model| {
-            opt_add(acc, model.data.memory.clone())
-        });
+        if let Some(memory) = &self.data.memory {
+            // If root model has the value, return it directly
+            if memory.total.is_some() {
+                return self;
+            }
+        }
+        // Manually aggregate specified fields from children
+        let mut aggregated_memory = CgroupMemoryModel::default();
+        for child in &self.children {
+            if let Some(child_memory) = &child.data.memory {
+                aggregated_memory.total = opt_add(aggregated_memory.total, child_memory.total);
+                aggregated_memory.swap = opt_add(aggregated_memory.swap, child_memory.swap);
+                aggregated_memory.events_low =
+                    opt_add(aggregated_memory.events_low, child_memory.events_low);
+                aggregated_memory.events_high =
+                    opt_add(aggregated_memory.events_high, child_memory.events_high);
+                aggregated_memory.events_max =
+                    opt_add(aggregated_memory.events_max, child_memory.events_max);
+                aggregated_memory.events_oom =
+                    opt_add(aggregated_memory.events_oom, child_memory.events_oom);
+                aggregated_memory.events_oom_kill = opt_add(
+                    aggregated_memory.events_oom_kill,
+                    child_memory.events_oom_kill,
+                );
+            }
+        }
+        // Assign aggregated values to root memory model
+        if let Some(memory) = &mut self.data.memory {
+            memory.total = aggregated_memory.total;
+            memory.swap = aggregated_memory.swap;
+            memory.events_low = aggregated_memory.events_low;
+            memory.events_high = aggregated_memory.events_high;
+            memory.events_max = aggregated_memory.events_max;
+            memory.events_oom = aggregated_memory.events_oom;
+            memory.events_oom_kill = aggregated_memory.events_oom_kill;
+        }
         self
     }
+
+    fn get_by_path_iter(
+        &self,
+        mut path: impl Iterator<Item = impl AsRef<str>>,
+    ) -> Option<&CgroupModel> {
+        path.try_fold(self, |cur, p| cur.children.get(p.as_ref()))
+    }
+
+    pub fn get_by_path_str(&self, path: &str) -> Option<&CgroupModel> {
+        self.get_by_path_iter(path.split('/').filter(|x| !x.is_empty()))
+    }
 }
 
 impl Nameable for CgroupModel {
@@ -323,15 +335,7 @@ impl Nameable for SingleCgroupModel {
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct CgroupCpuModel {
     pub usage_pct: Option<f64>,
     pub user_pct: Option<f64>,
@@ -358,15 +362,7 @@ impl CgroupCpuModel {
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct CgroupStatModel {
     pub nr_descendants: Option<u32>,
     pub nr_dying_descendants: Option<u32>,
@@ -381,15 +377,7 @@ impl CgroupStatModel {
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct CgroupIoModel {
     pub rbytes_per_sec: Option<f64>,
     pub wbytes_per_sec: Option<f64>,
@@ -408,7 +396,7 @@ impl CgroupIoModel {
     pub fn new(begin: &cgroupfs::IoStat, end: &cgroupfs::IoStat, delta: Duration) -> CgroupIoModel {
         let rbytes_per_sec = count_per_sec!(begin.rbytes, end.rbytes, delta);
         let wbytes_per_sec = count_per_sec!(begin.wbytes, end.wbytes, delta);
-        let rwbytes_per_sec = opt_add(rbytes_per_sec.clone(), wbytes_per_sec.clone());
+        let rwbytes_per_sec = opt_add(rbytes_per_sec, wbytes_per_sec);
         CgroupIoModel {
             rbytes_per_sec,
             wbytes_per_sec,
@@ -463,15 +451,7 @@ impl std::ops::Add<&CgroupIoModel> for CgroupIoModel {
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct CgroupMemoryModel {
     pub total: Option<u64>,
     pub swap: Option<u64>,
@@ -487,6 +467,7 @@ pub struct CgroupMemoryModel {
     pub file_mapped: Option<u64>,
     pub file_dirty: Option<u64>,
     pub file_writeback: Option<u64>,
+    pub file_thp: Option<u64>,
     pub anon_thp: Option<u64>,
     pub inactive_anon: Option<u64>,
     pub active_anon: Option<u64>,
@@ -518,6 +499,11 @@ pub struct CgroupMemoryModel {
     pub events_max: Option<u64>,
     pub events_oom: Option<u64>,
     pub events_oom_kill: Option<u64>,
+    pub events_local_low: Option<u64>,
+    pub events_local_high: Option<u64>,
+    pub events_local_max: Option<u64>,
+    pub events_local_oom: Option<u64>,
+    pub events_local_oom_kill: Option<u64>,
 }
 
 impl std::ops::Add for CgroupMemoryModel {
@@ -539,6 +525,7 @@ impl std::ops::Add for CgroupMemoryModel {
             file_mapped: opt_add(self.file_mapped, other.file_mapped),
             file_dirty: opt_add(self.file_dirty, other.file_dirty),
             file_writeback: opt_add(self.file_writeback, other.file_writeback),
+            file_thp: opt_add(self.file_thp, other.file_thp),
             anon_thp: opt_add(self.anon_thp, other.anon_thp),
             inactive_anon: opt_add(self.inactive_anon, other.inactive_anon),
             active_anon: opt_add(self.active_anon, other.active_anon),
@@ -591,6 +578,11 @@ impl std::ops::Add for CgroupMemoryModel {
             events_max: opt_add(self.events_max, other.events_max),
             events_oom: opt_add(self.events_oom, other.events_oom),
             events_oom_kill: opt_add(self.events_oom_kill, other.events_oom_kill),
+            events_local_low: opt_add(self.events_local_low, other.events_local_low),
+            events_local_high: opt_add(self.events_local_high, other.events_local_high),
+            events_local_max: opt_add(self.events_local_max, other.events_local_max),
+            events_local_oom: opt_add(self.events_local_oom, other.events_local_oom),
+            events_local_oom_kill: opt_add(self.events_local_oom_kill, other.events_local_oom_kill),
         }
     }
 }
@@ -607,11 +599,47 @@ impl CgroupMemoryModel {
             ..Default::default()
         };
         if let Some(events) = &sample.memory_events {
-            model.events_low = events.low;
-            model.events_high = events.high;
-            model.events_max = events.max;
-            model.events_oom = events.oom;
-            model.events_oom_kill = events.oom_kill;
+            if let Some((
+                CgroupSample {
+                    memory_events: Some(last_memory_events),
+                    ..
+                },
+                delta,
+            )) = last
+            {
+                model.events_low = count_per_sec!(last_memory_events.low, events.low, delta, u64);
+                model.events_high =
+                    count_per_sec!(last_memory_events.high, events.high, delta, u64);
+                model.events_max = count_per_sec!(last_memory_events.max, events.max, delta, u64);
+                model.events_oom = count_per_sec!(last_memory_events.oom, events.oom, delta, u64);
+                model.events_oom_kill =
+                    count_per_sec!(last_memory_events.oom_kill, events.oom_kill, delta, u64);
+            }
+        }
+        if let Some(events_local) = &sample.memory_events_local {
+            if let Some((
+                CgroupSample {
+                    memory_events_local: Some(last_memory_events_local),
+                    ..
+                },
+                delta,
+            )) = last
+            {
+                model.events_local_low =
+                    count_per_sec!(last_memory_events_local.low, events_local.low, delta, u64);
+                model.events_local_high =
+                    count_per_sec!(last_memory_events_local.high, events_local.high, delta, u64);
+                model.events_local_max =
+                    count_per_sec!(last_memory_events_local.max, events_local.max, delta, u64);
+                model.events_local_oom =
+                    count_per_sec!(last_memory_events_local.oom, events_local.oom, delta, u64);
+                model.events_local_oom_kill = count_per_sec!(
+                    last_memory_events_local.oom_kill,
+                    events_local.oom_kill,
+                    delta,
+                    u64
+                );
+            }
         }
         if let Some(stat) = &sample.memory_stat {
             model.anon = stat.anon;
@@ -629,6 +657,7 @@ impl CgroupMemoryModel {
             model.file_mapped = stat.file_mapped;
             model.file_dirty = stat.file_dirty;
             model.file_writeback = stat.file_writeback;
+            model.file_thp = stat.file_thp;
             model.anon_thp = stat.anon_thp;
             model.inactive_anon = stat.inactive_anon;
             model.active_anon = stat.active_anon;
@@ -717,15 +746,7 @@ impl CgroupMemoryModel {
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct CgroupPidsModel {
     pub tids_current: Option<u64>,
 }
@@ -747,15 +768,7 @@ impl CgroupPidsModel {
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct CgroupPressureModel {
     pub cpu_some_pct: Option<f64>,
     pub cpu_full_pct: Option<f64>,
@@ -780,15 +793,7 @@ impl CgroupPressureModel {
         }
     }
 }
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct CgroupMemoryNumaModel {
     pub total: Option<u64>,
     pub anon: Option<u64>,
@@ -900,15 +905,7 @@ impl CgroupMemoryNumaModel {
 
 /// Cgroup properties. Without any cgroup configuration changes, these should
 /// typically be static.
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct CgroupProperties {
     pub cgroup_controllers: Option<BTreeSet<String>>,
     pub cgroup_subtree_control: Option<BTreeSet<String>>,
@@ -919,6 +916,8 @@ pub struct CgroupProperties {
     pub memory_max: Option<i64>,
     pub memory_swap_max: Option<i64>,
     pub memory_zswap_max: Option<i64>,
+    pub memory_oom_group: Option<u32>,
+    pub memory_zswap_writeback: Option<u32>,
     pub cpu_weight: Option<u32>,
     pub cpu_max_usec: Option<i64>,
     pub cpu_max_period_usec: Option<u64>,
@@ -940,6 +939,8 @@ impl CgroupProperties {
             memory_max: sample.memory_max,
             memory_swap_max: sample.memory_swap_max,
             memory_zswap_max: sample.memory_zswap_max,
+            memory_oom_group: sample.memory_oom_group,
+            memory_zswap_writeback: sample.memory_zswap_writeback,
             cpu_weight: sample.cpu_weight,
             cpu_max_usec: sample.cpu_max.as_ref().map(|v| v.max_usec),
             cpu_max_period_usec: sample.cpu_max.as_ref().map(|v| v.period_usec),
@@ -990,8 +991,6 @@ mod tests {
         let model: CgroupModel =
             serde_json::from_str(model_json).expect("Failed to deserialize cgroup model JSON");
         for (field_id, expected) in &[
-            // "path:" omitted falls back to querying self (root)
-            ("full_path", Some("")),
             // Ignore consecutive slashes
             ("path:///////.name", Some("<root>")),
             ("path:/system.slice/.full_path", Some("/system.slice")),
@@ -1008,7 +1007,7 @@ mod tests {
             assert_eq!(
                 model.query(
                     &CgroupModelFieldId::from_str(field_id)
-                        .map_err(|e| format!("Failed to parse field id {}: {:?}", field_id, e))
+                        .map_err(|e| format!("Failed to parse field id {field_id}: {e:?}"))
                         .unwrap()
                 ),
                 expected.map(|s| Field::Str(s.to_string()))
@@ -1038,4 +1037,76 @@ mod tests {
             Some(Field::F64(42.0))
         );
     }
+
+    #[test]
+    fn test_aggr_top_level_val() {
+        /*
+        Summary: This test verifies the aggregation of top-level values in a CgroupModel, executed in the aggr_top_level_val() function.
+        The test creates two child cgroup models with memory values, then uses them to create a root cgroup model.
+        The root cgroup model is aggregated using the aggr_top_level_val() function. During aggregation, local events should not be aggregated.
+        The test checks that the total memory is aggregated correctly and confirms that local events are not aggregated.
+        */
+        // Create two children cgroup models with memory values
+        let child1 = CgroupModel {
+            data: SingleCgroupModel {
+                name: "child1".to_string(),
+                memory: Some(CgroupMemoryModel {
+                    total: Some(100),
+                    swap: Some(50),
+                    events_low: Some(1),
+                    events_high: Some(2),
+                    events_max: Some(3),
+                    events_oom: Some(4),
+                    events_oom_kill: Some(5),
+                    events_local_low: Some(10),
+                    events_local_high: Some(20),
+                    ..Default::default()
+                }),
+                ..Default::default()
+            },
+            ..Default::default()
+        };
+        let child2 = CgroupModel {
+            data: SingleCgroupModel {
+                name: "child2".to_string(),
+                memory: Some(CgroupMemoryModel {
+                    total: Some(200),
+                    swap: Some(30),
+                    events_low: Some(1),
+                    events_high: Some(2),
+                    events_max: Some(3),
+                    events_oom: Some(4),
+                    events_oom_kill: Some(5),
+                    events_local_low: Some(10),
+                    events_local_high: Some(20),
+                    ..Default::default()
+                }),
+                ..Default::default()
+            },
+            ..Default::default()
+        };
+        // Create root cgroup model without pre-existing memory values
+        let mut root = CgroupModel {
+            data: SingleCgroupModel {
+                name: "root".to_string(),
+                memory: Some(CgroupMemoryModel::default()),
+                ..Default::default()
+            },
+            children: vec![child1, child2].into_iter().collect(),
+            ..Default::default()
+        };
+        // Aggregate top-level values
+        root = root.aggr_top_level_val();
+        // Check that the total memory is aggregated correctly
+        assert_eq!(root.data.memory.as_ref().unwrap().total, Some(300));
+        assert_eq!(root.data.memory.as_ref().unwrap().swap, Some(80));
+        assert_eq!(root.data.memory.as_ref().unwrap().events_low, Some(2));
+        assert_eq!(root.data.memory.as_ref().unwrap().events_high, Some(4));
+        assert_eq!(root.data.memory.as_ref().unwrap().events_max, Some(6));
+        assert_eq!(root.data.memory.as_ref().unwrap().events_oom, Some(8));
+        assert_eq!(root.data.memory.as_ref().unwrap().events_oom_kill, Some(10));
+        // Confirm local events are not aggregated
+        assert_eq!(root.data.memory.as_ref().unwrap().events_local_low, None);
+        assert_eq!(root.data.memory.as_ref().unwrap().events_local_high, None);
+    }
 }
