@@ -30,12 +30,16 @@ pub struct CollectorOptions {
     pub disable_disk_stat: bool,
     pub enable_btrfs_stats: bool,
     pub enable_ethtool_stats: bool,
+    pub enable_ksm_stats: bool,
     pub enable_resctrl_stats: bool,
+    pub enable_tc_stats: bool,
     pub btrfs_samples: u64,
     pub btrfs_min_pct: f64,
     pub cgroup_re: Option<Regex>,
     pub gpu_stats_receiver:
         Option<collector_plugin::Consumer<crate::gpu_stats_collector_plugin::SampleType>>,
+    pub tc_stats_receiver:
+        Option<collector_plugin::Consumer<crate::tc_collector_plugin::SampleType>>,
 }
 
 impl Default for CollectorOptions {
@@ -47,11 +51,14 @@ impl Default for CollectorOptions {
             disable_disk_stat: false,
             enable_btrfs_stats: false,
             enable_ethtool_stats: false,
+            enable_ksm_stats: false,
             enable_resctrl_stats: false,
+            enable_tc_stats: false,
             btrfs_samples: btrfs::DEFAULT_SAMPLES,
             btrfs_min_pct: btrfs::DEFAULT_MIN_PCT,
             cgroup_re: None,
             gpu_stats_receiver: None,
+            tc_stats_receiver: None,
         }
     }
 }
@@ -181,6 +188,7 @@ fn collect_sample(
     let btrfs_reader =
         btrfs::BtrfsReader::new(options.btrfs_samples, options.btrfs_min_pct, logger.clone());
     let ethtool_reader = ethtool::EthtoolReader::new();
+    let ksm_reader = procfs::KsmReader::new();
 
     // Take mutex, then take all values out of shared map and replace with default map
     //
@@ -211,7 +219,12 @@ fn collect_sample(
             stat: reader.read_stat()?,
             meminfo: reader.read_meminfo()?,
             vmstat: reader.read_vmstat()?,
-            slabinfo: reader.read_slabinfo().unwrap_or_default(),
+            slabinfo_vec: reader.read_slabinfo().unwrap_or_default(),
+            ksm: if !options.enable_ksm_stats {
+                None
+            } else {
+                Some(ksm_reader.read_ksm())
+            },
             hostname: get_hostname()?,
             kernel_version: match reader.read_kernel_version() {
                 Ok(k) => Some(k),
@@ -262,17 +275,8 @@ fn collect_sample(
             if let Some(gpu_stats_receiver) = &options.gpu_stats_receiver {
                 // It is possible to receive no sample if the
                 // collector has not updated since the previous take
-                // or the collector encountered a recoverable error
-                // (e.g. timeout). The behavior for now is to store an
-                // empty map. Alternatively we could store the latest
-                // sample and read that, but then we have to decide how
-                // stale the data can be.
-                Some(
-                    gpu_stats_receiver
-                        .try_take()
-                        .context("GPU stats collector had an error")?
-                        .unwrap_or_default(),
-                )
+                // or the collector encountered errors (e.g. timeout).
+                gpu_stats_receiver.take()
             } else {
                 None
             }
@@ -306,6 +310,11 @@ fn collect_sample(
                 }
             }
         },
+        tc: if let Some(tc_stats_receiver) = &options.tc_stats_receiver {
+            tc_stats_receiver.take()
+        } else {
+            None
+        },
     })
 }
 
@@ -373,13 +382,13 @@ fn collect_cgroup_sample(
         None
     };
     Ok(CgroupSample {
-        cpu_stat: wrap(reader.read_cpu_stat())?.map(Into::into),
+        cpu_stat: wrap(reader.read_cpu_stat())?,
         io_stat,
         tids_current: wrap(reader.read_pids_current())?,
         tids_max: wrap(reader.read_pids_max())?,
         memory_current: wrap(reader.read_memory_current().map(|v| v as i64))?,
-        memory_stat: wrap(reader.read_memory_stat())?.map(Into::into),
-        pressure: pressure_wrap(reader.read_pressure())?.map(Into::into),
+        memory_stat: wrap(reader.read_memory_stat())?,
+        pressure: pressure_wrap(reader.read_pressure())?,
         // We transpose at the end here to convert the
         // Option<Result<BTreeMap... into Result<Option<BTreeMap and
         // then bail any errors with `?` - leaving us with the
@@ -424,7 +433,10 @@ fn collect_cgroup_sample(
         memory_max: wrap(reader.read_memory_max())?,
         memory_swap_max: wrap(reader.read_memory_swap_max())?,
         memory_zswap_max: wrap(reader.read_memory_zswap_max())?,
-        memory_events: wrap(reader.read_memory_events())?.map(Into::into),
+        memory_zswap_writeback: wrap(reader.read_memory_zswap_writeback())?,
+        memory_oom_group: wrap(reader.read_memory_oom_group())?,
+        memory_events: wrap(reader.read_memory_events())?,
+        memory_events_local: wrap(reader.read_memory_events_local())?,
         inode_number: match reader.read_inode_number() {
             Ok(st_ino) => Some(st_ino as i64),
             Err(e) => {
@@ -432,8 +444,8 @@ fn collect_cgroup_sample(
                 None
             }
         },
-        cgroup_stat: wrap(reader.read_cgroup_stat())?.map(Into::into),
-        memory_numa_stat: wrap(reader.read_memory_numa_stat())?.map(Into::into),
+        cgroup_stat: wrap(reader.read_cgroup_stat())?,
+        memory_numa_stat: wrap(reader.read_memory_numa_stat())?,
         cpuset_cpus: wrap(reader.read_cpuset_cpus())?,
         cpuset_cpus_effective: wrap(reader.read_cpuset_cpus_effective())?,
         cpuset_mems: wrap(reader.read_cpuset_mems())?,
@@ -467,6 +479,13 @@ macro_rules! count_per_sec {
         }
         ret
     }};
+    ($a:ident, $b:ident, $delta:expr, $target_type:ty) => {{
+        let mut ret = None;
+        if $a <= $b {
+            ret = Some((($b - $a) as f64 / $delta.as_secs_f64()).ceil() as $target_type);
+        }
+        ret
+    }};
     ($a_opt:expr, $b_opt:expr, $delta:expr, $target_type:ty) => {{
         let mut ret = None;
         if let (Some(a), Some(b)) = ($a_opt, $b_opt) {
