@@ -23,7 +23,7 @@
 ///
 /// This list also servers as documentation for available field ids that could
 /// be used in other below crates. A test ensures that this list is up-to-date.
-pub const COMMON_MODEL_FIELD_IDS: [&str; 420] = [
+pub const COMMON_MODEL_FIELD_IDS: [&str; 482] = [
     "system.hostname",
     "system.kernel_version",
     "system.os_release",
@@ -103,16 +103,40 @@ pub const COMMON_MODEL_FIELD_IDS: [&str; 420] = [
     "system.vm.pgscan_kswapd",
     "system.vm.pgscan_direct",
     "system.vm.oom_kill",
-    "system.slab.<key>.name",
-    "system.slab.<key>.active_objs",
-    "system.slab.<key>.num_objs",
-    "system.slab.<key>.obj_size",
-    "system.slab.<key>.obj_per_slab",
-    "system.slab.<key>.num_slabs",
-    "system.slab.<key>.active_caches",
-    "system.slab.<key>.num_caches",
-    "system.slab.<key>.active_size",
-    "system.slab.<key>.total_size",
+    "system.slab.<idx>.name",
+    "system.slab.<idx>.active_objs",
+    "system.slab.<idx>.num_objs",
+    "system.slab.<idx>.obj_size",
+    "system.slab.<idx>.obj_per_slab",
+    "system.slab.<idx>.num_slabs",
+    "system.slab.<idx>.active_caches",
+    "system.slab.<idx>.num_caches",
+    "system.slab.<idx>.active_size",
+    "system.slab.<idx>.total_size",
+    "system.ksm.advisor_max_cpu",
+    "system.ksm.advisor_max_pages_to_scan",
+    "system.ksm.advisor_min_pages_to_scan",
+    "system.ksm.advisor_mode",
+    "system.ksm.advisor_target_scan_time",
+    "system.ksm.full_scans",
+    "system.ksm.general_profit",
+    "system.ksm.ksm_zero_pages",
+    "system.ksm.max_page_sharing",
+    "system.ksm.merge_across_nodes",
+    "system.ksm.pages_scanned",
+    "system.ksm.pages_shared",
+    "system.ksm.pages_sharing",
+    "system.ksm.pages_skipped",
+    "system.ksm.pages_to_scan",
+    "system.ksm.pages_unshared",
+    "system.ksm.pages_volatile",
+    "system.ksm.run",
+    "system.ksm.sleep_millisecs",
+    "system.ksm.smart_scan",
+    "system.ksm.stable_node_chains",
+    "system.ksm.stable_node_chains_prune_millisecs",
+    "system.ksm.stable_node_dups",
+    "system.ksm.use_zero_pages",
     "system.disks.<key>.name",
     "system.disks.<key>.disk_usage",
     "system.disks.<key>.partition_size",
@@ -157,6 +181,8 @@ pub const COMMON_MODEL_FIELD_IDS: [&str; 420] = [
     "cgroup.[path:/<cgroup_path>/.]props.memory_min",
     "cgroup.[path:/<cgroup_path>/.]props.memory_swap_max",
     "cgroup.[path:/<cgroup_path>/.]props.memory_zswap_max",
+    "cgroup.[path:/<cgroup_path>/.]props.memory_zswap_writeback",
+    "cgroup.[path:/<cgroup_path>/.]props.memory_oom_group",
     "cgroup.[path:/<cgroup_path>/.]cpu.usage_pct",
     "cgroup.[path:/<cgroup_path>/.]cpu.user_pct",
     "cgroup.[path:/<cgroup_path>/.]cpu.system_pct",
@@ -178,6 +204,7 @@ pub const COMMON_MODEL_FIELD_IDS: [&str; 420] = [
     "cgroup.[path:/<cgroup_path>/.]mem.file_mapped",
     "cgroup.[path:/<cgroup_path>/.]mem.file_dirty",
     "cgroup.[path:/<cgroup_path>/.]mem.file_writeback",
+    "cgroup.[path:/<cgroup_path>/.]mem.file_thp",
     "cgroup.[path:/<cgroup_path>/.]mem.anon_thp",
     "cgroup.[path:/<cgroup_path>/.]mem.inactive_anon",
     "cgroup.[path:/<cgroup_path>/.]mem.active_anon",
@@ -209,6 +236,11 @@ pub const COMMON_MODEL_FIELD_IDS: [&str; 420] = [
     "cgroup.[path:/<cgroup_path>/.]mem.events_max",
     "cgroup.[path:/<cgroup_path>/.]mem.events_oom",
     "cgroup.[path:/<cgroup_path>/.]mem.events_oom_kill",
+    "cgroup.[path:/<cgroup_path>/.]mem.events_local_low",
+    "cgroup.[path:/<cgroup_path>/.]mem.events_local_high",
+    "cgroup.[path:/<cgroup_path>/.]mem.events_local_max",
+    "cgroup.[path:/<cgroup_path>/.]mem.events_local_oom",
+    "cgroup.[path:/<cgroup_path>/.]mem.events_local_oom_kill",
     "cgroup.[path:/<cgroup_path>/.]io_details.<key>.rbytes_per_sec",
     "cgroup.[path:/<cgroup_path>/.]io_details.<key>.wbytes_per_sec",
     "cgroup.[path:/<cgroup_path>/.]io_details.<key>.rios_per_sec",
@@ -326,6 +358,7 @@ pub const COMMON_MODEL_FIELD_IDS: [&str; 420] = [
     "process.processes.<key>.cpu.user_pct",
     "process.processes.<key>.cpu.system_pct",
     "process.processes.<key>.cpu.num_threads",
+    "process.processes.<key>.cpu.processor",
     "process.processes.<key>.cmdline",
     "process.processes.<key>.exe_path",
     "network.interfaces.<key>.interface",
@@ -444,4 +477,33 @@ pub const COMMON_MODEL_FIELD_IDS: [&str; 420] = [
     "network.udp6.sndbuf_errors",
     "network.udp6.in_csum_errors",
     "network.udp6.ignored_multi",
+    "tc.tc.<idx>.backlog_per_sec",
+    "tc.tc.<idx>.bps",
+    "tc.tc.<idx>.bytes_per_sec",
+    "tc.tc.<idx>.interface",
+    "tc.tc.<idx>.drops_per_sec",
+    "tc.tc.<idx>.kind",
+    "tc.tc.<idx>.overlimits_per_sec",
+    "tc.tc.<idx>.packets_per_sec",
+    "tc.tc.<idx>.pps",
+    "tc.tc.<idx>.qdisc.fq_codel.ce_threshold",
+    "tc.tc.<idx>.qdisc.fq_codel.drop_batch_size",
+    "tc.tc.<idx>.qdisc.fq_codel.ecn",
+    "tc.tc.<idx>.qdisc.fq_codel.flows",
+    "tc.tc.<idx>.qdisc.fq_codel.interval",
+    "tc.tc.<idx>.qdisc.fq_codel.limit",
+    "tc.tc.<idx>.qdisc.fq_codel.memory_limit",
+    "tc.tc.<idx>.qdisc.fq_codel.quantum",
+    "tc.tc.<idx>.qdisc.fq_codel.target",
+    "tc.tc.<idx>.qlen",
+    "tc.tc.<idx>.requeues_per_sec",
+    "tc.tc.<idx>.xstats.fq_codel.ce_mark",
+    "tc.tc.<idx>.xstats.fq_codel.drop_overlimit_per_sec",
+    "tc.tc.<idx>.xstats.fq_codel.drop_overmemory_per_sec",
+    "tc.tc.<idx>.xstats.fq_codel.ecn_mark",
+    "tc.tc.<idx>.xstats.fq_codel.maxpacket",
+    "tc.tc.<idx>.xstats.fq_codel.memory_usage_per_sec",
+    "tc.tc.<idx>.xstats.fq_codel.new_flow_count_per_sec",
+    "tc.tc.<idx>.xstats.fq_codel.new_flows_len",
+    "tc.tc.<idx>.xstats.fq_codel.old_flows_len",
 ];
