@@ -131,8 +131,8 @@ pub const SAMPLE_MODEL_JSON: &str = r#"
             "pgscan_direct": 0,
             "oom_kill": 0
         },
-        "slab": {
-            "task_struct": {
+        "slab": [
+            {
                 "name": "task_group",
                 "active_objs": 13000,
                 "num_objs": 14000,
@@ -140,7 +140,7 @@ pub const SAMPLE_MODEL_JSON: &str = r#"
                 "obj_per_slab": 5,
                 "num_slabs": 3000
             },
-            "vmap_area": {
+            {
                 "name": "vmap_area",
                 "active_objs": 4000000,
                 "num_objs": 6000000,
@@ -148,6 +148,32 @@ pub const SAMPLE_MODEL_JSON: &str = r#"
                 "obj_per_slab": 64,
                 "num_slabs": 100000
             }
+        ],
+        "ksm": {
+            "advisor_max_cpu": 70,
+            "advisor_max_pages_to_scan": 30000,
+            "advisor_min_pages_to_scan": 500,
+            "advisor_mode": "scan-time",
+            "advisor_target_scan_time": 200,
+            "full_scans": 25,
+            "general_profit": 0,
+            "ksm_zero_pages": 0,
+            "max_page_sharing": 256,
+            "merge_across_nodes": 1,
+            "pages_scanned": 5149,
+            "pages_shared": 0,
+            "pages_sharing": 0,
+            "pages_skipped": 25,
+            "pages_to_scan": 100,
+            "pages_unshared": 0,
+            "pages_volatile": 0,
+            "run": 1,
+            "sleep_millisecs": 20,
+            "smart_scan": 1,
+            "stable_node_chains": 1,
+            "stable_node_chains_prune_millisecs": 2000,
+            "stable_node_dups": 0,
+            "use_zero_pages": 0
         },
         "disks": {
             "vda": {
@@ -216,6 +242,8 @@ pub const SAMPLE_MODEL_JSON: &str = r#"
                 "memory_max": null,
                 "memory_swap_max": null,
                 "memory_zswap_max": null,
+                "memory_oom_group": null,
+                "memory_zswap_writeback": null,
                 "cpu_weight": null,
                 "cpu_max_usec": null,
                 "cpu_max_period_usec": null,
@@ -301,6 +329,8 @@ pub const SAMPLE_MODEL_JSON: &str = r#"
                         "memory_max": -1,
                         "memory_swap_max": -1,
                         "memory_zswap_max": -1,
+                        "memory_oom_group": 1,
+                        "memory_zswap_writeback": 1,
                         "cpu_weight": 100,
                         "cpu_max_usec": -1,
                         "cpu_max_period_usec": 100000,
@@ -395,7 +425,9 @@ pub const SAMPLE_MODEL_JSON: &str = r#"
                         "memory_high": -1,
                         "memory_max": -1,
                         "memory_swap_max": 0,
+                        "memory_oom_group": 0,
                         "memory_zswap_max": 0,
+                        "memory_zswap_writeback": 1,
                         "cpu_weight": null,
                         "cpu_max_usec": null,
                         "cpu_max_period_usec": null,
@@ -482,7 +514,9 @@ pub const SAMPLE_MODEL_JSON: &str = r#"
                         "memory_high": null,
                         "memory_max": null,
                         "memory_swap_max": null,
+                        "memory_oom_group": 0,
                         "memory_zswap_max": null,
+                        "memory_zswap_writeback": 1,
                         "cpu_weight": 100,
                         "cpu_max_usec": 1500000,
                         "cpu_max_period_usec": 100000,
@@ -575,6 +609,8 @@ pub const SAMPLE_MODEL_JSON: &str = r#"
                                 "memory_max": null,
                                 "memory_swap_max": null,
                                 "memory_zswap_max": null,
+                                "memory_oom_group": 1,
+                                "memory_zswap_writeback": 0,
                                 "cpu_weight": 100,
                                 "cpu_max_usec": 1200000,
                                 "cpu_max_period_usec": 100000,
@@ -926,6 +962,49 @@ pub const SAMPLE_MODEL_JSON: &str = r#"
             "in_csum_errors": 0,
             "ignored_multi": 0
         }
+    },
+    "tc": {
+        "tc": [
+            {
+                "interface": "eth0",
+                "kind": "fq_codel",
+                "qlen": 42,
+                "bps": 420,
+                "pps": 1337,
+                "bytes_per_sec": 299792458,
+                "packets_per_sec": 314,
+                "backlog_per_sec": 2718281828,
+                "drops_per_sec": 8675309,
+                "requeues_per_sec": 12345,
+                "overlimits_per_sec": 314159,
+                "qdisc": {
+                    "fq_codel": {
+                        "target": 2701,
+                        "limit": 7,
+                        "interval": 3,
+                        "ecn": 6,
+                        "quantum": 42,
+                        "ce_threshold": 101,
+                        "drop_batch_size": 9000,
+                        "memory_limit": 123456,
+                        "flows": 1024
+                    }
+                },
+                "xstats": {
+                    "fq_codel": {
+                        "maxpacket": 8675309,
+                        "ecn_mark": 299792458,
+                        "new_flows_len": 314,
+                        "old_flows_len": 1729,
+                        "ce_mark": 42,
+                        "drop_overlimit_per_sec": 420,
+                        "new_flow_count_per_sec": 1337,
+                        "memory_usage_per_sec": 2718281828,
+                        "drop_overmemory_per_sec": 27182
+                    }
+                }
+            }
+        ]
     }
 }
 "#;
