@@ -14,15 +14,7 @@
 
 use super::*;
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct SystemModel {
     pub hostname: String,
     pub kernel_version: Option<String>,
@@ -39,7 +31,9 @@ pub struct SystemModel {
     #[queriable(subquery)]
     pub vm: VmModel,
     #[queriable(subquery)]
-    pub slab: BTreeMap<String, SingleSlabModel>,
+    pub slab: Vec<SingleSlabModel>,
+    #[queriable(subquery)]
+    pub ksm: Option<KsmModel>,
     #[queriable(subquery)]
     pub disks: BTreeMap<String, SingleDiskModel>,
     #[queriable(subquery)]
@@ -53,7 +47,7 @@ impl SystemModel {
             last.and_then(|(last, _)| last.stat.total_cpu.as_ref()),
             sample.stat.total_cpu.as_ref(),
         ) {
-            (Some(prev), Some(curr)) => SingleCpuModel::new(-1, &prev, &curr),
+            (Some(prev), Some(curr)) => SingleCpuModel::new(-1, prev, curr),
             _ => Default::default(),
         };
 
@@ -82,17 +76,17 @@ impl SystemModel {
             .unwrap_or_default();
 
         let mut slab = sample
-            .slabinfo
+            .slabinfo_vec
             .iter()
-            .map(|(name, slab_info)| (name.to_owned(), SingleSlabModel::new(slab_info)))
-            .collect::<BTreeMap<String, SingleSlabModel>>();
+            .map(SingleSlabModel::new)
+            .collect::<Vec<SingleSlabModel>>();
 
         let slab_total = slab.iter().fold(
             SingleSlabModel {
                 name: Some(String::from("TOTAL")),
                 ..Default::default()
             },
-            |mut acc, (_, slabinfo)| {
+            |mut acc, slabinfo| {
                 acc.active_objs = opt_add(acc.active_objs, slabinfo.active_objs);
                 acc.num_objs = opt_add(acc.num_objs, slabinfo.num_objs);
                 acc.num_slabs = opt_add(acc.num_slabs, slabinfo.num_slabs);
@@ -103,7 +97,9 @@ impl SystemModel {
                 acc
             },
         );
-        slab.insert(String::from("TOTAL"), slab_total);
+        slab.insert(0, slab_total);
+
+        let ksm = sample.ksm.as_ref().map(KsmModel::new);
 
         let mut disks: BTreeMap<String, SingleDiskModel> = BTreeMap::new();
         sample.disks.iter().for_each(|(disk_name, end_disk_stat)| {
@@ -113,7 +109,7 @@ impl SystemModel {
                     Some((last_sample, duration)) if last_sample.disks.contains_key(disk_name) => {
                         SingleDiskModel::new(
                             last_sample.disks.get(disk_name).unwrap(),
-                            &end_disk_stat,
+                            end_disk_stat,
                             duration,
                         )
                     }
@@ -149,6 +145,7 @@ impl SystemModel {
             mem,
             vm,
             slab,
+            ksm,
             disks,
             btrfs,
         }
@@ -161,15 +158,7 @@ impl Nameable for SystemModel {
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct ProcStatModel {
     pub total_interrupt_ct: Option<u64>,
     pub context_switches: Option<u64>,
@@ -192,15 +181,7 @@ impl ProcStatModel {
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct SingleCpuModel {
     pub idx: i32,
     pub usage_pct: Option<f64>,
@@ -268,7 +249,7 @@ impl SingleCpuModel {
                     system_pct: Some(system_usec as f64 * 100.0 / total_usec as f64),
                     nice_pct: Some(nice_usec as f64 * 100.0 / total_usec as f64),
                     iowait_pct: Some(iowait_usec as f64 * 100.0 / total_usec as f64),
-                    irq_pct: Some(iowait_usec as f64 * 100.0 / total_usec as f64),
+                    irq_pct: Some(irq_usec as f64 * 100.0 / total_usec as f64),
                     softirq_pct: Some(softirq_usec as f64 * 100.0 / total_usec as f64),
                     stolen_pct: Some(stolen_usec as f64 * 100.0 / total_usec as f64),
                     guest_pct: Some(guest_usec as f64 * 100.0 / total_usec as f64),
@@ -283,15 +264,7 @@ impl SingleCpuModel {
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct MemoryModel {
     pub total: Option<u64>,
     pub free: Option<u64>,
@@ -335,56 +308,48 @@ pub struct MemoryModel {
 impl MemoryModel {
     fn new(meminfo: &procfs::MemInfo) -> MemoryModel {
         MemoryModel {
-            total: meminfo.total.map(|v| v as u64),
-            free: meminfo.free.map(|v| v as u64),
-            available: meminfo.available.map(|v| v as u64),
-            buffers: meminfo.buffers.map(|v| v as u64),
-            cached: meminfo.cached.map(|v| v as u64),
-            swap_cached: meminfo.swap_cached.map(|v| v as u64),
-            active: meminfo.active.map(|v| v as u64),
-            inactive: meminfo.inactive.map(|v| v as u64),
-            anon: opt_add(meminfo.active_anon, meminfo.inactive_anon).map(|v| v as u64),
-            file: opt_add(meminfo.active_file, meminfo.inactive_file).map(|v| v as u64),
-            unevictable: meminfo.unevictable.map(|v| v as u64),
-            mlocked: meminfo.mlocked.map(|v| v as u64),
-            swap_total: meminfo.swap_total.map(|v| v as u64),
-            swap_free: meminfo.swap_free.map(|v| v as u64),
-            dirty: meminfo.dirty.map(|v| v as u64),
-            writeback: meminfo.writeback.map(|v| v as u64),
-            anon_pages: meminfo.anon_pages.map(|v| v as u64),
-            mapped: meminfo.mapped.map(|v| v as u64),
-            shmem: meminfo.shmem.map(|v| v as u64),
-            kreclaimable: meminfo.kreclaimable.map(|v| v as u64),
-            slab: meminfo.slab.map(|v| v as u64),
-            slab_reclaimable: meminfo.slab_reclaimable.map(|v| v as u64),
-            slab_unreclaimable: meminfo.slab_unreclaimable.map(|v| v as u64),
-            kernel_stack: meminfo.kernel_stack.map(|v| v as u64),
-            page_tables: meminfo.page_tables.map(|v| v as u64),
-            anon_huge_pages_bytes: meminfo.anon_huge_pages.map(|x| x as u64),
-            shmem_huge_pages_bytes: meminfo.shmem_huge_pages.map(|x| x as u64),
-            file_huge_pages_bytes: meminfo.file_huge_pages.map(|x| x as u64),
-            hugetlb: meminfo.hugetlb.map(|x| x as u64),
-            cma_total: meminfo.cma_total.map(|v| v as u64),
-            cma_free: meminfo.cma_free.map(|v| v as u64),
-            vmalloc_total: meminfo.vmalloc_total.map(|v| v as u64),
-            vmalloc_used: meminfo.vmalloc_used.map(|v| v as u64),
-            vmalloc_chunk: meminfo.vmalloc_chunk.map(|v| v as u64),
-            direct_map_4k: meminfo.direct_map_4k.map(|v| v as u64),
-            direct_map_2m: meminfo.direct_map_2m.map(|v| v as u64),
-            direct_map_1g: meminfo.direct_map_1g.map(|v| v as u64),
+            total: meminfo.total,
+            free: meminfo.free,
+            available: meminfo.available,
+            buffers: meminfo.buffers,
+            cached: meminfo.cached,
+            swap_cached: meminfo.swap_cached,
+            active: meminfo.active,
+            inactive: meminfo.inactive,
+            anon: opt_add(meminfo.active_anon, meminfo.inactive_anon),
+            file: opt_add(meminfo.active_file, meminfo.inactive_file),
+            unevictable: meminfo.unevictable,
+            mlocked: meminfo.mlocked,
+            swap_total: meminfo.swap_total,
+            swap_free: meminfo.swap_free,
+            dirty: meminfo.dirty,
+            writeback: meminfo.writeback,
+            anon_pages: meminfo.anon_pages,
+            mapped: meminfo.mapped,
+            shmem: meminfo.shmem,
+            kreclaimable: meminfo.kreclaimable,
+            slab: meminfo.slab,
+            slab_reclaimable: meminfo.slab_reclaimable,
+            slab_unreclaimable: meminfo.slab_unreclaimable,
+            kernel_stack: meminfo.kernel_stack,
+            page_tables: meminfo.page_tables,
+            anon_huge_pages_bytes: meminfo.anon_huge_pages,
+            shmem_huge_pages_bytes: meminfo.shmem_huge_pages,
+            file_huge_pages_bytes: meminfo.file_huge_pages,
+            hugetlb: meminfo.hugetlb,
+            cma_total: meminfo.cma_total,
+            cma_free: meminfo.cma_free,
+            vmalloc_total: meminfo.vmalloc_total,
+            vmalloc_used: meminfo.vmalloc_used,
+            vmalloc_chunk: meminfo.vmalloc_chunk,
+            direct_map_4k: meminfo.direct_map_4k,
+            direct_map_2m: meminfo.direct_map_2m,
+            direct_map_1g: meminfo.direct_map_1g,
         }
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct VmModel {
     pub pgpgin_per_sec: Option<f64>,
     pub pgpgout_per_sec: Option<f64>,
@@ -408,20 +373,12 @@ impl VmModel {
             pgsteal_direct: count_per_sec!(begin.pgsteal_direct, end.pgsteal_direct, duration, u64),
             pgscan_kswapd: count_per_sec!(begin.pgscan_kswapd, end.pgscan_kswapd, duration, u64),
             pgscan_direct: count_per_sec!(begin.pgscan_direct, end.pgscan_direct, duration, u64),
-            oom_kill: end.oom_kill.map(|v| v as u64),
+            oom_kill: end.oom_kill,
         }
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct SingleSlabModel {
     pub name: Option<String>,
     pub active_objs: Option<u64>,
@@ -456,15 +413,66 @@ impl SingleSlabModel {
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
+pub struct KsmModel {
+    pub advisor_max_cpu: Option<u64>,
+    pub advisor_max_pages_to_scan: Option<u64>,
+    pub advisor_min_pages_to_scan: Option<u64>,
+    pub advisor_mode: Option<String>,
+    pub advisor_target_scan_time: Option<u64>,
+    pub full_scans: Option<u64>,
+    pub general_profit: Option<i64>,
+    pub ksm_zero_pages: Option<i64>,
+    pub max_page_sharing: Option<u64>,
+    pub merge_across_nodes: Option<u64>,
+    pub pages_scanned: Option<u64>,
+    pub pages_shared: Option<u64>,
+    pub pages_sharing: Option<u64>,
+    pub pages_skipped: Option<u64>,
+    pub pages_to_scan: Option<u64>,
+    pub pages_unshared: Option<u64>,
+    pub pages_volatile: Option<u64>,
+    pub run: Option<u64>,
+    pub sleep_millisecs: Option<u64>,
+    pub smart_scan: Option<u64>,
+    pub stable_node_chains: Option<u64>,
+    pub stable_node_chains_prune_millisecs: Option<u64>,
+    pub stable_node_dups: Option<u64>,
+    pub use_zero_pages: Option<u64>,
+}
+
+impl KsmModel {
+    fn new(ksm: &procfs::Ksm) -> KsmModel {
+        KsmModel {
+            advisor_max_cpu: ksm.advisor_max_cpu,
+            advisor_max_pages_to_scan: ksm.advisor_max_pages_to_scan,
+            advisor_min_pages_to_scan: ksm.advisor_min_pages_to_scan,
+            advisor_mode: ksm.advisor_mode.clone(),
+            advisor_target_scan_time: ksm.advisor_target_scan_time,
+            full_scans: ksm.full_scans,
+            general_profit: ksm.general_profit,
+            ksm_zero_pages: ksm.ksm_zero_pages,
+            max_page_sharing: ksm.max_page_sharing,
+            merge_across_nodes: ksm.merge_across_nodes,
+            pages_scanned: ksm.pages_scanned,
+            pages_shared: ksm.pages_shared,
+            pages_sharing: ksm.pages_sharing,
+            pages_skipped: ksm.pages_skipped,
+            pages_to_scan: ksm.pages_to_scan,
+            pages_unshared: ksm.pages_unshared,
+            pages_volatile: ksm.pages_volatile,
+            run: ksm.run,
+            sleep_millisecs: ksm.sleep_millisecs,
+            smart_scan: ksm.smart_scan,
+            stable_node_chains: ksm.stable_node_chains,
+            stable_node_chains_prune_millisecs: ksm.stable_node_chains_prune_millisecs,
+            stable_node_dups: ksm.stable_node_dups,
+            use_zero_pages: ksm.use_zero_pages,
+        }
+    }
+}
+
+#[::below_derive::queriable_derives]
 pub struct SingleDiskModel {
     pub name: Option<String>,
     pub disk_usage: Option<f32>,
@@ -508,8 +516,8 @@ impl SingleDiskModel {
             count_per_sec!(begin.write_sectors, end.write_sectors, duration).map(|val| val * 512.0);
         SingleDiskModel {
             name: end.name.clone(),
-            disk_usage: end.disk_usage.map(|v| v as f32),
-            partition_size: end.partition_size.map(|v| v as u64),
+            disk_usage: end.disk_usage,
+            partition_size: end.partition_size,
             filesystem_type: end.filesystem_type.clone(),
             read_bytes_per_sec,
             write_bytes_per_sec,
@@ -520,20 +528,20 @@ impl SingleDiskModel {
             )
             .map(|val| val * 512.0),
             disk_total_bytes_per_sec: opt_add(read_bytes_per_sec, write_bytes_per_sec),
-            read_completed: end.read_completed.map(|v| v as u64),
-            read_merged: end.read_merged.map(|v| v as u64),
-            read_sectors: end.read_sectors.map(|v| v as u64),
-            time_spend_read_ms: end.time_spend_read_ms.map(|v| v as u64),
-            write_completed: end.write_completed.map(|v| v as u64),
-            write_merged: end.write_merged.map(|v| v as u64),
-            write_sectors: end.write_sectors.map(|v| v as u64),
-            time_spend_write_ms: end.time_spend_write_ms.map(|v| v as u64),
-            discard_completed: end.discard_completed.map(|v| v as u64),
-            discard_merged: end.discard_merged.map(|v| v as u64),
-            discard_sectors: end.discard_sectors.map(|v| v as u64),
-            time_spend_discard_ms: end.time_spend_discard_ms.map(|v| v as u64),
-            major: end.major.map(|v| v as u64),
-            minor: end.minor.map(|v| v as u64),
+            read_completed: end.read_completed,
+            read_merged: end.read_merged,
+            read_sectors: end.read_sectors,
+            time_spend_read_ms: end.time_spend_read_ms,
+            write_completed: end.write_completed,
+            write_merged: end.write_merged,
+            write_sectors: end.write_sectors,
+            time_spend_write_ms: end.time_spend_write_ms,
+            discard_completed: end.discard_completed,
+            discard_merged: end.discard_merged,
+            discard_sectors: end.discard_sectors,
+            time_spend_discard_ms: end.time_spend_discard_ms,
+            major: end.major,
+            minor: end.minor,
         }
     }
 }
@@ -544,15 +552,7 @@ impl Nameable for SingleDiskModel {
     }
 }
 
-#[derive(
-    Clone,
-    Debug,
-    Default,
-    PartialEq,
-    Serialize,
-    Deserialize,
-    below_derive::Queriable
-)]
+#[::below_derive::queriable_derives]
 pub struct BtrfsModel {
     pub name: Option<String>,
     pub disk_fraction: Option<f64>,
@@ -563,8 +563,8 @@ impl BtrfsModel {
     fn new(end: &btrfs::BtrfsStat) -> BtrfsModel {
         BtrfsModel {
             name: end.name.clone(),
-            disk_fraction: end.disk_fraction.map(|v| v as f64),
-            disk_bytes: end.disk_bytes.map(|v| v as u64),
+            disk_fraction: end.disk_fraction,
+            disk_bytes: end.disk_bytes,
         }
     }
 }
@@ -591,7 +591,8 @@ mod test {
             "cpus": {},
             "mem": {},
             "vm": {},
-            "slab": {},
+            "slab": [],
+            "ksm": {},
             "disks": {
                 "sda": {
                     "name": "sda",
