@@ -13,6 +13,8 @@
 // limitations under the License.
 
 #![deny(clippy::all)]
+use std::cell::RefCell;
+use std::cell::RefMut;
 use std::collections::BTreeMap;
 use std::collections::HashMap;
 use std::fs::File;
@@ -20,15 +22,21 @@ use std::io::BufRead;
 use std::io::BufReader;
 use std::io::ErrorKind;
 use std::io::Read;
+use std::os::unix::ffi::OsStrExt;
 use std::path::Path;
 use std::path::PathBuf;
-use std::sync::mpsc::channel;
-use std::sync::mpsc::RecvTimeoutError;
+use std::str::FromStr;
+use std::sync::Arc;
 use std::time::Duration;
 
 use lazy_static::lazy_static;
+use libc::CLOCK_BOOTTIME;
+use libc::clock_gettime;
+use libc::timespec;
 use nix::sys;
 use openat::Dir;
+use parking_lot::Condvar;
+use parking_lot::Mutex;
 use slog::debug;
 use thiserror::Error;
 use threadpool::ThreadPool;
@@ -39,6 +47,9 @@ pub use types::*;
 #[cfg(test)]
 mod test;
 
+use common::util;
+
+pub const KSM_SYSFS: &str = "/sys/kernel/mm/ksm";
 pub const NET_SYSFS: &str = "/sys/class/net/";
 pub const NET_PROCFS: &str = "/proc/net";
 
@@ -91,6 +102,8 @@ pub enum Error {
     },
     #[error("Unexpected line ({1}) in file: {0:?}")]
     UnexpectedLine(PathBuf, String),
+    #[error("Error when reading uptime")]
+    UptimeError,
 }
 
 pub type Result<T> = std::result::Result<T, Error>;
@@ -116,7 +129,7 @@ macro_rules! parse_item {
     // Parse the second item of $line as type $t with the same
     // semantics as above
     ($path: expr, $line:ident, $t:tt) => {{
-        let mut items = $line.split_whitespace();
+        let mut items = $line.split_ascii_whitespace();
 
         // Advance past the name
         items.next();
@@ -146,6 +159,13 @@ macro_rules! parse_kb {
 pub struct ProcReader {
     path: PathBuf,
     threadpool: ThreadPool,
+    buffer: RefCell<Vec<u8>>,
+}
+
+impl Default for ProcReader {
+    fn default() -> Self {
+        Self::new()
+    }
 }
 
 impl ProcReader {
@@ -154,6 +174,7 @@ impl ProcReader {
             path: Path::new("/proc").to_path_buf(),
             // 5 threads max
             threadpool: ThreadPool::with_name("procreader_worker".to_string(), 5),
+            buffer: RefCell::new(Vec::new()),
         }
     }
 
@@ -163,27 +184,26 @@ impl ProcReader {
         reader
     }
 
+    fn read_file_to_str(&self, path: &Path) -> Result<RefMut<'_, str>> {
+        File::open(path)
+            .and_then(|file| util::read_kern_file_to_internal_buffer(&self.buffer, file))
+            .map_err(|e| Error::IoError(path.to_path_buf(), e))
+    }
+
     fn read_uptime_secs(&self) -> Result<u64> {
-        let path = self.path.join("uptime");
-        let file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let mut buf_reader = BufReader::new(file);
-        let mut line = String::new();
-        buf_reader
-            .read_line(&mut line)
-            .map_err(|e| Error::IoError(path.clone(), e))?;
-
-        let mut items = line.split_whitespace();
-
-        match parse_item!(path, items.next(), f64, line) {
-            Ok(Some(uptime)) => Ok(uptime.round() as u64),
-            Ok(None) => Err(Error::InvalidFileFormat(path)),
-            Err(e) => Err(e),
+        unsafe {
+            let mut ts: timespec = std::mem::zeroed();
+            if clock_gettime(CLOCK_BOOTTIME, &mut ts) == 0 {
+                Ok(ts.tv_sec as u64)
+            } else {
+                Err(Error::UptimeError)
+            }
         }
     }
 
-    fn process_cpu_stat(path: &PathBuf, line: &String) -> Result<CpuStat> {
+    fn process_cpu_stat(path: &Path, line: &str) -> Result<CpuStat> {
         // Format is like "cpu9 6124418 452468 3062529 230073290 216237 0 45647 0 0 0"
-        let mut items = line.split_whitespace();
+        let mut items = line.split_ascii_whitespace();
         let mut cpu: CpuStat = Default::default();
 
         // Advance past "cpu*" item
@@ -205,22 +225,18 @@ impl ProcReader {
 
     pub fn read_kernel_version(&self) -> Result<String> {
         let path = self.path.join("sys/kernel/osrelease");
-        match std::fs::read_to_string(&path) {
-            Ok(kernel_version) => Ok(kernel_version.trim_matches('\n').trim().into()),
-            Err(e) => Err(Error::IoError(path, e)),
-        }
+        let content = self.read_file_to_str(&path)?;
+        Ok(content.trim_matches('\n').trim().into())
     }
 
     pub fn read_stat(&self) -> Result<Stat> {
         let path = self.path.join("stat");
-        let file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let buf_reader = BufReader::new(file);
+        let content = self.read_file_to_str(&path)?;
         let mut stat: Stat = Default::default();
         let mut cpus_map = BTreeMap::new();
-        for line in buf_reader.lines() {
-            let line = line.map_err(|e| Error::IoError(path.clone(), e))?;
 
-            let mut items = line.split_whitespace();
+        for line in content.lines() {
+            let mut items = line.split_ascii_whitespace();
             if let Some(item) = items.next() {
                 match item {
                     "intr" => {
@@ -241,15 +257,15 @@ impl ProcReader {
                     }
                     x => {
                         if x == "cpu" {
-                            stat.total_cpu = Some(Self::process_cpu_stat(&path, &line)?);
+                            stat.total_cpu = Some(Self::process_cpu_stat(&path, line)?);
                         } else if let Some(cpu_suffix) = x.strip_prefix("cpu") {
                             let cpu_id =
                                 parse_item!(&path, Some(cpu_suffix.to_owned()), u32, line)?
                                     .unwrap();
                             let existing =
-                                cpus_map.insert(cpu_id, Self::process_cpu_stat(&path, &line)?);
+                                cpus_map.insert(cpu_id, Self::process_cpu_stat(&path, line)?);
                             if existing.is_some() {
-                                return Err(Error::UnexpectedLine(path, line));
+                                return Err(Error::UnexpectedLine(path, line.to_string()));
                             }
                         }
                     }
@@ -269,14 +285,11 @@ impl ProcReader {
 
     pub fn read_meminfo(&self) -> Result<MemInfo> {
         let path = self.path.join("meminfo");
-        let file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let buf_reader = BufReader::new(file);
+        let content = self.read_file_to_str(&path)?;
         let mut meminfo: MemInfo = Default::default();
 
-        for line in buf_reader.lines() {
-            let line = line.map_err(|e| Error::IoError(path.clone(), e))?;
-
-            let mut items = line.split_whitespace();
+        for line in content.lines() {
+            let mut items = line.split_ascii_whitespace();
             if let Some(item) = items.next() {
                 match item {
                     "MemTotal:" => meminfo.total = parse_kb!(path, items.next(), line)?,
@@ -354,14 +367,11 @@ impl ProcReader {
 
     pub fn read_vmstat(&self) -> Result<VmStat> {
         let path = self.path.join("vmstat");
-        let file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let buf_reader = BufReader::new(file);
+        let content = self.read_file_to_str(&path)?;
         let mut vmstat: VmStat = Default::default();
 
-        for line in buf_reader.lines() {
-            let line = line.map_err(|e| Error::IoError(path.clone(), e))?;
-
-            let mut items = line.split_whitespace();
+        for line in content.lines() {
+            let mut items = line.split_ascii_whitespace();
             if let Some(item) = items.next() {
                 match item {
                     "pgpgin" => vmstat.pgpgin = parse_item!(path, items.next(), u64, line)?,
@@ -393,33 +403,31 @@ impl ProcReader {
         }
     }
 
-    pub fn read_slabinfo(&self) -> Result<SlabInfoMap> {
+    pub fn read_slabinfo(&self) -> Result<Vec<SlabInfo>> {
         let path = self.path.join("slabinfo");
-        let file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let buf_reader = BufReader::new(file);
-        let mut slab_info_map: SlabInfoMap = Default::default();
+        let content = self.read_file_to_str(&path)?;
+        let mut slab_info_vec = vec![];
 
         // The first line is version, second line is headers:
         //
         // slabinfo - version: 2.1
         // # name            <active_objs> <num_objs> <objsize> <objperslab> <pagesperslab> : tunables <limit> <batchcount> <sharedfactor> : slabdata <active_slabs> <num_slabs> <sharedavail>
         //
-        for line in buf_reader.lines().skip(2) {
-            let line = line.map_err(|e| Error::IoError(path.clone(), e))?;
-            let mut items = line.split_whitespace();
-            let mut slab_info: SlabInfo = Default::default();
-            let name = items.next().unwrap().to_owned();
-            slab_info.name = Some(name.clone());
-            slab_info.active_objs = parse_item!(path, items.next(), u64, line)?;
-            slab_info.num_objs = parse_item!(path, items.next(), u64, line)?;
-            slab_info.obj_size = parse_item!(path, items.next(), u64, line)?;
-            slab_info.obj_per_slab = parse_item!(path, items.next(), u64, line)?;
-            slab_info.pages_per_slab = parse_item!(path, items.next(), u64, line)?;
-            slab_info.active_slabs = parse_item!(path, items.nth(7), u64, line)?;
-            slab_info.num_slabs = parse_item!(path, items.next(), u64, line)?;
-            slab_info_map.insert(name, slab_info);
+        for line in content.lines().skip(2) {
+            let mut items = line.split_ascii_whitespace();
+            let slab_info = SlabInfo {
+                name: Some(items.next().unwrap().to_owned()),
+                active_objs: parse_item!(path, items.next(), u64, line)?,
+                num_objs: parse_item!(path, items.next(), u64, line)?,
+                obj_size: parse_item!(path, items.next(), u64, line)?,
+                obj_per_slab: parse_item!(path, items.next(), u64, line)?,
+                pages_per_slab: parse_item!(path, items.next(), u64, line)?,
+                active_slabs: parse_item!(path, items.nth(7), u64, line)?,
+                num_slabs: parse_item!(path, items.next(), u64, line)?,
+            };
+            slab_info_vec.push(slab_info);
         }
-        Ok(slab_info_map)
+        Ok(slab_info_vec)
     }
 
     fn read_disk_fsinfo(&self, mount_info: &MountInfo) -> Option<(f32, u64)> {
@@ -427,6 +435,7 @@ impl ProcReader {
             if let Ok(stat) = sys::statvfs::statvfs(Path::new(&mount_point)) {
                 let disk_usage = ((stat.blocks() - stat.blocks_available()) as f32 * 100.0)
                     / stat.blocks() as f32;
+                #[allow(clippy::unnecessary_cast)]
                 let partition_size = stat.blocks() as u64 * stat.block_size() as u64;
                 return Some((disk_usage, partition_size));
             }
@@ -439,13 +448,11 @@ impl ProcReader {
         // mount of each mount source. The first mount is what shows in
         // the 'df' command and reflects the usage of the device.
         let path = self.path.join("self/mountinfo");
-        let file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let buf_reader = BufReader::new(file);
+        let content = self.read_file_to_str(&path)?.to_string();
         let mut mount_info_map: HashMap<String, MountInfo> = HashMap::new();
 
-        for line in buf_reader.lines() {
-            let line = line.map_err(|e| Error::IoError(path.clone(), e))?;
-            if let Ok(mount_info) = self.process_mount_info(&path, &line) {
+        for line in content.lines() {
+            if let Ok(mount_info) = self.process_mount_info(&path, line) {
                 if let Some(mount_source) = mount_info.mount_source.clone() {
                     mount_info_map.entry(mount_source).or_insert(mount_info);
                 }
@@ -460,7 +467,7 @@ impl ProcReader {
     }
 
     fn process_mount_info(&self, path: &Path, line: &str) -> Result<MountInfo> {
-        let mut items = line.split_whitespace();
+        let mut items = line.split_ascii_whitespace();
         let mut mount_info = MountInfo {
             mnt_id: parse_item!(path, items.next(), i32, line)?,
             parent_mnt_id: parse_item!(path, items.next(), i32, line)?,
@@ -484,14 +491,11 @@ impl ProcReader {
 
     pub fn read_disk_stats_and_fsinfo(&self) -> Result<DiskMap> {
         let path = self.path.join("diskstats");
-        let file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let buf_reader = BufReader::new(file);
+        let content = self.read_file_to_str(&path)?.to_string();
         let mut disk_map: DiskMap = Default::default();
         let mount_info_map = self.read_mount_info_map().unwrap_or_default();
 
-        for line in buf_reader.lines() {
-            let line = line.map_err(|e| Error::IoError(path.clone(), e))?;
-
+        for line in content.lines() {
             let stats_vec: Vec<&str> = line.split(' ').filter(|item| !item.is_empty()).collect();
             let mut stats_iter = stats_vec.iter();
             let mut disk_stat = DiskStat {
@@ -527,7 +531,7 @@ impl ProcReader {
                     disk_stat.disk_usage = Some(disk_usage);
                     disk_stat.partition_size = Some(partition_size);
                 }
-                disk_stat.filesystem_type = mount_info.fs_type.clone();
+                disk_stat.filesystem_type.clone_from(&mount_info.fs_type)
             }
 
             disk_map.insert(disk_name, disk_stat);
@@ -542,15 +546,10 @@ impl ProcReader {
 
     fn read_pid_stat_from_path<P: AsRef<Path>>(&self, path: P) -> Result<PidStat> {
         let path = path.as_ref().join("stat");
-        let file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let mut buf_reader = BufReader::new(file);
+        let content = self.read_file_to_str(&path)?;
         let mut pidstat: PidStat = Default::default();
 
-        let mut line = String::new();
-        buf_reader
-            .read_line(&mut line)
-            .map_err(|e| Error::IoError(path.clone(), e))?;
-
+        let mut line = content.to_string();
         {
             let b_opt = line.find('(');
             let e_opt = line.rfind(')');
@@ -560,31 +559,39 @@ impl ProcReader {
             }
         }
 
-        let items: Vec<_> = line.split_whitespace().collect();
-        pidstat.pid = parse_item!(path, items.get(0), i32, line)?;
-        if let Some(c) = parse_item!(path, items.get(1), char, line)? {
-            if let Some(state) = PidState::from_char(c) {
-                pidstat.state = Some(state);
-            } else {
-                return Err(Error::InvalidPidState(path, c));
+        for (index, item) in line.split_ascii_whitespace().enumerate() {
+            match index {
+                0 => pidstat.pid = parse_item!(path, Some(item), i32, line)?,
+                1 => {
+                    if let Some(c) = parse_item!(path, Some(item), char, line)? {
+                        if let Some(state) = PidState::from_char(c) {
+                            pidstat.state = Some(state);
+                        } else {
+                            return Err(Error::InvalidPidState(path, c));
+                        }
+                    }
+                }
+                2 => pidstat.ppid = parse_item!(path, Some(item), i32, line)?,
+                3 => pidstat.pgrp = parse_item!(path, Some(item), i32, line)?,
+                4 => pidstat.session = parse_item!(path, Some(item), i32, line)?,
+                8 => pidstat.minflt = parse_item!(path, Some(item), u64, line)?,
+                10 => pidstat.majflt = parse_item!(path, Some(item), u64, line)?,
+                12 => pidstat.user_usecs = parse_usec!(path, Some(item), line)?,
+                13 => pidstat.system_usecs = parse_usec!(path, Some(item), line)?,
+                18 => pidstat.num_threads = parse_item!(path, Some(item), u64, line)?,
+                20 => {
+                    let uptime = self.read_uptime_secs()?;
+                    pidstat.running_secs = parse_sec!(path, Some(item), line)?
+                        .map(|running_secs_since_boot| (uptime - running_secs_since_boot));
+                }
+                22 => {
+                    pidstat.rss_bytes =
+                        parse_item!(path, Some(item), u64, line)?.map(|pages| pages * *PAGE_SIZE)
+                }
+                37 => pidstat.processor = parse_item!(path, Some(item), i32, line)?,
+                _ => {}
             }
         }
-        pidstat.ppid = parse_item!(path, items.get(2), i32, line)?;
-        pidstat.pgrp = parse_item!(path, items.get(3), i32, line)?;
-        pidstat.session = parse_item!(path, items.get(4), i32, line)?;
-        pidstat.minflt = parse_item!(path, items.get(8), u64, line)?;
-        pidstat.majflt = parse_item!(path, items.get(10), u64, line)?;
-        pidstat.user_usecs = parse_usec!(path, items.get(12), line)?;
-        pidstat.system_usecs = parse_usec!(path, items.get(13), line)?;
-        pidstat.num_threads = parse_item!(path, items.get(18), u64, line)?;
-
-        let uptime = self.read_uptime_secs()?;
-        pidstat.running_secs = parse_sec!(path, items.get(20), line)?
-            .map(|running_secs_since_boot| (uptime - running_secs_since_boot) as u64);
-
-        pidstat.rss_bytes =
-            parse_item!(path, items.get(22), u64, line)?.map(|pages| pages * *PAGE_SIZE);
-        pidstat.processor = parse_item!(path, items.get(37), i32, line)?;
 
         if pidstat == Default::default() {
             Err(Error::InvalidFileFormat(path))
@@ -606,17 +613,13 @@ impl ProcReader {
 
     fn read_pid_status_from_path<P: AsRef<Path>>(&self, path: P) -> Result<PidStatus> {
         let path = path.as_ref().join("status");
-
-        let file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let buf_reader = BufReader::new(file);
+        let content = self.read_file_to_str(&path)?;
         let mut pidstatus: PidStatus = Default::default();
 
-        for line in buf_reader.lines() {
-            let line = line.map_err(|e| Error::IoError(path.clone(), e))?;
-
+        for line in content.lines() {
             let mut items = line.split(':');
             if let Some(item) = items.next() {
-                let mut values = items.flat_map(|s| s.split_whitespace());
+                let mut values = items.flat_map(|s| s.split_ascii_whitespace());
                 match item {
                     "NStgid" => {
                         pidstatus.ns_tgid = Some(values.filter_map(|s| s.parse().ok()).collect());
@@ -642,16 +645,13 @@ impl ProcReader {
         self.read_pid_status_from_path(self.path.join(pid.to_string()))
     }
 
-    fn read_pid_io_from_path<P: AsRef<Path>>(path: P) -> Result<PidIo> {
+    fn read_pid_io_from_path<P: AsRef<Path>>(&self, path: P) -> Result<PidIo> {
         let path = path.as_ref().join("io");
-        let file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let buf_reader = BufReader::new(file);
+        let content = self.read_file_to_str(&path)?;
         let mut pidio: PidIo = Default::default();
 
-        for line in buf_reader.lines() {
-            let line = line.map_err(|e| Error::IoError(path.clone(), e))?;
-
-            let mut items = line.split_whitespace();
+        for line in content.lines() {
+            let mut items = line.split_ascii_whitespace();
             if let Some(item) = items.next() {
                 match item {
                     "read_bytes:" => pidio.rbytes = parse_item!(path, items.next(), u64, line)?,
@@ -669,30 +669,29 @@ impl ProcReader {
     }
 
     pub fn read_pid_io(&self, pid: u32) -> Result<PidIo> {
-        Self::read_pid_io_from_path(self.path.join(pid.to_string()))
+        self.read_pid_io_from_path(self.path.join(pid.to_string()))
     }
 
-    fn read_pid_cgroup_from_path<P: AsRef<Path>>(path: P) -> Result<String> {
+    fn read_pid_cgroup_from_path<P: AsRef<Path>>(&self, path: P) -> Result<String> {
         let path = path.as_ref().join("cgroup");
-        let file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let buf_reader = BufReader::new(file);
+        let content = self.read_file_to_str(&path)?;
 
         let mut cgroup_path = None;
-        for line in buf_reader.lines() {
-            let line = line.map_err(|e| Error::IoError(path.clone(), e))?;
+        for line in content.lines() {
             // Lines contain three colon separated fields:
             //   hierarchy-ID:controller-list:cgroup-path
             // A line starting with "0::" would be an entry for cgroup v2.
             // Otherwise, the line containing "pids" controller is what we want
             // for cgroup v1.
-            let parts: Vec<_> = line.splitn(3, ':').collect();
-            if parts.len() == 3 {
-                if parts[0] == "0" && parts[1] == "" {
-                    cgroup_path = Some(parts[2].to_owned());
-                    // cgroup v2 takes precedence
-                    break;
-                } else if parts[1].split(',').any(|c| c == "pids") {
-                    cgroup_path = Some(parts[2].to_owned());
+            let mut parts = line.splitn(3, ':');
+            if let (Some(hierarchy_id), Some(controller_list), Some(path)) =
+                (parts.next(), parts.next(), parts.next())
+            {
+                if hierarchy_id == "0" && controller_list.is_empty() {
+                    return Ok(path.to_owned());
+                } else if controller_list.split(',').any(|c| c == "pids") {
+                    // Not return, since if cgroup v2 is found it takes precedence
+                    cgroup_path = Some(path.to_owned());
                 }
             }
         }
@@ -700,67 +699,74 @@ impl ProcReader {
     }
 
     pub fn read_pid_cgroup(&self, pid: u32) -> Result<String> {
-        Self::read_pid_cgroup_from_path(self.path.join(pid.to_string()))
+        self.read_pid_cgroup_from_path(self.path.join(pid.to_string()))
     }
 
-    pub fn read_pid_cmdline(&mut self, pid: u32) -> Result<Option<Vec<String>>> {
+    pub fn read_pid_cmdline(&self, pid: u32) -> Result<Option<Vec<String>>> {
         self.read_pid_cmdline_from_path(self.path.join(pid.to_string()))
     }
 
     fn read_pid_cmdline_from_path_blocking<P: AsRef<Path>>(path: P) -> Result<Option<Vec<String>>> {
         let path = path.as_ref().join("cmdline");
         let mut file = File::open(&path).map_err(|e| Error::IoError(path.clone(), e))?;
-        let mut buf = Vec::new();
-        match file
-            .read_to_end(&mut buf)
-            .map_err(|e| Error::IoError(path.clone(), e))?
-        {
+        let mut buf = [0; 4096];
+        let bytes_read = file
+            .read(&mut buf)
+            .map_err(|e| Error::IoError(path.clone(), e))?;
+
+        if bytes_read == 0 {
             // It's a zombie process and those don't have cmdlines
-            0 => Ok(None),
-            _ => {
-                Ok(Some(
-                    buf
-                        // /proc/pid/cmdline is split by nul bytes
-                        .split(|c| *c == 0)
-                        .filter(|s| !s.is_empty())
-                        .map(|s| {
-                            // Choose not to error on invalid utf8 b/c it's a process's
-                            // right to do crazy things if they want. No need for us to
-                            // erorr on it.
-                            String::from_utf8_lossy(s).to_string()
-                        })
-                        .collect::<Vec<String>>(),
-                ))
-            }
+            return Ok(None);
         }
+
+        Ok(Some(
+            buf[..bytes_read]
+                // /proc/pid/cmdline is split by nul bytes
+                .split(|c| *c == 0)
+                .filter(|s| !s.is_empty())
+                .map(|s| {
+                    // It's a process' right to put invalid UTF8 here
+                    String::from_utf8_lossy(s).to_string()
+                })
+                .collect::<Vec<String>>(),
+        ))
     }
 
     /// Do /proc/pid/cmdline reads off-thread in a threadpool b/c kernel needs
     /// to take the target process's mmap_sem semaphore and could block for a long
     /// time. This way, we don't suffer a priority inversion (b/c this crate can be
     /// run from a high priority binary).
-    fn read_pid_cmdline_from_path<P: AsRef<Path>>(
-        &mut self,
-        path: P,
-    ) -> Result<Option<Vec<String>>> {
+    fn read_pid_cmdline_from_path<P: AsRef<Path>>(&self, path: P) -> Result<Option<Vec<String>>> {
+        // If the current number of active workers is equal to or exceeds the maximum, we avoid
+        // enqueuing a new job to prevent unnecessary work since the task would likely time out.
+        // This check is inherently racy: we might erroneously assume the pool is full when it
+        // isn't, but not the reverse, because this is the only place where new tasks are scheduled
+        // synchronously.
+        if self.threadpool.active_count() >= self.threadpool.max_count() {
+            return Ok(None);
+        }
+
         let path = path.as_ref().to_owned();
-        let (tx, rx) = channel();
+        let cmdline_data = Arc::new((Mutex::new(None), Condvar::new()));
+        let cmdline_data_clone = Arc::clone(&cmdline_data);
+
         self.threadpool.execute(move || {
-            // This is OK to ignore. cmdline receiver hanging up is expected
-            // after timeout.
-            let _ = tx.send(Self::read_pid_cmdline_from_path_blocking(path));
+            let result = Self::read_pid_cmdline_from_path_blocking(path);
+            let (mutex, cvar) = &*cmdline_data_clone;
+            *mutex.lock() = Some(result);
+            cvar.notify_one();
         });
 
-        // 20ms should be more than enough for an in-memory procfs read and also high enough for a
-        // page fault
-        match rx.recv_timeout(Duration::from_millis(20)) {
-            Ok(c) => c,
-            Err(RecvTimeoutError::Timeout) => Ok(None),
-            Err(RecvTimeoutError::Disconnected) => panic!("cmdline sender hung up"),
+        let (mutex, cvar) = &*cmdline_data;
+        let mut data_lock = mutex.lock();
+        if data_lock.is_none() {
+            // 20ms should be more than enough for an in-memory procfs read or a page fault
+            cvar.wait_for(&mut data_lock, Duration::from_millis(20));
         }
+        data_lock.take().unwrap_or(Ok(None))
     }
 
-    fn read_pid_exe_path_from_path<P: AsRef<Path>>(path: P) -> Result<String> {
+    fn read_pid_exe_path_from_path<P: AsRef<Path>>(&self, path: P) -> Result<String> {
         let path = path.as_ref().join("exe");
         std::fs::read_link(path.clone())
             .map_err(|e| Error::IoError(path, e))
@@ -768,10 +774,23 @@ impl ProcReader {
     }
 
     pub fn read_pid_exe_path(&self, pid: u32) -> Result<String> {
-        Self::read_pid_exe_path_from_path(self.path.join(pid.to_string()))
+        self.read_pid_exe_path_from_path(self.path.join(pid.to_string()))
     }
 
-    pub fn read_all_pids(&mut self) -> Result<PidMap> {
+    fn ascii_digits_to_i32(digits: &[u8]) -> Option<i32> {
+        let mut result = 0i32;
+        for digit in digits {
+            let value = digit.wrapping_sub(b'0');
+            if value <= 9 {
+                result = result * 10 + value as i32;
+            } else {
+                return None;
+            }
+        }
+        Some(result)
+    }
+
+    pub fn read_all_pids(&self) -> Result<PidMap> {
         let mut pidmap: PidMap = Default::default();
         for entry in
             std::fs::read_dir(&self.path).map_err(|e| Error::IoError(self.path.clone(), e))?
@@ -785,22 +804,20 @@ impl ProcReader {
                 }
                 ent => ent.map_err(|e| Error::IoError(self.path.clone(), e))?,
             };
-            match entry.metadata() {
-                Ok(ref m) if m.is_dir() => {}
-                _ => continue,
-            }
 
-            let mut is_pid = true;
-            for c in entry.file_name().to_string_lossy().chars() {
-                if !c.is_ascii_digit() {
-                    is_pid = false;
-                    break;
-                }
-            }
-            if !is_pid {
+            if !entry
+                .file_type()
+                .map_err(|e| Error::IoError(self.path.clone(), e))?
+                .is_dir()
+            {
                 continue;
             }
 
+            let pid = match Self::ascii_digits_to_i32(entry.file_name().as_bytes()) {
+                Some(pid) => pid,
+                None => continue,
+            };
+
             let mut pidinfo: PidInfo = Default::default();
 
             match self.read_pid_stat_from_path(entry.path()) {
@@ -823,7 +840,7 @@ impl ProcReader {
                 res => pidinfo.status = res?,
             }
 
-            match Self::read_pid_io_from_path(entry.path()) {
+            match self.read_pid_io_from_path(entry.path()) {
                 Err(Error::IoError(_, ref e))
                     if e.raw_os_error().map_or(false, |ec| {
                         ec == 2 || ec == 3 /* ENOENT or ESRCH */
@@ -841,7 +858,7 @@ impl ProcReader {
                 res => pidinfo.io = res?,
             }
 
-            match Self::read_pid_cgroup_from_path(entry.path()) {
+            match self.read_pid_cgroup_from_path(entry.path()) {
                 Err(Error::IoError(_, ref e))
                     if e.raw_os_error()
                         .map_or(false, |ec| ec == 2 || ec == 3 /* ENOENT or ESRCH */) =>
@@ -864,18 +881,10 @@ impl ProcReader {
             // Swallow the error since reading the /proc/pid/exe
             // 1. will need root permission to trace link
             // 2. Even with root permission, some exe will have broken link, kworker for example.
-            if let Ok(s) = Self::read_pid_exe_path_from_path(entry.path()) {
+            if let Ok(s) = self.read_pid_exe_path_from_path(entry.path()) {
                 pidinfo.exe_path = Some(s);
             }
 
-            let file_name = entry.file_name();
-            let pid_str = file_name.to_string_lossy();
-            let pid = pid_str.parse::<i32>().map_err(|_| Error::ParseError {
-                line: String::new(),
-                item: pid_str.to_string(),
-                type_name: "pid".to_string(),
-                path: self.path.clone(),
-            })?;
             pidmap.insert(pid, pidinfo);
         }
 
@@ -967,7 +976,7 @@ impl NetReader {
 
     fn read_iface_stat(
         interface_dir: &Dir,
-        cur_path: &PathBuf,
+        cur_path: &Path,
         stat_item: &str,
     ) -> Result<Option<u64>> {
         let file = match interface_dir.open_file(stat_item) {
@@ -992,17 +1001,17 @@ impl NetReader {
         }
     }
 
-    fn read_all_iface_stats(&self, interface: &str, cur_path: &PathBuf) -> Result<InterfaceStat> {
+    fn read_all_iface_stats(&self, interface: &str, cur_path: &Path) -> Result<InterfaceStat> {
         let interface_dir = self
             .interface_dir
             .read_link(interface)
-            .map_err(|e| Error::IoError(cur_path.clone(), e))?;
+            .map_err(|e| Error::IoError(cur_path.to_path_buf(), e))?;
         let stats_dir = self
             .interface_dir
             .sub_dir(interface_dir.as_path())
             .map_err(|e| Error::IoError(interface_dir, e))?
             .sub_dir("statistics")
-            .map_err(|e| Error::IoError(cur_path.clone(), e))?;
+            .map_err(|e| Error::IoError(cur_path.to_path_buf(), e))?;
         let cur_path = cur_path.join(interface).join("statistics");
         let mut net_stat: InterfaceStat = Default::default();
         parse_interface_stats!(
@@ -1096,8 +1105,8 @@ impl NetReader {
             }
 
             let key_header = fields[0];
-            let keys: Vec<&str> = fields[1].split_whitespace().collect();
-            let vals: Vec<&str> = vals[1].split_whitespace().collect();
+            let keys: Vec<&str> = fields[1].split_ascii_whitespace().collect();
+            let vals: Vec<&str> = vals[1].split_ascii_whitespace().collect();
 
             if keys.is_empty() && keys.len() != vals.len() {
                 return Err(Error::InvalidFileFormat(cur_path));
@@ -1144,7 +1153,7 @@ impl NetReader {
                 _ => continue,
             };
 
-            let kv = line.split_whitespace().collect::<Vec<&str>>();
+            let kv = line.split_ascii_whitespace().collect::<Vec<&str>>();
             if kv.len() != 2 {
                 return Err(Error::InvalidFileFormat(cur_path));
             }
@@ -1363,6 +1372,80 @@ impl NetReader {
     }
 }
 
+pub struct KsmReader {
+    path: PathBuf,
+}
+
+impl Default for KsmReader {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
+impl KsmReader {
+    pub fn new() -> KsmReader {
+        KsmReader {
+            path: Path::new(KSM_SYSFS).to_path_buf(),
+        }
+    }
+
+    pub fn new_with_custom_path(path: PathBuf) -> KsmReader {
+        KsmReader { path }
+    }
+
+    pub fn read_ksm(&self) -> Ksm {
+        Ksm {
+            advisor_max_cpu: self.read("advisor_max_cpu"),
+            advisor_max_pages_to_scan: self.read("advisor_max_pages_to_scan"),
+            advisor_min_pages_to_scan: self.read("advisor_min_pages_to_scan"),
+            advisor_mode: self.read_selection("advisor_mode"),
+            advisor_target_scan_time: self.read("advisor_target_scan_time"),
+            full_scans: self.read("full_scans"),
+            general_profit: self.read("general_profit"),
+            ksm_zero_pages: self.read("ksm_zero_pages"),
+            max_page_sharing: self.read("max_page_sharing"),
+            merge_across_nodes: self.read("merge_across_nodes"),
+            pages_scanned: self.read("pages_scanned"),
+            pages_shared: self.read("pages_shared"),
+            pages_sharing: self.read("pages_sharing"),
+            pages_skipped: self.read("pages_skipped"),
+            pages_to_scan: self.read("pages_to_scan"),
+            pages_unshared: self.read("pages_unshared"),
+            pages_volatile: self.read("pages_volatile"),
+            run: self.read("run"),
+            sleep_millisecs: self.read("sleep_millisecs"),
+            smart_scan: self.read("smart_scan"),
+            stable_node_chains: self.read("stable_node_chains"),
+            stable_node_chains_prune_millisecs: self.read("stable_node_chains_prune_millisecs"),
+            stable_node_dups: self.read("stable_node_dups"),
+            use_zero_pages: self.read("use_zero_pages"),
+        }
+    }
+
+    fn read<F>(&self, name: &str) -> Option<F>
+    where
+        F: FromStr,
+    {
+        std::fs::read_to_string(self.path.join(name))
+            .ok()?
+            .trim()
+            .parse()
+            .ok()
+    }
+
+    // parses from string representing one selection out of some choices
+    // i.e. "one [two] three" -> returns "two"
+    fn read_selection(&self, name: &str) -> Option<String> {
+        let val: String = self.read(name)?;
+        let left_bracket_idx = val.find('[')?;
+        let right_bracket_idx = val.rfind(']')?;
+        if left_bracket_idx >= right_bracket_idx {
+            return None;
+        }
+        Some(val[left_bracket_idx + 1..right_bracket_idx].to_string())
+    }
+}
+
 /// Wraps the result into an `Option` if the result is not an error.
 /// If the error is of type `ENOENT`, it is returned as `Ok(None)`.
 /// Else, the error itself is returned.
