@@ -12,18 +12,40 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use model::SingleCgroupModelFieldId;
-use model::SingleProcessModelFieldId;
 use RenderFormat::Duration;
+use RenderFormat::DurationSec;
 use RenderFormat::MaxOrDuration;
 use RenderFormat::MaxOrReadableSize;
 use RenderFormat::PageReadableSize;
 use RenderFormat::Precision;
 use RenderFormat::ReadableSize;
 use RenderFormat::SectorReadableSize;
+use model::ProcessModelFieldId;
+use model::SingleCgroupModelFieldId;
+use model::SingleProcessModelFieldId;
 
 use super::*;
 
+impl HasRenderConfig for model::Model {
+    fn get_render_config_builder(field_id: &Self::FieldId) -> RenderConfigBuilder {
+        use model::ModelFieldId::*;
+        let rc = RenderConfigBuilder::new();
+        match field_id {
+            System(field_id) => model::SystemModel::get_render_config_builder(field_id),
+            Cgroup(field_id) => {
+                model::SingleCgroupModel::get_render_config_builder(&field_id.subquery_id.0)
+            }
+            Process(ProcessModelFieldId::Processes(field_id)) => {
+                model::SingleProcessModel::get_render_config_builder(&field_id.subquery_id.0)
+            }
+            Network(field_id) => model::NetworkModel::get_render_config_builder(field_id),
+            Gpu(_) => rc,
+            Resctrl(_) => rc,
+            Tc(_) => rc,
+        }
+    }
+}
+
 impl HasRenderConfig for model::SingleCgroupModel {
     fn get_render_config_builder(field_id: &Self::FieldId) -> RenderConfigBuilder {
         use model::SingleCgroupModelFieldId::*;
@@ -35,13 +57,13 @@ impl HasRenderConfig for model::SingleCgroupModel {
             Cpu(field_id) => model::CgroupCpuModel::get_render_config_builder(field_id),
             Io(field_id) => model::CgroupIoModel::get_render_config_builder(field_id),
             IoDetails(field_id) => {
-                model::CgroupIoModel::get_render_config_builder(&field_id.subquery_id)
+                model::CgroupIoModel::get_render_config_builder(&field_id.subquery_id.0)
             }
             Mem(field_id) => model::CgroupMemoryModel::get_render_config_builder(field_id),
             Pressure(field_id) => model::CgroupPressureModel::get_render_config_builder(field_id),
             CgroupStat(field_id) => model::CgroupStatModel::get_render_config_builder(field_id),
             MemNuma(field_id) => {
-                model::CgroupMemoryNumaModel::get_render_config_builder(&field_id.subquery_id)
+                model::CgroupMemoryNumaModel::get_render_config_builder(&field_id.subquery_id.0)
             }
             Props(field_id) => model::CgroupProperties::get_render_config_builder(field_id),
             Pids(field_id) => model::CgroupPidsModel::get_render_config_builder(field_id),
@@ -203,6 +225,11 @@ impl HasRenderConfigForDump for model::SingleCgroupModel {
                 EventsMax => None,
                 EventsOom => Some(counter),
                 EventsOomKill => Some(counter),
+                EventsLocalLow => None,
+                EventsLocalHigh => None,
+                EventsLocalMax => None,
+                EventsLocalOom => Some(counter),
+                EventsLocalOomKill => Some(counter),
                 Anon => Some(gauge.unit("bytes")),
                 File => Some(gauge.unit("bytes")),
                 Kernel => Some(gauge.unit("bytes")),
@@ -215,6 +242,7 @@ impl HasRenderConfigForDump for model::SingleCgroupModel {
                 FileMapped => Some(gauge.unit("bytes")),
                 FileDirty => Some(gauge.unit("bytes")),
                 FileWriteback => Some(gauge.unit("bytes")),
+                FileThp => Some(gauge.unit("bytes")),
                 AnonThp => Some(gauge.unit("bytes")),
                 InactiveAnon => Some(gauge.unit("bytes")),
                 ActiveAnon => Some(gauge.unit("bytes")),
@@ -318,11 +346,16 @@ impl HasRenderConfig for model::CgroupMemoryModel {
         match field_id {
             Total => rc.title("Mem").format(ReadableSize),
             Swap => rc.title("Mem Swap").format(ReadableSize),
-            EventsLow => rc.title("Events Low"),
-            EventsHigh => rc.title("Events High"),
-            EventsMax => rc.title("Events Max"),
-            EventsOom => rc.title("Events OOM"),
-            EventsOomKill => rc.title("Events Kill"),
+            EventsLow => rc.title("Events Low/s"),
+            EventsHigh => rc.title("Events High/s"),
+            EventsMax => rc.title("Events Max/s"),
+            EventsOom => rc.title("Events OOM/s"),
+            EventsOomKill => rc.title("Events Kill/s"),
+            EventsLocalLow => rc.title("Events Local Low/s"),
+            EventsLocalHigh => rc.title("Events Local High/s"),
+            EventsLocalMax => rc.title("Events Local Max/s"),
+            EventsLocalOom => rc.title("Events Local OOM/s"),
+            EventsLocalOomKill => rc.title("Events Local Kill/s"),
             Anon => rc.title("Anon").format(ReadableSize),
             File => rc.title("File").format(ReadableSize),
             Kernel => rc.title("Kernel").format(ReadableSize),
@@ -335,6 +368,7 @@ impl HasRenderConfig for model::CgroupMemoryModel {
             FileMapped => rc.title("File Mapped").format(ReadableSize),
             FileDirty => rc.title("File Dirty").format(ReadableSize),
             FileWriteback => rc.title("File WB").format(ReadableSize),
+            FileThp => rc.title("File THP").format(ReadableSize),
             AnonThp => rc.title("Anon THP").format(ReadableSize),
             InactiveAnon => rc.title("Inactive Anon").format(ReadableSize),
             ActiveAnon => rc.title("Active Anon").format(ReadableSize),
@@ -394,7 +428,7 @@ impl HasRenderConfig for model::NetworkModel {
         use model::NetworkModelFieldId::*;
         match field_id {
             Interfaces(field_id) => {
-                model::SingleNetModel::get_render_config_builder(&field_id.subquery_id)
+                model::SingleNetModel::get_render_config_builder(&field_id.subquery_id.0)
             }
             Tcp(field_id) => model::TcpModel::get_render_config_builder(field_id),
             Ip(field_id) => model::IpModel::get_render_config_builder(field_id),
@@ -726,8 +760,8 @@ impl HasRenderConfig for model::SingleNetModel {
         let rc = RenderConfigBuilder::new();
         match field_id {
             Interface => rc.title("Interface"),
-            RxBytesPerSec => rc.title("RX Bytes/s").format(ReadableSize),
-            TxBytesPerSec => rc.title("TX Bytes/s").format(ReadableSize),
+            RxBytesPerSec => rc.title("RX Bytes/s").format(ReadableSize).suffix("/s"),
+            TxBytesPerSec => rc.title("TX Bytes/s").format(ReadableSize).suffix("/s"),
             ThroughputPerSec => rc.title("I/O Bytes/s").format(ReadableSize),
             RxPacketsPerSec => rc.title("RX Pkts/s"),
             TxPacketsPerSec => rc.title("TX Pkts/s"),
@@ -812,7 +846,7 @@ impl HasRenderConfigForDump for model::SingleNetModel {
 impl HasRenderConfig for Vec<model::SingleQueueModel> {
     fn get_render_config_builder(field_id: &Self::FieldId) -> RenderConfigBuilder {
         let mut rc =
-            model::SingleQueueModel::get_render_config_builder(&field_id.subquery_id).get();
+            model::SingleQueueModel::get_render_config_builder(&field_id.subquery_id.0).get();
         rc.title = rc.title.map(|title| title.to_string());
         rc.into()
     }
@@ -827,7 +861,7 @@ impl HasRenderConfigForDump for Vec<model::SingleQueueModel> {
             .idx
             .expect("VecFieldId without index should not have render config");
         self.get(idx)
-            .map(|queue| queue.get_openmetrics_config_for_dump(&field_id.subquery_id))?
+            .map(|queue| queue.get_openmetrics_config_for_dump(&field_id.subquery_id.0))?
     }
 }
 
@@ -886,7 +920,7 @@ impl HasRenderConfig for model::SingleProcessModel {
             NsTgid => rc.title("NStgid").width(12),
             Comm => rc.title("Comm").width(30),
             State => rc.title("State"),
-            UptimeSecs => rc.title("Uptime(sec)"),
+            UptimeSecs => rc.title("Uptime(sec)").format(DurationSec),
             Cgroup => rc.title("Cgroup").width(50).fold(FoldOption::Name),
             Io(field_id) => model::ProcessIoModel::get_render_config_builder(field_id),
             Mem(field_id) => model::ProcessMemoryModel::get_render_config_builder(field_id),
@@ -971,6 +1005,7 @@ impl HasRenderConfigForDump for model::SingleProcessModel {
                 UserPct => Some(gauge.unit("percent")),
                 SystemPct => Some(gauge.unit("percent")),
                 NumThreads => Some(counter),
+                Processor => Some(counter),
             },
             // OpenMetrics does not support strings
             Cmdline => None,
@@ -1022,6 +1057,7 @@ impl HasRenderConfig for model::ProcessCpuModel {
             UserPct => rc.title("CPU User").format(Precision(2)).suffix("%"),
             SystemPct => rc.title("CPU System").format(Precision(2)).suffix("%"),
             NumThreads => rc.title("Threads"),
+            Processor => rc.title("Processor"),
         }
     }
 }
@@ -1042,12 +1078,15 @@ impl HasRenderConfig for model::SystemModel {
             Mem(field_id) => model::MemoryModel::get_render_config_builder(field_id),
             Vm(field_id) => model::VmModel::get_render_config_builder(field_id),
             Slab(field_id) => {
-                model::SingleSlabModel::get_render_config_builder(&field_id.subquery_id)
+                model::SingleSlabModel::get_render_config_builder(&field_id.subquery_id.0)
             }
+            Ksm(field_id) => model::KsmModel::get_render_config_builder(field_id),
             Disks(field_id) => {
-                model::SingleDiskModel::get_render_config_builder(&field_id.subquery_id)
+                model::SingleDiskModel::get_render_config_builder(&field_id.subquery_id.0)
+            }
+            Btrfs(field_id) => {
+                model::BtrfsModel::get_render_config_builder(&field_id.subquery_id.0)
             }
-            Btrfs(field_id) => model::BtrfsModel::get_render_config_builder(&field_id.subquery_id),
         }
     }
 }
@@ -1071,6 +1110,7 @@ impl HasRenderConfigForDump for model::SystemModel {
             Mem(field_id) => self.mem.get_openmetrics_config_for_dump(field_id),
             Vm(field_id) => self.vm.get_openmetrics_config_for_dump(field_id),
             Slab(_) => None,
+            Ksm(_) => None,
             // Same as with NetworkModel, we leave disk dumping to `disk` category
             Disks(_) => None,
             // Same as with above, we leave btrfs dumping to `btrfs` category
@@ -1159,12 +1199,13 @@ impl HasRenderConfigForDump for model::SingleCpuModel {
 
 impl HasRenderConfig for BTreeMap<u32, model::SingleCpuModel> {
     fn get_render_config_builder(field_id: &Self::FieldId) -> RenderConfigBuilder {
-        let mut rc = model::SingleCpuModel::get_render_config_builder(&field_id.subquery_id).get();
+        let mut rc =
+            model::SingleCpuModel::get_render_config_builder(&field_id.subquery_id.0).get();
         rc.title = rc.title.map(|title| {
             format!(
                 "CPU {} {}",
                 field_id
-                    .key
+                    .idx
                     .expect("BTreeMapFieldId without key should not have render config"),
                 title
             )
@@ -1179,10 +1220,10 @@ impl HasRenderConfigForDump for BTreeMap<u32, model::SingleCpuModel> {
         field_id: &Self::FieldId,
     ) -> Option<RenderOpenMetricsConfigBuilder> {
         let key = field_id
-            .key
+            .idx
             .expect("BTreeMapFieldId without key should not have render config");
         self.get(&key)
-            .map(|cpu| cpu.get_openmetrics_config_for_dump(&field_id.subquery_id))?
+            .map(|cpu| cpu.get_openmetrics_config_for_dump(&field_id.subquery_id.0))?
     }
 }
 
@@ -1358,6 +1399,74 @@ impl HasRenderConfigForDump for model::SingleSlabModel {
     }
 }
 
+impl HasRenderConfig for model::KsmModel {
+    fn get_render_config_builder(field_id: &Self::FieldId) -> RenderConfigBuilder {
+        use model::KsmModelFieldId::*;
+        let rc = RenderConfigBuilder::new();
+        match field_id {
+            AdvisorMaxCpu => rc.title("AdvisorMaxCpu"),
+            AdvisorMaxPagesToScan => rc.title("AdvisorMaxPagesToScan"),
+            AdvisorMinPagesToScan => rc.title("AdvisorMinPagesToScan"),
+            AdvisorMode => rc.title("AdvisorMode"),
+            AdvisorTargetScanTime => rc.title("AdvisorTargetScanTime"),
+            FullScans => rc.title("FullScans"),
+            GeneralProfit => rc.title("GeneralProfit"),
+            KsmZeroPages => rc.title("KsmZeroPages"),
+            MaxPageSharing => rc.title("MaxPageSharing"),
+            MergeAcrossNodes => rc.title("MergeAcrossNodes"),
+            PagesScanned => rc.title("PagesScanned"),
+            PagesShared => rc.title("PagesShared"),
+            PagesSharing => rc.title("PagesSharing"),
+            PagesSkipped => rc.title("PagesSkipped"),
+            PagesToScan => rc.title("PagesToScan"),
+            PagesUnshared => rc.title("PagesUnshared"),
+            PagesVolatile => rc.title("PagesVolatile"),
+            Run => rc.title("Run"),
+            SleepMillisecs => rc.title("SleepMillisecs"),
+            SmartScan => rc.title("SmartScan"),
+            StableNodeChains => rc.title("StableNodeChains"),
+            StableNodeChainsPruneMillisecs => rc.title("StableNodeChainsPruneMillisecs"),
+            StableNodeDups => rc.title("StableNodeDups"),
+            UseZeroPages => rc.title("UseZeroPages"),
+        }
+    }
+}
+
+impl HasRenderConfigForDump for model::KsmModel {
+    fn get_openmetrics_config_for_dump(
+        &self,
+        field_id: &Self::FieldId,
+    ) -> Option<RenderOpenMetricsConfigBuilder> {
+        use model::KsmModelFieldId::*;
+        match field_id {
+            AdvisorMaxCpu => Some(counter()),
+            AdvisorMaxPagesToScan => Some(counter()),
+            AdvisorMinPagesToScan => Some(counter()),
+            AdvisorMode => Some(counter()),
+            AdvisorTargetScanTime => Some(counter()),
+            FullScans => Some(counter()),
+            GeneralProfit => Some(counter()),
+            KsmZeroPages => Some(counter()),
+            MaxPageSharing => Some(counter()),
+            MergeAcrossNodes => Some(counter()),
+            PagesScanned => Some(counter()),
+            PagesShared => Some(counter()),
+            PagesSharing => Some(counter()),
+            PagesSkipped => Some(counter()),
+            PagesToScan => Some(counter()),
+            PagesUnshared => Some(counter()),
+            PagesVolatile => Some(counter()),
+            Run => Some(counter()),
+            SleepMillisecs => Some(counter()),
+            SmartScan => Some(counter()),
+            StableNodeChains => Some(counter()),
+            StableNodeChainsPruneMillisecs => Some(counter()),
+            StableNodeDups => Some(counter()),
+            UseZeroPages => Some(counter()),
+        }
+    }
+}
+
 impl HasRenderConfig for model::SingleDiskModel {
     fn get_render_config_builder(field_id: &Self::FieldId) -> RenderConfigBuilder {
         use model::SingleDiskModelFieldId::*;
@@ -1542,6 +1651,8 @@ impl HasRenderConfig for model::CgroupProperties {
             MemoryMax => rc.title("Mem Max").format(MaxOrReadableSize),
             MemorySwapMax => rc.title("Swap Max").format(MaxOrReadableSize),
             MemoryZswapMax => rc.title("Zswap Max").format(MaxOrReadableSize),
+            MemoryOomGroup => rc.title("OOM Group").format(Precision(1)),
+            MemoryZswapWriteback => rc.title("Zswap WB").format(Precision(1)),
             CpuWeight => rc.title("CPU Weight"),
             CpusetCpus => rc.title("Allowed CPUs"),
             CpusetCpusEffective => rc.title("Effective CPUs"),
@@ -1552,3 +1663,183 @@ impl HasRenderConfig for model::CgroupProperties {
         }
     }
 }
+
+impl HasRenderConfig for model::SingleTcModel {
+    fn get_render_config_builder(field_id: &Self::FieldId) -> RenderConfigBuilder {
+        use model::SingleTcModelFieldId::*;
+        let rc = RenderConfigBuilder::new();
+        match field_id {
+            Interface => rc.title("Interface"),
+            Kind => rc.title("Kind"),
+            Qlen => rc.title("Queue Length"),
+            Bps => rc.title("Bps").format(ReadableSize).suffix("/s"),
+            Pps => rc.title("Pps").suffix("/s"),
+            BytesPerSec => rc.title("Bytes").format(ReadableSize).suffix("/s"),
+            PacketsPerSec => rc.title("Packets").suffix("/s"),
+            BacklogPerSec => rc.title("Backlog").suffix("/s"),
+            DropsPerSec => rc.title("Drops").suffix("/s"),
+            RequeuesPerSec => rc.title("Requeues").suffix("/s"),
+            OverlimitsPerSec => rc.title("Overlimits").suffix("/s"),
+            Qdisc(field_id) => model::QDiscModel::get_render_config_builder(field_id),
+            Xstats(field_id) => model::XStatsModel::get_render_config_builder(field_id),
+        }
+    }
+}
+
+impl HasRenderConfigForDump for model::SingleTcModel {
+    fn get_openmetrics_config_for_dump(
+        &self,
+        field_id: &Self::FieldId,
+    ) -> Option<RenderOpenMetricsConfigBuilder> {
+        use model::SingleTcModelFieldId::*;
+        let gauge = gauge()
+            .label("interface", &self.interface)
+            .label("qdisc", &self.kind);
+        match field_id {
+            Interface => None,
+            Kind => None,
+            Qlen => Some(gauge),
+            Bps => Some(gauge.unit("bytes_per_second")),
+            Pps => Some(gauge.unit("packets_per_second")),
+            BytesPerSec => Some(gauge.unit("bytes_per_second")),
+            PacketsPerSec => Some(gauge.unit("packets_per_second")),
+            BacklogPerSec => Some(gauge.unit("packets_per_second")),
+            DropsPerSec => Some(gauge.unit("packets_per_second")),
+            RequeuesPerSec => Some(gauge.unit("packets_per_second")),
+            OverlimitsPerSec => Some(gauge.unit("packets_per_second")),
+            Qdisc(field_id) => self
+                .qdisc
+                .as_ref()
+                .and_then(|qdisc| qdisc.get_openmetrics_config_for_dump(field_id)),
+            Xstats(field_id) => self
+                .xstats
+                .as_ref()
+                .and_then(|xstats| xstats.get_openmetrics_config_for_dump(field_id)),
+        }
+    }
+}
+
+impl HasRenderConfig for model::QDiscModel {
+    fn get_render_config_builder(field_id: &Self::FieldId) -> RenderConfigBuilder {
+        use model::QDiscModelFieldId::*;
+        match field_id {
+            FqCodel(field_id) => model::FqCodelQDiscModel::get_render_config_builder(field_id),
+        }
+    }
+}
+
+impl HasRenderConfigForDump for model::QDiscModel {
+    fn get_openmetrics_config_for_dump(
+        &self,
+        field_id: &Self::FieldId,
+    ) -> Option<RenderOpenMetricsConfigBuilder> {
+        use model::QDiscModelFieldId::*;
+        match field_id {
+            FqCodel(field_id) => self
+                .fq_codel
+                .as_ref()
+                .and_then(|fq_codel| fq_codel.get_openmetrics_config_for_dump(field_id)),
+        }
+    }
+}
+
+impl HasRenderConfig for model::FqCodelQDiscModel {
+    fn get_render_config_builder(field_id: &Self::FieldId) -> RenderConfigBuilder {
+        use model::FqCodelQDiscModelFieldId::*;
+        let rc = RenderConfigBuilder::new();
+        match field_id {
+            Target => rc.title("Target"),
+            Limit => rc.title("Limit"),
+            Interval => rc.title("Interval"),
+            Ecn => rc.title("Ecn"),
+            Quantum => rc.title("Quantum"),
+            CeThreshold => rc.title("CeThreshold"),
+            DropBatchSize => rc.title("DropBatchSize"),
+            MemoryLimit => rc.title("MemoryLimit"),
+            Flows => rc.title("Flows"),
+        }
+    }
+}
+
+impl HasRenderConfig for model::XStatsModel {
+    fn get_render_config_builder(field_id: &Self::FieldId) -> RenderConfigBuilder {
+        use model::XStatsModelFieldId::*;
+        match field_id {
+            FqCodel(field_id) => model::FqCodelXStatsModel::get_render_config_builder(field_id),
+        }
+    }
+}
+
+impl HasRenderConfigForDump for model::XStatsModel {
+    fn get_openmetrics_config_for_dump(
+        &self,
+        field_id: &Self::FieldId,
+    ) -> Option<RenderOpenMetricsConfigBuilder> {
+        use model::XStatsModelFieldId::*;
+        match field_id {
+            FqCodel(field_id) => self
+                .fq_codel
+                .as_ref()
+                .and_then(|fq_codel| fq_codel.get_openmetrics_config_for_dump(field_id)),
+        }
+    }
+}
+
+impl HasRenderConfigForDump for model::FqCodelQDiscModel {
+    fn get_openmetrics_config_for_dump(
+        &self,
+        field_id: &Self::FieldId,
+    ) -> Option<RenderOpenMetricsConfigBuilder> {
+        use model::FqCodelQDiscModelFieldId::*;
+        match field_id {
+            Target => Some(gauge()),
+            Limit => Some(gauge()),
+            Interval => Some(gauge()),
+            Ecn => Some(gauge()),
+            Quantum => Some(gauge()),
+            CeThreshold => Some(gauge()),
+            DropBatchSize => Some(gauge()),
+            MemoryLimit => Some(gauge()),
+            Flows => Some(gauge()),
+        }
+    }
+}
+
+impl HasRenderConfig for model::FqCodelXStatsModel {
+    fn get_render_config_builder(field_id: &Self::FieldId) -> RenderConfigBuilder {
+        use model::FqCodelXStatsModelFieldId::*;
+        let rc = RenderConfigBuilder::new();
+        match field_id {
+            Maxpacket => rc.title("MaxPacket"),
+            EcnMark => rc.title("EcnMark"),
+            NewFlowsLen => rc.title("NewFlowsLen"),
+            OldFlowsLen => rc.title("OldFlowsLen"),
+            CeMark => rc.title("CeMark"),
+            DropOverlimitPerSec => rc.title("DropOverlimit").suffix("/s"),
+            NewFlowCountPerSec => rc.title("NewFlowCount").suffix("/s"),
+            MemoryUsagePerSec => rc.title("MemoryUsage").suffix("/s"),
+            DropOvermemoryPerSec => rc.title("DropOvermemory").suffix("/s"),
+        }
+    }
+}
+
+impl HasRenderConfigForDump for model::FqCodelXStatsModel {
+    fn get_openmetrics_config_for_dump(
+        &self,
+        field_id: &Self::FieldId,
+    ) -> Option<RenderOpenMetricsConfigBuilder> {
+        use model::FqCodelXStatsModelFieldId::*;
+        let gauge = gauge();
+        match field_id {
+            Maxpacket => Some(gauge),
+            EcnMark => Some(gauge),
+            NewFlowsLen => Some(gauge),
+            OldFlowsLen => Some(gauge),
+            CeMark => Some(gauge),
+            DropOverlimitPerSec => Some(gauge),
+            NewFlowCountPerSec => Some(gauge),
+            MemoryUsagePerSec => Some(gauge),
+            DropOvermemoryPerSec => Some(gauge),
+        }
+    }
+}
