@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#![deny(clippy::all)]
+
 use std::collections::BTreeMap;
 use std::fmt::Display;
 use std::fmt::Formatter;
@@ -22,6 +24,7 @@ mod default_configs;
 use common::open_source_shim;
 use common::util::convert_bytes;
 use common::util::convert_duration;
+use common::util::convert_duration_sec;
 use common::util::convert_freq;
 use common::util::fold_string;
 use model::Field;
@@ -48,9 +51,12 @@ pub enum RenderFormat {
     MaxOrReadableSize,
     /// Frequency. Format with human-readable freq with suffixes (MHz, GHz etc.)
     ReadableFrequency,
-    /// Only works on int Fields. Displays duration with human readable
+    /// Only works on int Fields. Displays us duration with human readable
     /// suffixes (us, ms, s, etc.)
     Duration,
+    /// Only works on int Fields. Displays long duration inf human readable
+    /// format (?d ?h ?m)
+    DurationSec,
     /// Only works on int Fields. -1 displays "max" else displays duration with
     /// human readable suffixes (us, ms, s, etc.)
     MaxOrDuration,
@@ -295,10 +301,16 @@ impl From<RenderConfig> for RenderConfigBuilder {
     }
 }
 
+/// Pad `val` with spaces on right side to a total length of `width` characters.
 pub fn get_fixed_width(val: &str, width: usize) -> String {
     format!("{val:width$.width$}", val = val, width = width)
 }
 
+/// Same as `get_fixed_width`, except text is right justified.
+pub fn get_fixed_width_rjust(val: &str, width: usize) -> String {
+    format!("{val:>width$.width$}", val = val, width = width)
+}
+
 impl RenderConfig {
     pub fn update<T: Into<Self>>(mut self, overrides: T) -> Self {
         let overrides = overrides.into();
@@ -352,6 +364,10 @@ impl RenderConfig {
                     let field = u64::from(field);
                     convert_duration(field)
                 }
+                DurationSec => {
+                    let field = u64::from(field);
+                    convert_duration_sec(field)
+                }
                 MaxOrDuration => {
                     let field = i64::from(field);
                     if field == -1 {
