@@ -12,38 +12,43 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#![deny(clippy::all)]
+#![allow(clippy::too_many_arguments)]
 #![recursion_limit = "256"]
 
-use std::cell::RefCell;
 use std::fs;
 use std::io;
-use std::os::unix::fs::PermissionsExt;
 use std::path::PathBuf;
 use std::process::exit;
-use std::rc::Rc;
-use std::sync::mpsc::channel;
-use std::sync::mpsc::Receiver;
-use std::sync::mpsc::RecvTimeoutError;
-use std::sync::mpsc::TryRecvError;
 use std::sync::Arc;
 use std::sync::Mutex;
+use std::sync::mpsc::Receiver;
+use std::sync::mpsc::RecvTimeoutError;
+use std::sync::mpsc::SyncSender;
+use std::sync::mpsc::TryRecvError;
+use std::sync::mpsc::channel;
+use std::sync::mpsc::sync_channel;
 use std::thread;
+use std::thread::JoinHandle;
 use std::time::Duration;
 use std::time::Instant;
 use std::time::SystemTime;
 
-use anyhow::anyhow;
-use anyhow::bail;
 use anyhow::Context;
 use anyhow::Error;
 use anyhow::Result;
+use anyhow::anyhow;
+use anyhow::bail;
 use clap::CommandFactory;
 use clap::Parser;
-use clap_complete::generate;
 use clap_complete::Shell;
+use clap_complete::generate;
 use cursive::Cursive;
 use indicatif::ProgressBar;
+use nix::sys::mman::MlockAllFlags;
+use nix::sys::mman::mlockall;
 use regex::Regex;
+use scopeguard::guard;
 use signal_hook::iterator::Signals;
 use slog::debug;
 use slog::error;
@@ -52,8 +57,6 @@ use tar::Archive;
 use tar::Builder as TarBuilder;
 use tempfile::TempDir;
 use tokio::runtime::Builder as TB;
-use uzers::get_current_uid;
-use uzers::get_user_by_uid;
 
 mod exitstat;
 #[cfg(test)]
@@ -64,12 +67,12 @@ use common::logutil;
 use common::open_source_shim;
 use config::BelowConfig;
 use dump::DumpCommand;
-use store::advance::new_advance_local;
-use store::advance::new_advance_remote;
 use store::ChunkSizePo2;
 use store::CompressionMode;
 use store::DataFrame;
 use store::Store;
+use store::advance::new_advance_local;
+use store::advance::new_advance_remote;
 use view::ViewState;
 
 open_source_shim!();
@@ -202,6 +205,9 @@ enum Command {
         /// Options for compression
         #[clap(flatten)]
         compress_opts: CompressOpts,
+        /// Max number of samples we will buffer before writing becomes blocking
+        #[clap(short, default_value = "10")]
+        writer_buffer_size: usize,
     },
     /// Replay historical data (interactive)
     Replay {
@@ -329,13 +335,6 @@ pub enum Service {
     Off,
 }
 
-// Whether or not to redirect log to stderr on fs failure
-#[derive(PartialEq)]
-pub enum RedirectLogOnFail {
-    On,
-    Off,
-}
-
 fn bump_memlock_rlimit() -> Result<()> {
     // TODO(T78976996) remove the fbcode_gate once we can exit stats is
     // enabled for opensource
@@ -353,44 +352,6 @@ fn bump_memlock_rlimit() -> Result<()> {
     Ok(())
 }
 
-fn create_log_dir(path: &PathBuf) -> Result<()> {
-    if path.exists() && !path.is_dir() {
-        bail!("{} exists and is not a directory", path.to_string_lossy());
-    }
-
-    if !path.is_dir() {
-        match fs::create_dir_all(path) {
-            Ok(()) => {}
-            Err(e) => {
-                bail!(
-                    "Failed to create dir {}: {}\nTry sudo.",
-                    path.to_string_lossy(),
-                    e
-                );
-            }
-        }
-    }
-
-    let dir = fs::File::open(path).unwrap();
-    let mut perm = dir.metadata().unwrap().permissions();
-
-    if perm.mode() & 0o777 != 0o777 {
-        perm.set_mode(0o777);
-        match dir.set_permissions(perm) {
-            Ok(()) => {}
-            Err(e) => {
-                bail!(
-                    "Failed to set permissions on {}: {}",
-                    path.to_string_lossy(),
-                    e
-                );
-            }
-        }
-    }
-
-    Ok(())
-}
-
 // Exitstat runs a bpf program that hooks into process exit events. This allows below to keep
 // track of processes whose lifetimes are shorter than polling interval.
 fn start_exitstat(
@@ -405,7 +366,7 @@ fn start_exitstat(
         .spawn(move || {
             match exit_driver.drive() {
                 Ok(_) => {}
-                Err(e) => bpf_err_send.send(e).unwrap(),
+                Err(e) => bpf_err_send.send(e).expect("Failed to send error"),
             };
         })
         .expect("Failed to spawn thread");
@@ -418,7 +379,7 @@ pub fn start_gpu_stats_thread_and_get_stats_receiver(
     logger: slog::Logger,
     interval: Duration,
 ) -> Result<model::collector_plugin::Consumer<model::gpu_stats_collector_plugin::SampleType>> {
-    let target_interval = interval.clone();
+    let target_interval = interval;
     let gpu_collector = gpu_stats::get_gpu_stats_collector_plugin(init, logger.clone())
         .context("Failed to initialize GPU stats collector")?;
     let (mut collector, receiver) = model::collector_plugin::collector_consumer(gpu_collector);
@@ -475,12 +436,84 @@ pub fn start_gpu_stats_thread_and_get_stats_receiver(
     Ok(receiver)
 }
 
+fn start_tc_stats_thread_and_get_stats_receiver(
+    logger: slog::Logger,
+    interval: Duration,
+) -> Result<model::collector_plugin::Consumer<model::tc_collector_plugin::SampleType>> {
+    let tc_collector = model::tc_collector_plugin::TcStatsCollectorPlugin::new(logger.clone())
+        .context("Failed to initialize TC stats collector")?;
+    let (mut collector, receiver) = model::collector_plugin::collector_consumer(tc_collector);
+
+    // Start a thread to collect TC stats
+    let target_interval = interval;
+    thread::Builder::new()
+        .name("tc_stats_collector".to_owned())
+        .spawn(move || {
+            // Exponential backoff on unrecoverable errors
+            const EXP_BACKOFF_FACTOR: u32 = 2;
+            const MAX_BACKOFF_SECS: u64 = 900;
+            let max_backoff = Duration::from_secs(MAX_BACKOFF_SECS);
+            let mut interval = target_interval;
+            loop {
+                let collect_instant = Instant::now();
+                let rt = TB::new_current_thread()
+                    .thread_name("create_fburl")
+                    .build()
+                    .expect("Failed to build tokio runtime.");
+                match rt.block_on(collector.collect_and_update()) {
+                    Ok(_) => {
+                        interval = target_interval;
+                    }
+                    Err(e) => {
+                        interval = std::cmp::min(
+                            interval
+                                .saturating_mul(EXP_BACKOFF_FACTOR),
+                            max_backoff,
+                        );
+                        error!(
+                            logger,
+                            "TC stats collection backing off {:?} because of unrecoverable error: {:?}",
+                            interval,
+                            e
+                        );
+                    }
+                }
+                let collect_duration = Instant::now().duration_since(collect_instant);
+
+                const COLLECT_DURATION_WARN_THRESHOLD: u64 = 2;
+                if collect_duration > Duration::from_secs(COLLECT_DURATION_WARN_THRESHOLD) {
+                    warn!(
+                        logger,
+                        "TC collection took {} > {}",
+                        collect_duration.as_secs_f64(),
+                        COLLECT_DURATION_WARN_THRESHOLD
+                    );
+                }
+                if interval > collect_duration {
+                    let sleep_duration = interval - collect_duration;
+                    std::thread::sleep(sleep_duration);
+                }
+            }
+        })
+        .expect("Failed to spawn thread");
+
+    Ok(receiver)
+}
+
 /// Returns true if other end disconnected, false otherwise
 fn check_for_exitstat_errors(logger: &slog::Logger, receiver: &Receiver<Error>) -> bool {
     // Print an error but don't exit on bpf issues. Do this b/c we can't always
     // be sure what kind of kernel we're running on and if it's new enough.
     match receiver.try_recv() {
-        Ok(e) => error!(logger, "{:#}", e),
+        Ok(e) => {
+            // When running as non-root for live, ignore EACCESS
+            if let Some(e) = e.downcast_ref::<libbpf_rs::Error>() {
+                if e.kind() == libbpf_rs::ErrorKind::PermissionDenied {
+                    return false;
+                }
+            }
+            error!(logger, "{:#}", e);
+        }
         Err(TryRecvError::Empty) => {}
         Err(TryRecvError::Disconnected) => {
             warn!(logger, "bpf error channel disconnected");
@@ -518,6 +551,66 @@ fn cleanup_store(
     Ok(())
 }
 
+pub struct WorkerTask {
+    post_collect_sys_time: SystemTime,
+    data: DataFrame,
+}
+
+fn start_store_writer_thread(
+    logger: slog::Logger,
+    mut store: store::StoreWriter,
+    store_size_limit: Option<u64>,
+    retention: Option<Duration>,
+    writer_buffer_size: usize,
+) -> Result<(JoinHandle<()>, SyncSender<WorkerTask>)> {
+    let (send_task, recv_task) = sync_channel::<WorkerTask>(writer_buffer_size);
+    let handle = thread::Builder::new()
+        .name("store_writer".to_owned())
+        .spawn(move || {
+            loop {
+                let loop_start_time = Instant::now();
+                match recv_task.recv() {
+                    Ok(write_task) => {
+                        match store.put(write_task.post_collect_sys_time, &write_task.data) {
+                            Ok(/* new shard */ true) => {
+                                cleanup_store(
+                                    &store,
+                                    &logger,
+                                    store_size_limit,
+                                    /* retention */ None,
+                                )
+                                .expect("cleanup_store failed");
+                            }
+                            Ok(/* new shard */ false) => {}
+                            Err(e) => {
+                                error!(logger, "{:#}", e);
+                                // no need to report/cleanup
+                                continue;
+                            }
+                        }
+                        statistics::report_writer_time_ms(
+                            Instant::now().duration_since(loop_start_time),
+                        );
+                    }
+                    Err(_) => {
+                        warn!(
+                            logger,
+                            "store_write input channel disconnected, exiting thread"
+                        );
+                        // end loop
+                        break;
+                    }
+                }
+
+                // Only check against retention and not size limit. Size limit is only
+                // checked on creation of successful write to a new shard.
+                cleanup_store(&store, &logger, /* store_size_limit */ None, retention)
+                    .expect("cleanup_store failed");
+            }
+        })?;
+    Ok((handle, send_task))
+}
+
 /// Special Error that indicates the program should stop now. It represents an
 /// actual signal, e.g. SIGINT, SIGTERM, that is handled by below and thus below
 /// can shutdown gracefully.
@@ -539,27 +632,17 @@ pub fn run<F>(
     debug: bool,
     below_config: &BelowConfig,
     _service: Service,
-    redirect: RedirectLogOnFail,
     command: F,
 ) -> i32
 where
     F: FnOnce(init::InitToken, &BelowConfig, slog::Logger, Receiver<Error>) -> Result<()>,
 {
     let (err_sender, err_receiver) = channel();
-    let mut log_dir = below_config.log_dir.clone();
-    let user = get_user_by_uid(get_current_uid()).expect("Failed to get current user for logging");
-
-    log_dir.push(format!("error_{}.log", user.name().to_string_lossy()));
-
-    if let Err(e) = create_log_dir(&below_config.log_dir) {
-        eprintln!("{:#}", e);
-        return 1;
-    }
-
-    let logger = logging::setup(init, log_dir, debug, redirect);
+    let log_path = below_config.log_dir.join("below.log");
+    let logger = logging::setup(init, log_path, debug);
     setup_log_on_panic(logger.clone());
 
-    match Signals::new(&[signal_hook::consts::SIGINT, signal_hook::consts::SIGTERM]) {
+    match Signals::new([signal_hook::consts::SIGINT, signal_hook::consts::SIGTERM]) {
         Ok(mut signals) => {
             let logger = logger.clone();
             let err_sender = err_sender.clone();
@@ -574,7 +657,9 @@ where
                         }
                         term_now = true;
                         error!(logger, "Stop signal received: {}, exiting.", signal);
-                        err_sender.send(anyhow!(StopSignal { signal })).unwrap();
+                        err_sender
+                            .send(anyhow!(StopSignal { signal }))
+                            .expect("Failed to send stop signal");
                     }
                 })
                 .expect("Failed to spawn thread");
@@ -586,7 +671,7 @@ where
     }
 
     #[cfg(fbcode_build)]
-    facebook::init(
+    let _guard = facebook::init(
         init,
         logger.clone(),
         _service,
@@ -654,24 +739,23 @@ fn real_main(init: init::InitToken) {
     let rc = match cmd {
         Command::External(command) => commands::run_command(init, debug, below_config, command),
         Command::Live {
-            ref interval_s,
-            ref host,
-            ref port,
+            interval_s,
+            host,
+            port,
         } => {
             let host = host.clone();
-            let port = port.clone();
+            let port = *port;
             run(
                 init,
                 debug,
                 below_config,
                 Service::Off,
-                RedirectLogOnFail::On,
                 |_, below_config, logger, errs| {
                     live(
                         init,
                         logger,
                         errs,
-                        Duration::from_secs(*interval_s as u64),
+                        Duration::from_secs(*interval_s),
                         debug,
                         below_config,
                         host,
@@ -681,15 +765,16 @@ fn real_main(init: init::InitToken) {
             )
         }
         Command::Record {
-            ref interval_s,
-            ref retain_for_s,
-            ref store_size_limit,
-            ref collect_io_stat,
-            ref port,
-            ref skew_detection_threshold_ms,
-            ref disable_disk_stat,
-            ref disable_exitstats,
-            ref compress_opts,
+            interval_s,
+            retain_for_s,
+            store_size_limit,
+            collect_io_stat,
+            port,
+            skew_detection_threshold_ms,
+            disable_disk_stat,
+            disable_exitstats,
+            compress_opts,
+            writer_buffer_size,
         } => {
             logutil::set_current_log_target(logutil::TargetLog::Term);
             run(
@@ -697,15 +782,14 @@ fn real_main(init: init::InitToken) {
                 debug,
                 below_config,
                 Service::On(*port),
-                RedirectLogOnFail::Off,
                 |init, below_config, logger, errs| {
                     record(
                         init,
                         logger,
                         errs,
-                        Duration::from_secs(*interval_s as u64),
+                        Duration::from_secs(*interval_s),
                         below_config,
-                        retain_for_s.map(|r| Duration::from_secs(r as u64)),
+                        retain_for_s.map(Duration::from_secs),
                         *store_size_limit,
                         *collect_io_stat,
                         Duration::from_millis(*skew_detection_threshold_ms),
@@ -713,20 +797,21 @@ fn real_main(init: init::InitToken) {
                         *disable_disk_stat,
                         *disable_exitstats,
                         compress_opts,
+                        *writer_buffer_size,
                     )
                 },
             )
         }
         Command::Replay {
-            ref time,
-            ref host,
-            ref port,
-            ref yesterdays,
-            ref snapshot,
+            time,
+            host,
+            port,
+            yesterdays,
+            snapshot,
         } => {
             let time = time.clone();
             let host = host.clone();
-            let port = port.clone();
+            let port = *port;
             let days_adjuster = yesterdays.clone();
             let snapshot = snapshot.clone();
             run(
@@ -734,7 +819,6 @@ fn real_main(init: init::InitToken) {
                 debug,
                 below_config,
                 Service::Off,
-                RedirectLogOnFail::Off,
                 |_, below_config, logger, errs| {
                     replay(
                         logger,
@@ -750,25 +834,24 @@ fn real_main(init: init::InitToken) {
             )
         }
         Command::Snapshot {
-            ref begin,
-            ref end,
-            ref duration,
-            ref output,
-            ref host,
-            ref port,
+            begin,
+            end,
+            duration,
+            output,
+            host,
+            port,
         } => {
             let begin = begin.clone();
             let end = end.clone();
             let duration = duration.clone();
             let output = output.clone();
             let host = host.clone();
-            let port = port.clone();
+            let port = *port;
             run(
                 init,
                 debug,
                 below_config,
                 Service::Off,
-                RedirectLogOnFail::Off,
                 |_, below_config, logger, _errs| {
                     snapshot(
                         logger,
@@ -783,28 +866,27 @@ fn real_main(init: init::InitToken) {
                 },
             )
         }
-        Command::Debug { ref cmd } => match cmd {
-            DebugCommand::DumpStore { ref time, ref json } => {
+        Command::Debug { cmd } => match cmd {
+            DebugCommand::DumpStore { time, json } => {
                 let time = time.clone();
-                let json = json.clone();
+                let json = *json;
                 run(
                     init,
                     debug,
                     below_config,
                     Service::Off,
-                    RedirectLogOnFail::Off,
                     |_, below_config, logger, _errs| dump_store(logger, time, below_config, json),
                 )
             }
             DebugCommand::ConvertStore {
-                ref begin,
-                ref end,
-                ref duration,
-                ref from_store_dir,
-                ref to_store_dir,
-                ref host,
-                ref port,
-                ref compress_opts,
+                begin,
+                end,
+                duration,
+                from_store_dir,
+                to_store_dir,
+                host,
+                port,
+                compress_opts,
             } => {
                 let begin = begin.clone();
                 let end = end.clone();
@@ -812,13 +894,12 @@ fn real_main(init: init::InitToken) {
                 let from_store_dir = from_store_dir.clone();
                 let to_store_dir = to_store_dir.clone();
                 let host = host.clone();
-                let port = port.clone();
+                let port = *port;
                 run(
                     init,
                     debug,
                     below_config,
                     Service::Off,
-                    RedirectLogOnFail::Off,
                     |_, below_config, logger, _errs| {
                         convert_store(
                             logger,
@@ -837,14 +918,14 @@ fn real_main(init: init::InitToken) {
             }
         },
         Command::Dump {
-            ref host,
-            ref port,
-            ref snapshot,
-            ref cmd,
+            host,
+            port,
+            snapshot,
+            cmd,
         } => {
             let store_dir = below_config.store_dir.clone();
             let host = host.clone();
-            let port = port.clone();
+            let port = *port;
             let snapshot = snapshot.clone();
             let cmd = cmd.clone();
             run(
@@ -852,17 +933,13 @@ fn real_main(init: init::InitToken) {
                 debug,
                 below_config,
                 Service::Off,
-                RedirectLogOnFail::Off,
                 |_, _below_config, logger, errs| {
                     dump::run(logger, errs, store_dir, host, port, snapshot, cmd)
                 },
             )
         }
-        Command::GenerateCompletions {
-            ref shell,
-            ref output,
-        } => {
-            generate_completions(shell.clone(), output.clone())
+        Command::GenerateCompletions { shell, output } => {
+            generate_completions(*shell, output.clone())
                 .unwrap_or_else(|_| panic!("Failed to generate completions for {:?}", shell));
             0
         }
@@ -890,12 +967,12 @@ fn replay(
         (Some(host), None) => new_advance_remote(logger.clone(), host, port, timestamp)?,
         (None, Some(snapshot)) => {
             let mut tarball =
-                Archive::new(fs::File::open(&snapshot).context("Failed to open snapshot file")?);
-            let mut snapshot_dir = TempDir::with_prefix("snapshot_replay.")?.into_path();
+                Archive::new(fs::File::open(snapshot).context("Failed to open snapshot file")?);
+            let mut snapshot_dir = TempDir::with_prefix("snapshot_replay.")?.keep();
             tarball.unpack(&snapshot_dir)?;
             // Find and append the name of the original snapshot directory
             for path in fs::read_dir(&snapshot_dir)? {
-                snapshot_dir.push(path.unwrap().file_name());
+                snapshot_dir.push(path?.file_name());
             }
             new_advance_local(logger.clone(), snapshot_dir, timestamp)
         }
@@ -920,10 +997,8 @@ fn replay(
 
     cliutil::check_initial_sample_time_with_requested_time(model.timestamp, timestamp);
 
-    let mut view = view::View::new_with_advance(
-        model,
-        view::ViewMode::Replay(Rc::new(RefCell::new(advance))),
-    );
+    let mut view =
+        view::View::new_with_advance(model, view::ViewMode::Replay(Arc::new(Mutex::new(advance))));
     logutil::set_current_log_target(logutil::TargetLog::File);
 
     let sink = view.cb_sink().clone();
@@ -960,6 +1035,7 @@ fn record(
     disable_disk_stat: bool,
     disable_exitstats: bool,
     compress_opts: &CompressOpts,
+    writer_buffer_size: usize,
 ) -> Result<()> {
     debug!(logger, "Starting up!");
 
@@ -967,14 +1043,6 @@ fn record(
         bump_memlock_rlimit()?;
     }
 
-    let mut store = store::StoreWriter::new(
-        logger.clone(),
-        &below_config.store_dir,
-        compress_opts.to_compression_mode()?,
-        store::Format::Cbor,
-    )?;
-    let mut stats = statistics::Statistics::new(init.clone());
-
     let (exit_buffer, bpf_errs) = if disable_exitstats {
         (Arc::new(Mutex::new(procfs::PidMap::new())), None)
     } else {
@@ -1002,6 +1070,15 @@ fn record(
         None
     };
 
+    let tc_stats_receiver = if below_config.enable_tc_stats {
+        Some(start_tc_stats_thread_and_get_stats_receiver(
+            logger.clone(),
+            interval,
+        )?)
+    } else {
+        None
+    };
+
     let mut collector = model::Collector::new(
         logger.clone(),
         model::CollectorOptions {
@@ -1011,14 +1088,50 @@ fn record(
             disable_disk_stat,
             enable_btrfs_stats: below_config.enable_btrfs_stats,
             enable_ethtool_stats: below_config.enable_ethtool_stats,
+            enable_ksm_stats: below_config.enable_ksm_stats,
             enable_resctrl_stats: below_config.enable_resctrl_stats,
+            enable_tc_stats: below_config.enable_tc_stats,
             btrfs_samples: below_config.btrfs_samples,
             btrfs_min_pct: below_config.btrfs_min_pct,
             cgroup_re,
             gpu_stats_receiver,
+            tc_stats_receiver,
         },
     );
 
+    let store = store::StoreWriter::new(
+        logger.clone(),
+        &below_config.store_dir,
+        compress_opts.to_compression_mode()?,
+        store::Format::Cbor,
+    )?;
+
+    let mut stats = statistics::Statistics::new(init);
+
+    let (writer_thread, send_task) = start_store_writer_thread(
+        logger.clone(),
+        store,
+        store_size_limit,
+        retention,
+        writer_buffer_size,
+    )?;
+
+    let send_task = guard(send_task, |s| {
+        drop(s);
+        let _ = writer_thread.join();
+    });
+
+    if below_config.mlock_record {
+        let ret = mlockall(MlockAllFlags::MCL_CURRENT);
+        if ret.is_err() {
+            warn!(
+                logger,
+                "mlockall failed: {}. Continue without mlock",
+                std::io::Error::last_os_error()
+            );
+        }
+    }
+
     loop {
         if !disable_exitstats {
             // Anything that comes over the error channel is an error
@@ -1052,43 +1165,36 @@ fn record(
                 collection_skew.as_millis(),
                 skew_detection_threshold.as_millis()
             );
-
-            stats.report_collection_skew();
+            statistics::report_collection_skew();
         }
 
         match collected_sample {
             Ok(s) => {
-                let frame = DataFrame { sample: s };
-                match store.put(post_collect_sys_time, &frame) {
-                    Ok(/* new shard */ true) => {
-                        cleanup_store(&store, &logger, store_size_limit, /* retention */ None)?
-                    }
-                    Ok(/* new shard */ false) => {}
-                    Err(e) => error!(logger, "{:#}", e),
-                }
                 if below_config.enable_gpu_stats {
-                    stats.report_nr_accelerators(&frame.sample);
+                    stats.report_nr_accelerators(&s);
                 }
+                send_task
+                    .send(WorkerTask {
+                        post_collect_sys_time,
+                        data: DataFrame { sample: s },
+                    })
+                    .expect("Failed to send write task");
             }
             Err(e) => {
                 // Handle cgroupfs errors
-                match e.downcast_ref::<cgroupfs::Error>() {
+                if let Some(cgroupfs::Error::NotCgroup2(_)) = e.downcast_ref::<cgroupfs::Error>() {
                     // Unrecoverable error -- below only supports cgroup2
-                    Some(cgroupfs::Error::NotCgroup2(_)) => bail!(e),
-                    _ => {}
-                };
-
+                    bail!(e)
+                }
                 error!(logger, "{:#}", e);
             }
         };
 
-        // Only check against retention and not size limit. Size limit is only
-        // checked on creation of successful write to a new shard.
-        cleanup_store(&store, &logger, /* store_size_limit */ None, retention)?;
-
         stats.report_store_size(below_config.store_dir.as_path());
 
         let collect_duration = Instant::now().duration_since(collect_instant);
+        statistics::report_collection_time_ms(collect_duration);
+
         // Sleep for at least 1s to avoid sample collision
         let sleep_duration = if interval > collect_duration {
             std::cmp::max(Duration::from_secs(1), interval - collect_duration)
@@ -1107,17 +1213,14 @@ fn live_local(
     debug: bool,
     below_config: &BelowConfig,
 ) -> Result<()> {
-    match bump_memlock_rlimit() {
-        Err(e) => {
-            warn!(
-                logger,
-                #"V",
-                "Failed to initialize BPF: {}. Data collection will be degraded. \
-                You can ignore this warning or try to run with sudo.",
-                &e
-            );
-        }
-        _ => {}
+    if let Err(e) = bump_memlock_rlimit() {
+        warn!(
+            logger,
+            # "V",
+            "Failed to initialize BPF: {}. Data collection will be degraded. \
+            You can ignore this warning or try to run with sudo.",
+            &e
+        );
     };
 
     let (exit_buffer, bpf_errs) = start_exitstat(logger.clone(), debug);
@@ -1140,6 +1243,7 @@ fn live_local(
             exit_data: exit_buffer,
             enable_btrfs_stats: below_config.enable_btrfs_stats,
             enable_ethtool_stats: below_config.enable_ethtool_stats,
+            enable_ksm_stats: below_config.enable_ksm_stats,
             enable_resctrl_stats: below_config.enable_resctrl_stats,
             btrfs_samples: below_config.btrfs_samples,
             btrfs_min_pct: below_config.btrfs_min_pct,
@@ -1157,7 +1261,7 @@ fn live_local(
     adv.initialize();
     let mut view = view::View::new_with_advance(
         collector.collect_and_update_model()?,
-        view::ViewMode::Live(Rc::new(RefCell::new(adv))),
+        view::ViewMode::Live(Arc::new(Mutex::new(adv))),
     );
 
     let sink = view.cb_sink().clone();
@@ -1232,10 +1336,9 @@ fn live_remote(
 
     advance.initialize();
     let mut view = match advance.get_latest_sample() {
-        Some(model) => view::View::new_with_advance(
-            model,
-            view::ViewMode::Live(Rc::new(RefCell::new(advance))),
-        ),
+        Some(model) => {
+            view::View::new_with_advance(model, view::ViewMode::Live(Arc::new(Mutex::new(advance))))
+        }
         None => return Err(anyhow!("No data could be found!")),
     };
 
@@ -1266,9 +1369,8 @@ fn live_remote(
                     let view_state = s.user_data::<ViewState>().expect("user data not set");
 
                     if let view::ViewMode::Live(adv) = view_state.mode.clone() {
-                        match adv.borrow_mut().advance(store::Direction::Forward) {
-                            Some(data) => view_state.update(data),
-                            None => {}
+                        if let Some(data) = adv.lock().unwrap().advance(store::Direction::Forward) {
+                            view_state.update(data)
                         }
                     }
                 });
@@ -1448,12 +1550,12 @@ fn snapshot(
 
     // Create a directory for the output files
     // Format: snapshot_<timestamp_begin>_<timestamp_end>
-    let temp_folder = TempDir::with_prefix(&format!(
+    let temp_folder = TempDir::with_prefix(format!(
         "snapshot_{:011}_{:011}.",
         timestamp_begin, timestamp_end
     ))
     .context("Failed to create temporary folder for snapshot")?;
-    let snapshot_store_path = temp_folder.into_path();
+    let snapshot_store_path = temp_folder.keep();
 
     // Build compression options to ensure snapshot is compressed before tarball
     let compress_opts = CompressOpts {
@@ -1520,6 +1622,10 @@ fn setup_log_on_panic(logger: slog::Logger) {
             },
         };
 
+        if logutil::get_current_log_target() == logutil::TargetLog::File {
+            logutil::set_current_log_target(logutil::TargetLog::All);
+        }
+
         match info.location() {
             Some(location) => {
                 error!(
