@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#![deny(clippy::all)]
+
 use std::fs;
 use std::fs::File;
 use std::fs::OpenOptions;
@@ -22,9 +24,9 @@ use std::path::Path;
 use std::path::PathBuf;
 use std::time::SystemTime;
 
-use anyhow::bail;
 use anyhow::Context;
 use anyhow::Result;
+use anyhow::bail;
 use bitflags::bitflags;
 use common::fileutil::get_dir_size;
 use common::open_source_shim;
@@ -397,8 +399,10 @@ impl StoreWriter {
 
     /// For the given `DataFrame` and an optional Compressor mut ref, returns a
     /// tuple consisting of:
+    ///
     ///   1) Raw bytes to write to the data file
     ///   2) Flags to write to the index entry
+    ///
     /// For compressed write, the Compressor will be initialized if None, and
     /// potentially updated. is_key_frame is used to indicate the start of a new
     /// chunk if dictionary compression is enabled.
@@ -729,7 +733,7 @@ pub fn read_next_sample<P: AsRef<Path>>(
     cursor.get_next(&get_unix_timestamp(timestamp), direction)
 }
 
-pub trait Store {
+pub trait Store: Send + Sync {
     // We intentionally make this trait generic which not tied to the DataFrame and Model
     // type for ease of testing.
     // For LocalStore and RemoteStore, SampleType will be DataFrame
@@ -937,7 +941,7 @@ mod tests {
 
         // States, (compression_mode, format), that we transition between when
         // writing
-        let states = vec![
+        let states = [
             (CompressionMode::None, Format::Cbor),
             (CompressionMode::Zstd, Format::Cbor),
             (
@@ -1517,7 +1521,7 @@ mod tests {
             for i in 0..n {
                 frame.sample.cgroup.memory_current = Some(n as i64 + i as i64);
                 writer
-                    .put(timestamp + Duration::from_secs(i as u64), &frame)
+                    .put(timestamp + Duration::from_secs(i), &frame)
                     .expect("Failed to store data");
             }
             let dir_size_after = get_dir_size(dir_path_buf.clone());
@@ -1528,7 +1532,7 @@ mod tests {
                 dir_size_after,
                 n,
             );
-            return dir_size_after - dir_size;
+            dir_size_after - dir_size
         };
 
         let num_shards = 7;
