@@ -16,12 +16,12 @@ use std::collections::HashSet;
 
 use base_render::RenderConfig;
 use cursive::utils::markup::StyledString;
-use model::sort_queriables;
 use model::CgroupModel;
 use model::CgroupModelFieldId;
 use model::Queriable;
 use model::SingleCgroupModel;
 use model::SingleCgroupModelFieldId;
+use model::sort_queriables;
 
 use crate::cgroup_view::CgroupState;
 use crate::render::ViewItem;
@@ -116,7 +116,8 @@ impl CgroupTab {
 
             let collapsed = state
                 .collapsed_cgroups
-                .borrow()
+                .lock()
+                .unwrap()
                 .contains(&cgroup.data.full_path);
             let row = self.get_line(&cgroup.data, collapsed, offset, cgroup.recreate_flag);
             // Each row is (label, value), where label is visible and value is used
@@ -134,10 +135,10 @@ impl CgroupTab {
             let mut children = Vec::from_iter(&cgroup.children);
             if let Some(sort_order) = state.sort_order.as_ref() {
                 // field_id that query its own data
-                let field_id = CgroupModelFieldId {
-                    path: Some(vec![]),
-                    subquery_id: sort_order.clone(),
-                };
+                let field_id = CgroupModelFieldId::new(
+                    Some(model::CgroupPath { path: vec![] }),
+                    sort_order.clone(),
+                );
                 sort_queriables(&mut children, &field_id, state.reverse);
             }
 
@@ -146,7 +147,8 @@ impl CgroupTab {
                 for child_cgroup in &children {
                     state
                         .collapsed_cgroups
-                        .borrow_mut()
+                        .lock()
+                        .unwrap()
                         .insert(child_cgroup.data.full_path.clone());
                 }
             }
@@ -264,6 +266,7 @@ pub mod default_tabs {
     use model::CgroupMemoryModelFieldId::File;
     use model::CgroupMemoryModelFieldId::FileDirty;
     use model::CgroupMemoryModelFieldId::FileMapped;
+    use model::CgroupMemoryModelFieldId::FileThp;
     use model::CgroupMemoryModelFieldId::FileWriteback;
     use model::CgroupMemoryModelFieldId::InactiveAnon;
     use model::CgroupMemoryModelFieldId::InactiveFile;
@@ -314,8 +317,10 @@ pub mod default_tabs {
     use model::CgroupPropertiesFieldId::MemoryLow;
     use model::CgroupPropertiesFieldId::MemoryMax;
     use model::CgroupPropertiesFieldId::MemoryMin;
+    use model::CgroupPropertiesFieldId::MemoryOomGroup;
     use model::CgroupPropertiesFieldId::MemorySwapMax;
     use model::CgroupPropertiesFieldId::MemoryZswapMax;
+    use model::CgroupPropertiesFieldId::MemoryZswapWriteback;
     use model::CgroupPropertiesFieldId::TidsMax;
     use model::CgroupStatModelFieldId::NrDescendants;
     use model::CgroupStatModelFieldId::NrDyingDescendants;
@@ -372,6 +377,7 @@ pub mod default_tabs {
             ViewItem::from_default(Mem(FileMapped)),
             ViewItem::from_default(Mem(FileDirty)),
             ViewItem::from_default(Mem(FileWriteback)),
+            ViewItem::from_default(Mem(FileThp)),
             ViewItem::from_default(Mem(AnonThp)),
             ViewItem::from_default(Mem(InactiveAnon)),
             ViewItem::from_default(Mem(ActiveAnon)),
@@ -441,6 +447,8 @@ pub mod default_tabs {
             ViewItem::from_default(Props(MemoryMax)),
             ViewItem::from_default(Props(MemorySwapMax)),
             ViewItem::from_default(Props(MemoryZswapMax)),
+            ViewItem::from_default(Props(MemoryZswapWriteback)),
+            ViewItem::from_default(Props(MemoryOomGroup)),
             ViewItem::from_default(Props(CpuMaxUsec)),
             ViewItem::from_default(Props(CpuMaxPeriodUsec)),
             ViewItem::from_default(Props(CpuWeight)),
