@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#![deny(clippy::all)]
+
 /// View Module defines how to render below inside terminal.
 ///
 /// ## High level architecture
@@ -50,9 +52,9 @@
 /// * Column names: The column names line also called title line in below_derive. It defines the table column of
 ///   the following selectable view. A user can press `,` or `.` to switch between different columns and press `s`
 ///   or `S` to sort in ascending or descending order.
-use std::cell::RefCell;
 use std::collections::HashMap;
-use std::rc::Rc;
+use std::sync::Arc;
+use std::sync::Mutex;
 use std::time::Duration;
 use std::time::SystemTime;
 
@@ -64,6 +66,9 @@ use common::util::get_belowrc_filename;
 use common::util::get_belowrc_view_section_key;
 use crossterm::event::DisableMouseCapture;
 use crossterm::execute;
+use cursive::Cursive;
+use cursive::CursiveRunnable;
+use cursive::ScreenId;
 use cursive::event::Event;
 use cursive::theme::BaseColor;
 use cursive::theme::Color;
@@ -76,9 +81,6 @@ use cursive::views::OnEventView;
 use cursive::views::Panel;
 use cursive::views::ResizedView;
 use cursive::views::ScreensView;
-use cursive::Cursive;
-use cursive::CursiveRunnable;
-use cursive::ScreenId;
 use model::CgroupModel;
 #[cfg(fbcode_build)]
 use model::GpuModel;
@@ -88,6 +90,7 @@ use model::ProcessModel;
 use model::SystemModel;
 use store::Advance;
 use toml::value::Value;
+use viewrc::ViewRc;
 extern crate render as base_render;
 
 open_source_shim!();
@@ -95,8 +98,6 @@ open_source_shim!();
 mod cgroup_tabs;
 pub mod cgroup_view;
 pub mod command_palette;
-mod core_tabs;
-mod core_view;
 mod default_styles;
 mod filter_popup;
 mod help_menu;
@@ -105,6 +106,8 @@ mod process_view;
 mod render;
 pub mod stats_view;
 mod status_bar;
+mod summary_view;
+mod system_tabs;
 mod system_view;
 mod tab_view;
 
@@ -146,7 +149,7 @@ macro_rules! view_warn {
             crate::MainViewState::Cgroup => crate::cgroup_view::ViewType::cp_warn($c, &msg),
             crate::MainViewState::Process(_) =>
                 crate::process_view::ViewType::cp_warn($c, &msg),
-            crate::MainViewState::Core => crate::core_view::ViewType::cp_warn($c, &msg),
+            crate::MainViewState::System => crate::system_view::ViewType::cp_warn($c, &msg),
             #[cfg(fbcode_build)]
             crate::MainViewState::Gpu => crate::gpu_view::ViewType::cp_warn($c, &msg),
         }
@@ -170,32 +173,29 @@ pub enum ProcessZoomState {
 pub enum MainViewState {
     Cgroup,
     Process(ProcessZoomState),
-    Core,
+    System,
     #[cfg(fbcode_build)]
     Gpu,
 }
 
 impl MainViewState {
     pub fn is_process_zoom_state(&self) -> bool {
-        match &self {
-            &MainViewState::Process(zoom) if zoom != &ProcessZoomState::NoZoom => true,
-            _ => false,
-        }
+        matches!(&self, &MainViewState::Process(zoom) if zoom != &ProcessZoomState::NoZoom)
     }
 }
 
 #[derive(Clone)]
 pub enum ViewMode {
-    Live(Rc<RefCell<Advance>>),
-    Pause(Rc<RefCell<Advance>>),
-    Replay(Rc<RefCell<Advance>>),
+    Live(Arc<Mutex<Advance>>),
+    Pause(Arc<Mutex<Advance>>),
+    Replay(Arc<Mutex<Advance>>),
 }
 
 // Invoked either when the data view was explicitly advanced, or
 // periodically (during live mode)
 fn refresh(c: &mut Cursive) {
     status_bar::refresh(c);
-    system_view::refresh(c);
+    summary_view::refresh(c);
     let current_state = c
         .user_data::<ViewState>()
         .expect("No data stored in Cursive object!")
@@ -204,7 +204,7 @@ fn refresh(c: &mut Cursive) {
     match current_state {
         MainViewState::Cgroup => cgroup_view::CgroupView::refresh(c),
         MainViewState::Process(_) => process_view::ProcessView::refresh(c),
-        MainViewState::Core => core_view::CoreView::refresh(c),
+        MainViewState::System => system_view::SystemView::refresh(c),
         #[cfg(fbcode_build)]
         MainViewState::Gpu => gpu_view::GpuView::refresh(c),
     }
@@ -239,17 +239,21 @@ pub struct ViewState {
     /// can certainly go higher (b/c of a loaded system or other delays).
     pub lowest_time_elapsed: Duration,
     pub timestamp: SystemTime,
-    pub system: Rc<RefCell<SystemModel>>,
-    pub cgroup: Rc<RefCell<CgroupModel>>,
-    pub process: Rc<RefCell<ProcessModel>>,
-    pub network: Rc<RefCell<NetworkModel>>,
+    // TODO: Replace other fields with model
+    pub model: Arc<Mutex<Model>>,
+    pub system: Arc<Mutex<SystemModel>>,
+    pub cgroup: Arc<Mutex<CgroupModel>>,
+    pub process: Arc<Mutex<ProcessModel>>,
+    pub network: Arc<Mutex<NetworkModel>>,
     #[cfg(fbcode_build)]
-    pub gpu: Rc<RefCell<Option<GpuModel>>>,
+    pub gpu: Arc<Mutex<Option<GpuModel>>>,
     pub main_view_state: MainViewState,
     pub main_view_screens: HashMap<String, ScreenId>,
     pub mode: ViewMode,
-    pub event_controllers: Rc<RefCell<HashMap<Event, controllers::Controllers>>>,
-    pub cmd_controllers: Rc<RefCell<HashMap<&'static str, controllers::Controllers>>>,
+    pub viewrc: ViewRc,
+    pub viewrc_error: Option<String>,
+    pub event_controllers: Arc<Mutex<HashMap<Event, controllers::Controllers>>>,
+    pub cmd_controllers: Arc<Mutex<HashMap<&'static str, controllers::Controllers>>>,
 }
 
 impl ViewState {
@@ -259,30 +263,42 @@ impl ViewState {
             self.lowest_time_elapsed = model.time_elapsed;
         }
         self.timestamp = model.timestamp;
-        self.system.replace(model.system);
-        self.cgroup.replace(model.cgroup);
-        self.process.replace(model.process);
-        self.network.replace(model.network);
+        *self.model.lock().unwrap() = model.clone();
+        *self.system.lock().unwrap() = model.system;
+        *self.cgroup.lock().unwrap() = model.cgroup;
+        *self.process.lock().unwrap() = model.process;
+        *self.network.lock().unwrap() = model.network;
         #[cfg(fbcode_build)]
-        self.gpu.replace(model.gpu);
+        {
+            *self.gpu.lock().unwrap() = model.gpu;
+        }
     }
 
-    pub fn new_with_advance(main_view_state: MainViewState, model: Model, mode: ViewMode) -> Self {
+    pub fn new_with_advance(
+        main_view_state: MainViewState,
+        model: Model,
+        mode: ViewMode,
+        viewrc: ViewRc,
+        viewrc_error: Option<String>,
+    ) -> Self {
         Self {
             time_elapsed: model.time_elapsed,
             lowest_time_elapsed: model.time_elapsed,
             timestamp: model.timestamp,
-            system: Rc::new(RefCell::new(model.system)),
-            cgroup: Rc::new(RefCell::new(model.cgroup)),
-            process: Rc::new(RefCell::new(model.process)),
-            network: Rc::new(RefCell::new(model.network)),
+            model: Arc::new(Mutex::new(model.clone())),
+            system: Arc::new(Mutex::new(model.system)),
+            cgroup: Arc::new(Mutex::new(model.cgroup)),
+            process: Arc::new(Mutex::new(model.process)),
+            network: Arc::new(Mutex::new(model.network)),
             #[cfg(fbcode_build)]
-            gpu: Rc::new(RefCell::new(model.gpu)),
+            gpu: Arc::new(Mutex::new(model.gpu)),
             main_view_state,
             main_view_screens: HashMap::new(),
             mode,
-            event_controllers: Rc::new(RefCell::new(HashMap::new())),
-            cmd_controllers: Rc::new(RefCell::new(controllers::make_cmd_controller_map())),
+            viewrc,
+            viewrc_error,
+            event_controllers: Arc::new(Mutex::new(HashMap::new())),
+            cmd_controllers: Arc::new(Mutex::new(controllers::make_cmd_controller_map())),
         }
     }
 
@@ -295,27 +311,24 @@ impl ViewState {
     }
 
     pub fn is_paused(&self) -> bool {
-        match self.mode {
-            ViewMode::Pause(_) => true,
-            _ => false,
-        }
+        matches!(self.mode, ViewMode::Pause(_))
     }
 }
 
 impl View {
     pub fn new_with_advance(model: model::Model, mode: ViewMode) -> View {
         let mut inner = cursive::CursiveRunnable::new(|| {
-            let backend = cursive::backends::crossterm::Backend::init().map(|backend| {
-                Box::new(cursive_buffered_backend::BufferedBackend::new(backend))
-                    as Box<(dyn cursive::backend::Backend)>
-            });
+            let backend = cursive::backends::crossterm::Backend::init();
             execute!(std::io::stdout(), DisableMouseCapture).expect("Failed to disable mouse.");
             backend
         });
+        let (viewrc, viewrc_error) = viewrc::ViewRc::new();
         inner.set_user_data(ViewState::new_with_advance(
             MainViewState::Cgroup,
             model,
             mode,
+            viewrc,
+            viewrc_error,
         ));
         View { inner }
     }
@@ -331,7 +344,7 @@ impl View {
     pub fn generate_event_controller_map(c: &mut Cursive, filename: String) {
         // Verify belowrc file format
         let cmdrc_opt = match std::fs::read_to_string(filename) {
-            Ok(belowrc_str) => match belowrc_str.parse::<Value>() {
+            Ok(belowrc_str) => match toml::from_str::<Value>(&belowrc_str) {
                 Ok(belowrc) => belowrc
                     .get(get_belowrc_cmd_section_key())
                     .map(|cmdrc| cmdrc.to_owned()),
@@ -345,19 +358,19 @@ impl View {
 
         let event_controller_map = controllers::make_event_controller_map(c, &cmdrc_opt);
 
-        c.user_data::<ViewState>()
+        *c.user_data::<ViewState>()
             .expect("No data stored in Cursive object!")
             .event_controllers
-            .replace(event_controller_map);
+            .lock()
+            .unwrap() = event_controller_map;
     }
 
     pub fn run(&mut self) -> Result<()> {
-        let (viewrc, viewrc_error) = viewrc::ViewRc::new();
-
         let mut theme = self.inner.current_theme().clone();
         theme.palette[PaletteColor::Background] = Color::TerminalDefault;
         theme.palette[PaletteColor::View] = Color::TerminalDefault;
         theme.palette[PaletteColor::Primary] = Color::TerminalDefault;
+        theme.palette[PaletteColor::TitlePrimary] = Color::TerminalDefault;
         theme.palette[PaletteColor::Highlight] = Color::Dark(BaseColor::Cyan);
         theme.palette[PaletteColor::HighlightText] = Color::Dark(BaseColor::Black);
         theme.shadow = false;
@@ -403,10 +416,10 @@ impl View {
         let init_warnings = get_last_log_to_display();
 
         let status_bar = status_bar::new(&mut self.inner);
-        let system_view = system_view::new(&mut self.inner);
-        let cgroup_view = cgroup_view::CgroupView::new(&mut self.inner, &viewrc);
+        let summary_view = summary_view::new(&mut self.inner);
+        let cgroup_view = cgroup_view::CgroupView::new(&mut self.inner);
         let process_view = process_view::ProcessView::new(&mut self.inner);
-        let core_view = core_view::CoreView::new(&mut self.inner);
+        let system_view = system_view::SystemView::new(&mut self.inner);
         #[cfg(fbcode_build)]
         let gpu_view = gpu_view::GpuView::new(&mut self.inner);
 
@@ -427,8 +440,8 @@ impl View {
             ))),
         );
         main_view_screens.insert(
-            "core_view_panel".to_owned(),
-            screens_view.add_screen(BoxedView::boxed(ResizedView::with_full_screen(core_view))),
+            "system_view_panel".to_owned(),
+            screens_view.add_screen(BoxedView::boxed(ResizedView::with_full_screen(system_view))),
         );
         #[cfg(fbcode_build)]
         main_view_screens.insert(
@@ -440,7 +453,7 @@ impl View {
             .add_fullscreen_layer(ResizedView::with_full_screen(
                 LinearLayout::vertical()
                     .child(Panel::new(status_bar))
-                    .child(Panel::new(system_view))
+                    .child(summary_view)
                     .child(
                         OnEventView::new(screens_view.with_name("main_view_screens"))
                             .with_name("dynamic_view"),
@@ -452,7 +465,14 @@ impl View {
             .expect("Could not set focus at initialization!");
 
         // Set default view from viewrc
-        if let Some(view) = &viewrc.default_view {
+        if let Some(view) = self
+            .inner
+            .user_data::<ViewState>()
+            .expect("No data stored in Cursive object!")
+            .viewrc
+            .default_view
+            .clone()
+        {
             let main_view_state = &mut self
                 .inner
                 .user_data::<ViewState>()
@@ -468,15 +488,21 @@ impl View {
                     set_active_screen(&mut self.inner, "process_view_panel")
                 }
                 viewrc::DefaultFrontView::System => {
-                    *main_view_state = MainViewState::Core;
-                    set_active_screen(&mut self.inner, "core_view_panel")
+                    *main_view_state = MainViewState::System;
+                    set_active_screen(&mut self.inner, "system_view_panel")
                 }
             }
         }
 
         // Raise warning message if failed to map the customized command.
         Self::generate_event_controller_map(&mut self.inner, get_belowrc_filename());
-        if let Some(msg) = viewrc_error {
+        if let Some(msg) = &self
+            .inner
+            .user_data::<ViewState>()
+            .expect("No data stored in Cursive object!")
+            .viewrc_error
+        {
+            let msg = msg.clone();
             let c = &mut self.inner;
             view_warn!(c, "{}", msg);
         }
@@ -492,9 +518,9 @@ impl View {
 
 #[cfg(test)]
 pub mod fake_view {
-    use std::cell::RefCell;
     use std::path::PathBuf;
-    use std::rc::Rc;
+    use std::sync::Arc;
+    use std::sync::Mutex;
 
     use common::logutil::get_logger;
     use cursive::views::DummyView;
@@ -502,18 +528,20 @@ pub mod fake_view {
     use model::Collector;
     use store::advance::new_advance_local;
 
+    use self::viewrc::ViewRc;
     use super::*;
-    use crate::cgroup_view::CgroupView;
-    use crate::command_palette::CommandPalette;
-    use crate::stats_view::StatsView;
     use crate::MainViewState;
     use crate::ViewMode;
     use crate::ViewState;
+    use crate::cgroup_view::CgroupView;
+    use crate::command_palette::CommandPalette;
+    use crate::stats_view::StatsView;
 
     pub struct FakeView {
         pub inner: CursiveRunnable,
     }
 
+    #[allow(clippy::new_without_default)]
     impl FakeView {
         pub fn new() -> Self {
             let time = SystemTime::now();
@@ -528,7 +556,9 @@ pub mod fake_view {
             let mut user_data = ViewState::new_with_advance(
                 MainViewState::Cgroup,
                 model,
-                ViewMode::Live(Rc::new(RefCell::new(advance))),
+                ViewMode::Live(Arc::new(Mutex::new(advance))),
+                ViewRc::default(),
+                None,
             );
             // Dummy screen to make switching panel no-op except state changes.
             inner.add_layer(
@@ -538,7 +568,7 @@ pub mod fake_view {
             user_data.main_view_screens = [
                 ("cgroup_view_panel".to_owned(), 0),
                 ("process_view_panel".to_owned(), 0),
-                ("core_view_panel".to_owned(), 0),
+                ("system_view_panel".to_owned(), 0),
             ]
             .into();
             inner.set_user_data(user_data);
@@ -547,7 +577,7 @@ pub mod fake_view {
         }
 
         pub fn add_cgroup_view(&mut self) {
-            let cgroup_view = CgroupView::new(&mut self.inner, &Default::default());
+            let cgroup_view = CgroupView::new(&mut self.inner);
             self.inner.add_layer(cgroup_view);
         }
 
